@@ -1,80 +1,5 @@
-/* libmondo-files.c                                  file manipulation
-   $Id: libmondo-files.c 1505 2007-06-15 09:14:21Z bruno $
-.
-
-
-04/16/04
-- find_home_of_exe() really does return NULL now if file not found
-
-03/22/04
-- added mode_of_file()
-
-10/02/03
-- cleaned up grab_percentage_from_last_line_of_file()
-
-09/18
-- added int make_grub_install_scriptlet()
-
-09/16
-- cleaned up mkisofs feedback window
-
-09/12
-- fixed Teuton-hostile bug in size_of_all_biggiefiles_K()
-
-09/05
-- added size_of_partition_in_mountlist_K()
-- size_of_all_biggiefiles_K() now calls get_phys_size_of_drive(fname)
-
-07/02
-- fixed calls to popup_and_get_string()
-
-05/19
-- added CP_BIN
-
-05/05
-- added Joshua Oreman's FreeBSD patches
-
-05/04
-- find_home_of_exe() now returns NULL if file not found
-
-04/26
-- if >4 media est'd, say one meeeellion
-
-04/25
-- fixed minor bug in find_home_of_exe()
-
-04/24
-- added lots of assert()'s and log_OS_error()'s
-
-04/07
-- fix find_home_of_exe()
-- cleaned up code a bit
-
-03/27
-- copy_mondo_and_mindi_stuff --- if _homedir_/payload.tgz exists then untar it to CD
-
-01/14/2003
-- if backup media type == nfs then don't estimate no. of media reqd
-
-11/25/2002
-- don't log/echo estimated # of media required if >=50
-
-10/01 - 11/09
-- chmod uses 0x, not decimal :)
-- added is_this_file_compressed()
-- replace convoluted grep with wc (KP)
-
-09/01 - 09/30
-- only show "number of media" estimate if no -x
-- run_program_and_log_output() now takes boolean operator to specify
-  whether it will log its activities in the event of _success_
-
-08/01 - 08/31
-- handle unknown media sizes
-- cleaned up some log_it() calls
-
-07/24
-- created
+/* file manipulation
+   $Id: libmondo-files.c 3150 2013-06-21 04:48:23Z bruno $
 */
 
 /**
@@ -84,6 +9,7 @@
 
 
 #include "my-stuff.h"
+#include "mr_mem.h"
 #include "mondostructures.h"
 #include "libmondo-files.h"
 
@@ -96,13 +22,16 @@
 #include "libmondo-string-EXT.h"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-files.c 1505 2007-06-15 09:14:21Z bruno $";
+//static char cvsid[] = "$Id: libmondo-files.c 3150 2013-06-21 04:48:23Z bruno $";
 
 extern char err_log_lines[NOOF_ERR_LINES][MAX_STR_LEN];
 
 extern int g_currentY;
 extern char *g_mondo_home;
 
+/* Reference to global bkpinfo */
+extern struct s_bkpinfo *bkpinfo;
+
 /**
  * @addtogroup fileGroup
  * @{
@@ -122,6 +51,7 @@ char *calc_checksum_of_file(char *filename)
 
 	/*@ pointers **************************************************** */
 	char *p;
+	char *q;
 	FILE *fin;
 
 	/*@ initialize pointers ***************************************** */
@@ -135,8 +65,11 @@ char *calc_checksum_of_file(char *filename)
 		sprintf(command, "md5sum \"%s\"", filename);
 		fin = popen(command, "r");
 		if (fin) {
-			(void) fgets(output, MAX_STR_LEN, fin);
-			p = strchr(output, ' ');
+			q = fgets(output, MAX_STR_LEN, fin);
+			if (!q) {
+				// FIXME
+			}
+			q = strchr(output, ' ');
 			paranoid_pclose(fin);
 		}
 	} else {
@@ -199,6 +132,7 @@ long count_lines_in_file(char *filename)
 	char command[MAX_STR_LEN * 2];
 	char incoming[MAX_STR_LEN];
 	char tmp[MAX_STR_LEN];
+	char *q;
 
 	/*@ long ******************************************************** */
 	long noof_lines = -1L;
@@ -226,8 +160,11 @@ long count_lines_in_file(char *filename)
 		if (feof(fin)) {
 			noof_lines = 0;
 		} else {
-			(void) fgets(incoming, MAX_STR_LEN - 1, fin);
-			while (strlen(incoming) > 0
+			q = fgets(incoming, MAX_STR_LEN - 1, fin);
+			if (!q) {
+				// FIXME
+			}
+			while (strlen(incoming) > 0 && (q != NULL)
 				   && incoming[strlen(incoming) - 1] < 32) {
 				incoming[strlen(incoming) - 1] = '\0';
 			}
@@ -281,6 +218,7 @@ void exclude_nonexistent_files(char *inout)
 	char outfname[MAX_STR_LEN];
 	char tmp[MAX_STR_LEN];
 	char incoming[MAX_STR_LEN];
+	char *q;
 
 	/*@ int ********************************************************* */
 	int i;
@@ -304,8 +242,8 @@ void exclude_nonexistent_files(char *inout)
 		log_OS_error("Unable to openout outfname");
 		return;
 	}
-	for (fgets(incoming, MAX_STR_LEN, fin); !feof(fin);
-		 fgets(incoming, MAX_STR_LEN, fin)) {
+	for (q = fgets(incoming, MAX_STR_LEN, fin); !feof(fin) && (q != NULL);
+		 q = fgets(incoming, MAX_STR_LEN, fin)) {
 		i = strlen(incoming) - 1;
 		if (i >= 0 && incoming[i] < 32) {
 			incoming[i] = '\0';
@@ -567,6 +505,7 @@ char *last_line_of_file(char *filename)
 	static char output[MAX_STR_LEN];
 	static char command[MAX_STR_LEN * 2];
 	static char tmp[MAX_STR_LEN];
+	char *p = NULL;
 
 	/*@ pointers **************************************************** */
 	FILE *fin;
@@ -582,7 +521,10 @@ char *last_line_of_file(char *filename)
 	}
 	sprintf(command, "tail -n1 %s", filename);
 	fin = popen(command, "r");
-	(void) fgets(output, MAX_STR_LEN, fin);
+	p = fgets(output, MAX_STR_LEN, fin);
+	if (p == NULL) {
+		// FIXME
+	}
 	paranoid_pclose(fin);
 	while (strlen(output) > 0 && output[strlen(output) - 1] < 32) {
 		output[strlen(output) - 1] = '\0';
@@ -638,6 +580,7 @@ make_checksum_list_file(char *filelist, char *cksumlist, char *comppath)
 	char curr_fname[1000];
 	char curr_cksum[1000];
 	char tmp[1000];
+	char *q;
 
 	/*@ long [long] ************************************************* */
 	off_t filelist_length;
@@ -666,8 +609,8 @@ make_checksum_list_file(char *filelist, char *cksumlist, char *comppath)
 		log_to_screen("Can't open checksum list");
 		return (1);
 	}
-	for (fgets(stub_fname, 999, fin); !feof(fin);
-		 fgets(stub_fname, 999, fin)) {
+	for (q = fgets(stub_fname, 999, fin); !feof(fin) && (q != NULL);
+		 q = fgets(stub_fname, 999, fin)) {
 		if (stub_fname[(i = strlen(stub_fname) - 1)] < 32) {
 			stub_fname[i] = '\0';
 		}
@@ -770,6 +713,7 @@ long noof_lines_that_match_wildcard(char *filelist_fname, char *wildcard)
 
 	/*@ buffers **************************************************** */
 	char incoming[MAX_STR_LEN];
+	char *p = NULL;
 
 	/*@ end vars *************************************************** */
 
@@ -780,12 +724,12 @@ long noof_lines_that_match_wildcard(char *filelist_fname, char *wildcard)
 		log_OS_error("Unable to openin filelist_fname");
 		return (0);
 	}
-	(void) fgets(incoming, MAX_STR_LEN - 1, fin);
-	while (!feof(fin)) {
+	p = fgets(incoming, MAX_STR_LEN - 1, fin);
+	while (!feof(fin) && (p != NULL)) {
 		if (strstr(incoming, wildcard)) {
 			matches++;
 		}
-		(void) fgets(incoming, MAX_STR_LEN - 1, fin);
+		p = fgets(incoming, MAX_STR_LEN - 1, fin);
 	}
 	paranoid_fclose(fin);
 	return (matches);
@@ -793,54 +737,6 @@ long noof_lines_that_match_wildcard(char *filelist_fname, char *wildcard)
 
 
 
-
-/**
- * Register our PID in a file in /var/run.
- * The PID will be put in /var/run/monitas-<tt>name_str</tt>.pid.
- * @param pid 0 to remove file, anything else to create it.
- * @param name_str The basename of the PID file (e.g. "mondo" or "server")
- * @note This function does not provide support against multiple instances, unless you check for that yourself.
- */
-void register_pid(pid_t pid, char *name_str)
-{
-	char tmp[MAX_STR_LEN + 1], lockfile_fname[MAX_STR_LEN + 1];
-	int res;
-	FILE *fin;
-
-	sprintf(lockfile_fname, "/var/run/monitas-%s.pid", name_str);
-	if (!pid) {
-		log_it("Unregistering PID");
-		if (unlink(lockfile_fname)) {
-			log_it("Error unregistering PID");
-		}
-		return;
-	}
-	if (does_file_exist(lockfile_fname)) {
-		tmp[0] = '\0';
-		if ((fin = fopen(lockfile_fname, "r"))) {
-			(void) fgets(tmp, MAX_STR_LEN, fin);
-			paranoid_fclose(fin);
-		} else {
-			log_OS_error("Unable to openin lockfile_fname");
-		}
-		pid = (pid_t) atol(tmp);
-		sprintf(tmp, "ps %ld > /dev/null 2> /dev/null", (long int) pid);
-		res = system(tmp);
-		if (!res) {
-			log_it
-				("I believe the daemon is already running. If it isn't, please delete %s and try again.",
-				 lockfile_fname);
-		}
-	}
-	sprintf(tmp, "echo %ld > %s 2> /dev/null", (long int) getpid(),
-			lockfile_fname);
-	if (system(tmp)) {
-		fatal_error("Cannot register PID");
-	}
-}
-
-
-
 /**
  * Determine the size (in KB) of @p dev in the mountlist in <tt>tmpdir</tt>/mountlist.txt.
  * @param tmpdir The tempdir where the mountlist is stored.
@@ -870,13 +766,14 @@ long size_of_partition_in_mountlist_K(char *tmpdir, char *dev)
  * @param bkpinfo The backup information structure. Only the @c bkpinfo->tmpdir field is used.
  * @return The total size of all biggiefiles in KB.
  */
-long size_of_all_biggiefiles_K(struct s_bkpinfo *bkpinfo)
+long size_of_all_biggiefiles_K()
 {
 	/*@ buffers ***************************************************** */
 	char *fname;
 	char *biggielist;
 	char *comment;
 	char *tmp;
+	char *q;
 	char *command;
 
 	/*@ long ******************************************************** */
@@ -901,8 +798,8 @@ long size_of_all_biggiefiles_K(struct s_bkpinfo *bkpinfo)
 			("Cannot open biggielist. OK, so estimate is based on filesets only.");
 	} else {
 		log_msg(4, "Reading it...");
-		for (fgets(fname, MAX_STR_LEN, fin); !feof(fin);
-			 fgets(fname, MAX_STR_LEN, fin)) {
+		for (q = fgets(fname, MAX_STR_LEN, fin); !feof(fin) && (q != NULL);
+			 q = fgets(fname, MAX_STR_LEN, fin)) {
 			if (fname[strlen(fname) - 1] <= 32) {
 				fname[strlen(fname) - 1] = '\0';
 			}
@@ -972,7 +869,7 @@ long long space_occupied_by_cd(char *mountpt)
 	  log_it("popen() FAILED: command=%s, mountpt=%s, fin=%d, errno=%d, strerror=%s", command, mountpt, fin, errno, strerror(errno));
 	  llres = 0;
 	} else {
-	  (void) fgets(tmp, MAX_STR_LEN, fin);
+	  p = fgets(tmp, MAX_STR_LEN, fin);
 	  paranoid_pclose(fin);
 	  p = strchr(tmp, '\t');
 	  if (p) {
@@ -1041,7 +938,7 @@ int whine_if_not_found(char *fname)
 	if (system(command)) {
 		log_to_screen(errorstr);
 		log_to_screen
-			("There may be hyperlink at http://www.mondorescue.com which");
+			("There may be hyperlink at http://www.mondorescue.org/downloads.shtml which");
 		log_to_screen("will take you to the relevant (missing) package.");
 		return (1);
 	} else {
@@ -1109,12 +1006,13 @@ int read_one_liner_data_file(char *fname, char *contents)
 		log_OS_error("Unable to openin fname");
 		return (1);
 	}
-	fscanf(fin, "%s\n", contents);
+	res = fscanf(fin, "%s\n", contents);
 	i = strlen(contents);
 	if (i > 0 && contents[i - 1] < 32) {
 		contents[i - 1] = '\0';
 	}
 	paranoid_fclose(fin);
+	res = 0;
 	return (res);
 }
 
@@ -1136,7 +1034,7 @@ int read_one_liner_data_file(char *fname, char *contents)
  * - @c bkpinfo->scratchdir
  * - @c bkpinfo->tmpdir
  */
-void copy_mondo_and_mindi_stuff_to_scratchdir(struct s_bkpinfo *bkpinfo)
+void copy_mondo_and_mindi_stuff_to_scratchdir()
 {
 	/*@ Char buffers ** */
 	char command[MAX_STR_LEN * 2];
@@ -1162,13 +1060,19 @@ void copy_mondo_and_mindi_stuff_to_scratchdir(struct s_bkpinfo *bkpinfo)
 	if (does_file_exist(tmp)) {
 		log_it("Untarring payload %s to scratchdir %s", tmp,
 			   bkpinfo->scratchdir);
-		(void) getcwd(old_pwd, MAX_STR_LEN - 1);
-		chdir(bkpinfo->scratchdir);
+		if (getcwd(old_pwd, MAX_STR_LEN - 1)) {
+			// FIXME
+		}
+		if (chdir(bkpinfo->scratchdir)) {
+			// FIXME
+		}
 		sprintf(command, "tar -zxvf %s", tmp);
 		if (run_program_and_log_output(command, FALSE)) {
 			fatal_error("Failed to untar payload");
 		}
-		chdir(old_pwd);
+		if (chdir(old_pwd)) {
+			// FIXME
+		}
 	}
 
 	sprintf(command, "cp -f %s/LAST-FILELIST-NUMBER %s", bkpinfo->tmpdir,
@@ -1209,110 +1113,118 @@ void copy_mondo_and_mindi_stuff_to_scratchdir(struct s_bkpinfo *bkpinfo)
 
 
 /**
- * Store the client's NFS configuration in files to be restored at restore-time.
- * Assumes that @c bkpinfo->media_type = nfs, but does not check for this.
+ * Store the client's NETFS configuration in files to be restored at restore-time.
+ * Assumes that @c bkpinfo->media_type = netfs, but does not check for this.
  * @param bkpinfo The backup information structure. Fields used:
- * - @c nfs_mount
- * - @c nfs_remote_dir
+ * - @c netfs_mount
+ * - @c netfs_remote_dir
  * - @c tmpdir
  */
-void store_nfs_config(struct s_bkpinfo *bkpinfo)
+void store_netfs_config()
 {
 
 	/*@ buffers ******** */
-	char nfs_dev[MAX_STR_LEN];
-	char mac_addr[MAX_STR_LEN];
-	char nfs_mount[MAX_STR_LEN];
-	char nfs_client_ipaddr[MAX_STR_LEN];
-	char nfs_client_netmask[MAX_STR_LEN];
-	char nfs_client_broadcast[MAX_STR_LEN];
-	char nfs_client_defgw[MAX_STR_LEN];
-	char nfs_server_ipaddr[MAX_STR_LEN];
+	char netfs_dev[MAX_STR_LEN];
+	char netfs_mount[MAX_STR_LEN];
+	char netfs_client_hwaddr[MAX_STR_LEN];
+	char netfs_client_ipaddr[MAX_STR_LEN];
+	char netfs_client_netmask[MAX_STR_LEN];
+	char netfs_client_broadcast[MAX_STR_LEN];
+	char netfs_client_defgw[MAX_STR_LEN];
+	char netfs_server_ipaddr[MAX_STR_LEN];
 	char tmp[MAX_STR_LEN];
 	char command[MAX_STR_LEN * 2];
 
 	/*@ pointers ***** */
 	char *p;
 
-	log_it("Storing NFS configuration");
-	strcpy(tmp, bkpinfo->nfs_mount);
+	log_it("Storing Network configuration");
+	strcpy(tmp, bkpinfo->netfs_mount);
 	p = strchr(tmp, ':');
 	if (!p) {
 		fatal_error
-			("NFS mount doesn't have a colon in it, e.g. 192.168.1.4:/home/nfs");
+			("Network mount doesn't have a colon in it, e.g. 192.168.1.4:/home/nfs");
 	}
 	*(p++) = '\0';
-	strcpy(nfs_server_ipaddr, tmp);
-	strcpy(nfs_mount, p);
+	strcpy(netfs_server_ipaddr, tmp);
+	strcpy(netfs_mount, p);
 
 	/* BERLIOS : there is a bug #67 here as it only considers the first NIC */
 	sprintf(command,
 			"ifconfig | tr '\n' '#' | sed s/##// | tr '#' ' ' | tr '' '\n' | head -n1 | cut -d' ' -f1");
-	strcpy(nfs_dev, call_program_and_get_last_line_of_output(command));
+	strcpy(netfs_dev, call_program_and_get_last_line_of_output(command));
+	sprintf(command,
+			"ifconfig %s | head -1 | awk '{print $5}'", netfs_dev);
+	strcpy(netfs_client_hwaddr, call_program_and_get_last_line_of_output(command));
 	sprintf(command,
 			"ifconfig | tr '\n' '#' | sed s/##// | tr '#' ' ' | tr '' '\\n' | head -n1 | tr -s '\t' ' ' | cut -d' ' -f7 | cut -d':' -f2");
-	strcpy(nfs_client_ipaddr,
+	strcpy(netfs_client_ipaddr,
 		   call_program_and_get_last_line_of_output(command));
 	sprintf(command,
 			"ifconfig | tr '\n' '#' | sed s/##// | tr '#' ' ' | tr '' '\\n' | head -n1 | tr -s '\t' ' ' | cut -d' ' -f9 | cut -d':' -f2");
-	strcpy(nfs_client_netmask,
+	strcpy(netfs_client_netmask,
 		   call_program_and_get_last_line_of_output(command));
 	sprintf(command,
 			"ifconfig | tr '\n' '#' | sed s/##// | tr '#' ' ' | tr '' '\\n' | head -n1 | tr -s '\t' ' ' | cut -d' ' -f8 | cut -d':' -f2");
-	strcpy(nfs_client_broadcast,
+	strcpy(netfs_client_broadcast,
 		   call_program_and_get_last_line_of_output(command));
 	sprintf(command,
 			"route -n | grep '^0.0.0.0' | awk '{print $2}'");
-	strcpy(nfs_client_defgw,
+	strcpy(netfs_client_defgw,
 		   call_program_and_get_last_line_of_output(command));
 	sprintf(tmp,
-			"nfs_client_ipaddr=%s; nfs_server_ipaddr=%s; nfs_mount=%s",
-			nfs_client_ipaddr, nfs_server_ipaddr, nfs_mount);
-	if (strlen(nfs_dev) < 2) {
+			"netfs_client_hwaddr=%s; netfs_client_ipaddr=%s; netfs_server_ipaddr=%s; netfs_mount=%s",
+			netfs_client_hwaddr, netfs_client_ipaddr, netfs_server_ipaddr, netfs_mount);
+	if (strlen(netfs_dev) < 2) {
 		fatal_error
-			("Unable to find ethN (eth0, eth1, ...) adapter via NFS mount you specified.");
+			("Unable to find ethN (eth0, eth1, ...) adapter via Network mount you specified.");
 	}
-	make_hole_for_dir("/var/cache/mondo-archive");
 	/********
-	* If the NFS device that found above is a bonded device, 
+	* If the Network device that found above is a bonded device, 
 	* we need to replace it with an ethN device or the
-	* networking will not start during an NFS restore.
+	* networking will not start during an Network restore.
 	*
-	* If the NFS device in nfs_dev begins with the word "bond", 
-	* look for the corresponding slave ethN device and copy it to nfs_dev.
+	* If the Network device in netfs_dev begins with the word "bond", or alb or aft
+	* look for the corresponding slave ethN device and copy it to netfs_dev.
 	* Using the common MAC address
 	********/
-	if (!strncmp(nfs_dev, "bond", 4)) {
-		log_to_screen("Found bonding device %s; looking for corresponding ethN slave device\n", nfs_dev);
+	if (!strncmp(netfs_dev, "bond", 4) || !strncmp(netfs_dev, "alb", 3) || !strncmp(netfs_dev, "aft", 3)) {
+		log_to_screen("Found bonding device %s; looking for corresponding ethN slave device\n", netfs_dev);
 		sprintf(command,
-				"ifconfig %s | awk '{print $5}' | head -n1", nfs_dev);
-		strcpy(mac_addr, call_program_and_get_last_line_of_output(command));
-		sprintf(command,
-				"ifconfig | grep -E '%s' | grep -v '%s' | head -n1 | cut -d' ' -f1", mac_addr,nfs_dev);
-		strcpy(nfs_dev, call_program_and_get_last_line_of_output(command));
-		log_to_screen("Replacing it with %s\n", nfs_dev);
+				"ifconfig | grep -E '%s' | grep -v '%s' | head -n1 | cut -d' ' -f1",netfs_client_hwaddr,netfs_dev);
+		strcpy(netfs_dev, call_program_and_get_last_line_of_output(command));
+		log_to_screen("Replacing it with %s\n", netfs_dev);
 	}
 
-	sprintf(tmp, "%s/NFS-DEV", bkpinfo->tmpdir);
-	write_one_liner_data_file(tmp, nfs_dev);
+	sprintf(tmp, "%s/NETFS-DEV", bkpinfo->tmpdir);
+	write_one_liner_data_file(tmp, netfs_dev);
 
-	sprintf(tmp, "%s/NFS-CLIENT-IPADDR", bkpinfo->tmpdir);
-	write_one_liner_data_file(tmp, nfs_client_ipaddr);
-	sprintf(tmp, "%s/NFS-CLIENT-NETMASK", bkpinfo->tmpdir);
-	write_one_liner_data_file(tmp, nfs_client_netmask);
-	sprintf(tmp, "%s/NFS-CLIENT-BROADCAST", bkpinfo->tmpdir);
-	write_one_liner_data_file(tmp, nfs_client_broadcast);
-	sprintf(tmp, "%s/NFS-CLIENT-DEFGW", bkpinfo->tmpdir);
-	write_one_liner_data_file(tmp, nfs_client_defgw);
-	sprintf(tmp, "%s/NFS-SERVER-IPADDR", bkpinfo->tmpdir);
-	write_one_liner_data_file(tmp, nfs_server_ipaddr);
-	sprintf(tmp, "%s/NFS-SERVER-MOUNT", bkpinfo->tmpdir);
-	write_one_liner_data_file(tmp, bkpinfo->nfs_mount);
-	sprintf(tmp, "%s/NFS-SERVER-PATH", bkpinfo->tmpdir);
-	write_one_liner_data_file(tmp, bkpinfo->nfs_remote_dir);
+	sprintf(tmp, "%s/NETFS-PROTO", bkpinfo->tmpdir);
+	write_one_liner_data_file(tmp, bkpinfo->netfs_proto);
+
+	sprintf(tmp, "%s/NETFS-CLIENT-HWADDR", bkpinfo->tmpdir);
+	write_one_liner_data_file(tmp, netfs_client_hwaddr);
+	sprintf(tmp, "%s/NETFS-CLIENT-IPADDR", bkpinfo->tmpdir);
+	write_one_liner_data_file(tmp, netfs_client_ipaddr);
+	sprintf(tmp, "%s/NETFS-CLIENT-NETMASK", bkpinfo->tmpdir);
+	write_one_liner_data_file(tmp, netfs_client_netmask);
+	sprintf(tmp, "%s/NETFS-CLIENT-BROADCAST", bkpinfo->tmpdir);
+	write_one_liner_data_file(tmp, netfs_client_broadcast);
+	sprintf(tmp, "%s/NETFS-CLIENT-DEFGW", bkpinfo->tmpdir);
+	write_one_liner_data_file(tmp, netfs_client_defgw);
+	sprintf(tmp, "%s/NETFS-SERVER-IPADDR", bkpinfo->tmpdir);
+	write_one_liner_data_file(tmp, netfs_server_ipaddr);
+	sprintf(tmp, "%s/NETFS-SERVER-MOUNT", bkpinfo->tmpdir);
+	write_one_liner_data_file(tmp, bkpinfo->netfs_mount);
+	if (bkpinfo->netfs_user) {
+		sprintf(tmp, "%s/NETFS-SERVER-USER", bkpinfo->tmpdir);
+		write_one_liner_data_file(tmp, bkpinfo->netfs_user);
+	}
+	sprintf(tmp, "%s/NETFS-SERVER-PATH", bkpinfo->tmpdir);
+	write_one_liner_data_file(tmp, bkpinfo->netfs_remote_dir);
 	sprintf(tmp, "%s/ISO-PREFIX", bkpinfo->tmpdir);
 	write_one_liner_data_file(tmp, bkpinfo->prefix);
-	log_it("Finished storing NFS configuration");
+	log_it("Finished storing Network configuration");
 }
 
 
@@ -1337,15 +1249,16 @@ void store_nfs_config(struct s_bkpinfo *bkpinfo)
  * @ingroup archiveGroup
  */
 void
-estimate_noof_media_required(struct s_bkpinfo *bkpinfo, long noof_sets)
+estimate_noof_media_required(long noof_sets)
 {
 	/*@ buffers *************** */
 	char tmp[MAX_STR_LEN];
+	char *mds = NULL;
 
 	/*@ long long ************* */
 	long long scratchLL;
 
-	if (bkpinfo->media_size[1] <= 0) {
+	if (bkpinfo->media_size <= 0) {
 		log_to_screen("Number of media required: UNKNOWN");
 		return;
 	}
@@ -1353,8 +1266,8 @@ estimate_noof_media_required(struct s_bkpinfo *bkpinfo, long noof_sets)
 	log_it("Estimating number of media required...");
 	scratchLL =
 		(long long) (noof_sets) * (long long) (bkpinfo->optimal_set_size)
-		+ (long long) (size_of_all_biggiefiles_K(bkpinfo));
-	scratchLL = (scratchLL / 1024) / bkpinfo->media_size[1];
+		+ (long long) (size_of_all_biggiefiles_K());
+	scratchLL = (scratchLL / 1024) / bkpinfo->media_size;
 	scratchLL++;
 	if (bkpinfo->use_lzo) {
 		scratchLL = (scratchLL * 2) / 3;
@@ -1367,9 +1280,10 @@ estimate_noof_media_required(struct s_bkpinfo *bkpinfo, long noof_sets)
 		scratchLL++;
 	}
 	if (scratchLL <= 1) {
+		mds = media_descriptor_string(bkpinfo->backup_media_type);
 		sprintf(tmp,
-				"Your backup will probably occupy a single %s. Maybe two.",
-				media_descriptor_string(bkpinfo->backup_media_type));
+				"Your backup will probably occupy a single %s. Maybe two.", mds);
+		mr_free(mds);
 	} else if (scratchLL > 4) {
 		sprintf(tmp,
 				"Your backup will occupy one meeeeellion media! (maybe %s)",
@@ -1446,48 +1360,4 @@ bool is_this_file_compressed(char *filename)
 	return (FALSE);
 }
 
-
-
-int mode_of_file(char *fname)
-{
-	struct stat buf;
-
-	if (lstat(fname, &buf)) {
-		return (-1);
-	}							// error
-	else {
-		return (buf.st_mode);
-	}
-}
-
-
-
-
-/**
- * Create a small script that mounts /boot, calls @c grub-install, and syncs the disks.
- * @param outfile Where to put the script.
- * @return 0 for success, 1 for failure.
- */
-int make_grub_install_scriptlet(char *outfile)
-{
-	FILE *fout;
-	char *tmp;
-	int retval = 0;
-
-	malloc_string(tmp);
-	if ((fout = fopen(outfile, "w"))) {
-		fprintf(fout,
-				"#!/bin/sh\n\nmount /boot > /dev/null 2> /dev/null\ngrub-install $@\nres=$?\nsync;sync;sync\nexit $res\n");
-		paranoid_fclose(fout);
-		log_msg(2, "Created %s", outfile);
-		sprintf(tmp, "chmod +x %s", outfile);
-		paranoid_system(tmp);
-		retval = 0;
-	} else {
-		retval = 1;
-	}
-	paranoid_free(tmp);
-	return (retval);
-}
-
 /* @} - end fileGroup */
