@@ -1,5 +1,5 @@
 /* libmondo-files.h
- * $Id: libmondo-files.h 684 2006-06-26 11:12:48Z bcornec $
+ * $Id: libmondo-files.h 2904 2011-11-21 02:46:46Z bruno $
  */
 
 #include "crcttt.h"
@@ -8,8 +8,6 @@
 unsigned int updcrc(unsigned int crc, unsigned int c);
 unsigned int updcrcr(unsigned int crc, unsigned int c);
 char *calc_checksum_of_file(char *filename);
-char *calc_file_ugly_minichecksum(char *curr_fname);
-char *calc_file_ugly_minichecksum(char *curr_fname);
 long count_lines_in_file(char *filename);
 bool does_file_exist(char *filename);
 void exclude_nonexistent_files(char *inout);
@@ -26,15 +24,14 @@ void make_list_of_files_to_ignore(char *ignorefiles_fname,
 								  char *filelist_fname,
 								  char *cklist_fname);
 long noof_lines_that_match_wildcard(char *filelist_fname, char *wildcard);
-void register_pid(pid_t pid, char *name_str);
-long size_of_all_biggiefiles_K(struct s_bkpinfo *bkpinfo);
+long size_of_all_biggiefiles_K();
 long long space_occupied_by_cd(char *mountpt);
 int whine_if_not_found(char *fname);
 int write_one_liner_data_file(char *fname, char *contents);
 
-void copy_mondo_and_mindi_stuff_to_scratchdir(struct s_bkpinfo *bkpinfo);
-void store_nfs_config(struct s_bkpinfo *bkpinfo);
-void estimate_noof_media_required(struct s_bkpinfo *bkpinfo, long);
+void copy_mondo_and_mindi_stuff_to_scratchdir();
+void store_netfs_config();
+void estimate_noof_media_required(long);
 bool is_this_file_compressed(char *);
 
 
@@ -46,7 +43,4 @@ bool is_this_file_compressed(char *);
 
 int make_hole_for_dir(char *outdir_fname);
 long size_of_partition_in_mountlist_K(char *tmpdir, char *dev);
-int make_grub_install_scriptlet(char *outfile);
-
 int read_one_liner_data_file(char *fname, char *contents);
-int mode_of_file(char *fname);
