@@ -1,46 +1,5 @@
-/* libmondo-mountlist.c                            subroutines for handling mountlist
-   $Id: libmondo-mountlist.c 1439 2007-05-15 23:30:46Z bruno $
-
-
-
-08/01
-- when evaluating mountlist, skip drive entirely if it does not exist
-
-07/14
-- always exclude /devpts, /proc, /sys from mountlist
-   
-06/29/2004
-- changed some char[] to *char
-- drivelist is struct now, not char[][]
-
-10/19/2003
-- format_device() --- contract /dev/md/N to /dev/mdN to
-  overcome devfs problems
-
-06/11
-- added support for 5th column in mountlist.txt, for labels
-
-05/08
-- added save_mountlist_to_disk() and load_mountlist()
-  and misc other funcs from mondorestore.c
-
-05/05
-- added Joshua Oreman's FreeBSD patches
-
-04/24
-- added some assert()'s
-
-04/23
-- cleaned up evaluate_drive_within_mountlist() a bit
-
-01/15/2003
-- added code for LVM (Brian Borgeson)
-
-10/19/2002
-- added some comments
-
-07/24/2002
-- created
+/* subroutines for handling mountlist
+   $Id: libmondo-mountlist.c 3060 2012-11-10 04:05:37Z bruno $
 */
 
 
@@ -50,6 +9,7 @@
  */
 
 #include "my-stuff.h"
+#include "mr_mem.h"
 #include "mondostructures.h"
 #include "libmondo-mountlist.h"
 #include "lib-common-externs.h"
@@ -60,13 +20,10 @@
 #include "newt-specific-EXT.h"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-mountlist.c 1439 2007-05-15 23:30:46Z bruno $";
+//static char cvsid[] = "$Id: libmondo-mountlist.c 3060 2012-11-10 04:05:37Z bruno $";
 
-/**
- * A global copy of @c bkpinfo, to aid in debugging. As the name implies, <em>don't use this</em>.
- * @ingroup globalGroup
- */
-struct s_bkpinfo *g_bkpinfo_DONTUSETHIS = NULL;
+/* Reference to global bkpinfo */
+extern struct s_bkpinfo *bkpinfo;
 
 /**
  * @addtogroup mountlistGroup
@@ -80,12 +37,10 @@ struct s_bkpinfo *g_bkpinfo_DONTUSETHIS = NULL;
  * errors as well as too many BSD partitions (more than 'h').
  * @param mountlist The mountlist to check.
  * @param drive The drive to check (e.g. @c /dev/hda).
- * @param flaws_str Where to put the found flaws (human-readable).
- * @return The number of flaws found (0 for success).
+ * @return The flaws string found (NULL for success) allocated to be freed
  * @see evaluate_mountlist
  */
-int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
-									char *drive, char *flaws_str)
+char *evaluate_drive_within_mountlist(struct mountlist_itself *mountlist, char *drive)
 #ifdef __FreeBSD__
 {
 // FreeBSD-specific version of evaluate_drive_within_mountlist()
@@ -94,7 +49,6 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 	int curr_part_no = 0;
 	int pos = 0, npos = 0;
 	int res = 0;
-	int mountpoint_copies = 0;
 	int device_copies = 0;
 	int i = 0;
 	int cur_sp_no = 0;
@@ -102,10 +56,12 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 	int foundsome = FALSE;
 
 	/*@ buffers ******************************************************** */
-	char tmp[MAX_STR_LEN];
+	char tmp = NULL;
 	char device[MAX_STR_LEN];
 	char mountpoint[MAX_STR_LEN];
 
+	char *flaws_str = NULL;
+
 	/*@ long *********************************************************** */
 	long physical_drive_size = 0;
 	long amount_allocated = 0;
@@ -114,20 +70,20 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 	char *part_table_fmt;
 
 	/*@ initialize ***************************************************** */
-	flaws_str[0] = '\0';
 	prev_part_no = 0;
-	tmp[0] = '\0';
+	mr_asprintf(&flaws_str, "%s", "");
 
 
 	physical_drive_size = get_phys_size_of_drive(drive);
 
 	if (physical_drive_size < 0) {
-		sprintf(tmp, " %s does not exist.", drive);
-		strcat(flaws_str, tmp);
+		mr_asprintf(&tmp, " %s does not exist.", drive);
+		mr_strcat(flaws_str, "%s", tmp);
 	} else {
-		sprintf(tmp, "%s is %ld MB", drive, physical_drive_size);
+		mr_asprintf(&tmp, "%s is %ld MB", drive, physical_drive_size);
 	}
 	log_it(tmp);
+	mr_free(tmp);
 
 
 	/* check DD */
@@ -146,33 +102,35 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 			strcpy(mountpoint, mountlist->el[pos].mountpoint);
 			/* is it too big? */
 			if (curr_part_no > 'h') {
-				sprintf(tmp, " Can only have up to 'h' in disklabel.");
+				mr_asprintf(&tmp, " Can only have up to 'h' in disklabel.");
 				log_it(tmp);
-				strcat(flaws_str, tmp);
+				mr_strcat(flaws_str, tmp);
+				mr_free(tmp);
 				res++;
 			}
 			/* does partition /dev/adXsYZ exist more than once in the mountlist? */
-			for (i = 0, mountpoint_copies = 0, device_copies = 0;
+			for (i = 0, device_copies = 0;
 				 i < mountlist->entries; i++) {
 				if (!strcmp(device, mountlist->el[i].device)) {
 					device_copies++;
 				}
 			}
 			if (device_copies > 1) {
-				sprintf(tmp, " %s %s's.", number_to_text(device_copies),
-						device);
+				mr_asprintf(&tmp, " %s %s's.", number_to_text(device_copies), device);
 				if (!strstr(flaws_str, tmp)) {
 					log_it(tmp);
-					strcat(flaws_str, tmp);
+					mr_strcat(flaws_str, tmp);
 					res++;
 				}
+				mr_free(tmp);
 			}
 			/* silly partition size? */
 			if (mountlist->el[pos].size < 8192
 				&& strcmp(mountlist->el[pos].mountpoint, "lvm")) {
-				sprintf(tmp, " %s is tiny!", device);
+				mr_asprintf(&tmp, " %s is tiny!", device);
 				log_it(tmp);
-				strcat(flaws_str, tmp);
+				mr_strcat(flaws_str, tmp);
+				mr_free(tmp);
 				res++;
 			}
 			/* mountpoint should begin with / unless it is swap, lvm or raid */
@@ -182,19 +140,21 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 				&& strcmp(mountlist->el[pos].mountpoint, "image")
 				&& strcmp(mountlist->el[pos].mountpoint, "none")
 				&& mountlist->el[pos].mountpoint[0] != '/') {
-				sprintf(tmp, " %s has a weird mountpoint.", device);
+				mr_asprintf(&tmp, " %s has a weird mountpoint.", device);
 				log_it(tmp);
-				strcat(flaws_str, tmp);
+				mr_strcat(flaws_str, tmp);
+				mr_free(tmp);
 				res++;
 			}
 			/* is format sensible? */
 			if (!is_this_a_valid_disk_format(mountlist->el[pos].format)) {
-				sprintf(tmp, " %s has unsupported format %s.", device, mountlist->el[pos].format);
+				mr_asprintf(&tmp, " %s has unsupported format %s.", device, mountlist->el[pos].format);
 				log_it(tmp);
-				strcat(flaws_str, tmp);
+				mr_strcat(flaws_str, tmp);
+				mr_free(tmp);
 				res++;
 			}
-			amount_allocated += mountlist->el[pos].size / 1024;
+			amount_allocated += mountlist->el[pos].size / 1024L;
 			prev_sp_no = cur_sp_no;
 		}
 	}
@@ -210,9 +170,8 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 				npos++;
 		}
 		if (((pos >= 0) || npos) && foundsome) {
-			sprintf(flaws_str + strlen(flaws_str),
-					" %s has both DD and PC-style partitions.", drive);
-			return ++res;		// fatal error
+			mr_strcat(flaws_str, " %s has both DD and PC-style partitions.", drive);
+			return(flaws_str);		// fatal error
 		}
 
 		build_partition_name(device, drive, curr_part_no);
@@ -221,16 +180,17 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 			/* gap in the partition list? */
 			if (curr_part_no - prev_part_no > 1) {
 				if (prev_part_no == 0) {
-					sprintf(tmp, " Gap prior to %s.", device);
+					mr_asprintf(&tmp, " Gap prior to %s.", device);
 					log_it(tmp);
-					strcat(flaws_str, tmp);
+					mr_strcat(flaws_str, "%s", tmp);
+					mr_free(tmp);
 					res++;
 				} else if (curr_part_no > 5
 						   || (curr_part_no <= 4 && prev_part_no > 0)) {
-					sprintf(tmp, " Gap between %ss%d and %d.", drive,
-							prev_part_no, curr_part_no);
+					mr_asprintf(&tmp, " Gap between %ss%d and %d.", drive, prev_part_no, curr_part_no);
 					log_it(tmp);
-					strcat(flaws_str, tmp);
+					mr_strcat(flaws_str, "%s", tmp);
+					mr_free(tmp);
 					res++;
 				}
 			}
@@ -239,33 +199,35 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 			/* no spare primary partitions to help accommodate the logical(s)? */
 			if ((curr_part_no >= 5 && prev_part_no == 4)
 				&& (strcmp(part_table_fmt, "MBR") == 0)) {
-				sprintf(tmp, " Partition %ss4 is occupied.", drive);
+				mr_asprintf(&tmp, " Partition %ss4 is occupied.", drive);
 				log_it(tmp);
-				strcat(flaws_str, tmp);
+				mr_strcat(flaws_str, "%s", tmp);
+				mr_free(tmp);
 				res++;
 			}
 			/* does partition /dev/adXsY exist more than once in the mountlist? */
-			for (i = 0, mountpoint_copies = 0, device_copies = 0;
+			for (i = 0, device_copies = 0;
 				 i < mountlist->entries; i++) {
 				if (!strcmp(device, mountlist->el[i].device)) {
 					device_copies++;
 				}
 			}
 			if (device_copies > 1) {
-				sprintf(tmp, " %s %s's.", number_to_text(device_copies),
-						device);
-				if (!strstr(flaws_str, tmp)) {
+				mr_asprintf(&tmp, " %s %s's.", number_to_text(device_copies), device);
+				if (!strstr(flaws_str, "%s", tmp)) {
 					log_it(tmp);
-					strcat(flaws_str, tmp);
+					mr_strcat(flaws_str, "%s", tmp);
 					res++;
 				}
+				mr_free(tmp);
 			}
 			/* silly partition size? */
 			if (mountlist->el[pos].size < 8192
 				&& strcmp(mountlist->el[pos].mountpoint, "lvm")) {
-				sprintf(tmp, " %s is tiny!", device);
+				mr_asprintf(&tmp, " %s is tiny!", device);
 				log_it(tmp);
-				strcat(flaws_str, tmp);
+				mr_strcat(flaws_str, "%s", tmp);
+				mr_free(tmp);
 				res++;
 			}
 			/* mountpoint should begin with / unless it is swap, lvm or raid */
@@ -275,16 +237,18 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 				&& strcmp(mountlist->el[pos].mountpoint, "image")
 				&& strcmp(mountlist->el[pos].mountpoint, "none")
 				&& mountlist->el[pos].mountpoint[0] != '/') {
-				sprintf(tmp, " %s has a weird mountpoint.", device);
+				mr_asprintf(&tmp, " %s has a weird mountpoint.", device);
 				log_it(tmp);
-				strcat(flaws_str, tmp);
+				mr_strcat(flaws_str, "%s", tmp);
+				mr_free(tmp);
 				res++;
 			}
 			/* is format sensible? */
 			if (!is_this_a_valid_disk_format(mountlist->el[pos].format)) {
-				sprintf(tmp, " %s has unsupported format %s.", device, mountlist->el[pos].format);
+				mr_asprintf(&tmp, " %s has unsupported format %s.", device, mountlist->el[pos].format);
 				log_it(tmp);
-				strcat(flaws_str, tmp);
+				mr_strcat(flaws_str, "%s", tmp);
+				mr_free(tmp);
 				res++;
 			}
 		} else {
@@ -298,33 +262,35 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 				strcpy(mountpoint, mountlist->el[pos].mountpoint);
 				/* is it too big? */
 				if (curr_part_no > 'h') {
-					sprintf(tmp, " Can only have up to 'h' in disklabel.");
+					mr_asprintf(&tmp, " Can only have up to 'h' in disklabel.");
 					log_it(tmp);
-					strcat(flaws_str, tmp);
+					mr_strcat(flaws_str, "%s", tmp);
+					mr_free(tmp);
 					res++;
 				}
 				/* does partition /dev/adXsYZ exist more than once in the mountlist? */
-				for (i = 0, mountpoint_copies = 0, device_copies = 0;
+				for (i = 0, device_copies = 0;
 					 i < mountlist->entries; i++) {
 					if (!strcmp(device, mountlist->el[i].device)) {
 						device_copies++;
 					}
 				}
 				if (device_copies > 1) {
-					sprintf(tmp, " %s %s's.",
-							number_to_text(device_copies), device);
+					mr_asprintf(&tmp, " %s %s's.", number_to_text(device_copies), device);
 					if (!strstr(flaws_str, tmp)) {
 						log_it(tmp);
-						strcat(flaws_str, tmp);
+						mr_strcat(flaws_str, "%s", tmp);
 						res++;
 					}
+					mr_free(tmp);
 				}
 				/* silly partition size? */
 				if (mountlist->el[pos].size < 8192
 					&& strcmp(mountlist->el[pos].mountpoint, "lvm")) {
-					sprintf(tmp, " %s is tiny!", device);
+					mr_asprintf(&tmp, " %s is tiny!", device);
 					log_it(tmp);
-					strcat(flaws_str, tmp);
+					mr_strcat(flaws_str, "%s", tmp);
+					mr_free(tmp);
 					res++;
 				}
 				/* mountpoint should begin with / unless it is swap, lvm or raid */
@@ -334,47 +300,64 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 					&& strcmp(mountlist->el[pos].mountpoint, "image")
 					&& strcmp(mountlist->el[pos].mountpoint, "none")
 					&& mountlist->el[pos].mountpoint[0] != '/') {
-					sprintf(tmp, " %s has a weird mountpoint.", device);
+					mr_asprintf(&tmp, " %s has a weird mountpoint.", device);
 					log_it(tmp);
-					strcat(flaws_str, tmp);
+					mr_strcat(flaws_str, "%s", tmp);
+					mr_free(tmp);
 					res++;
 				}
 				/* is format sensible? */
 				if (!is_this_a_valid_disk_format
 					(mountlist->el[pos].format)) {
-					sprintf(tmp, " %s has unsupported format %s.", device, mountlist->el[pos].format);
+					mr_asprintf(&tmp, " %s has unsupported format %s.", device, mountlist->el[pos].format);
 					log_it(tmp);
-					strcat(flaws_str, tmp);
+					mr_strcat(flaws_str, "%s", tmp);
+					mr_free(tmp);
 					res++;
 				}
-				amount_allocated += mountlist->el[pos].size / 1024;
+				amount_allocated += mountlist->el[pos].size / 1024L;
 				prev_sp_no = cur_sp_no;
 			}
 		}
 
 		/* OK, continue with main loop */
-		amount_allocated += mountlist->el[pos].size / 1024;
+		amount_allocated += mountlist->el[pos].size / 1024L;
+
+		/* This should only happen at the end but not tested now */
+		if (amount_allocated > physical_drive_size)	{ 
+			// Used to be +1, but what if you're 1 MB too high?
+			mr_asprintf(&tmp, " %ld MB over-allocated on %s.",
+					amount_allocated - physical_drive_size, drive);
+			log_it(tmp);
+
+			/* Do we have more than 1 MB per partition ? */
+			if (amount_allocated - physical_drive_size <= curr_part_no) {
+				/* Probably a rouding error, just passing over 
+				 * by reducing the last partition if possible */
+				amount_allocated -= mountlist->el[pos].size / 1024L;
+				mountlist->el[pos].size =  (mountlist->el[pos].size - curr_part_no*1024L);
+				amount_allocated += mountlist->el[pos].size / 1024L;
+			} else {
+				mr_strcat(flaws_str, "%s", tmp);
+				res++;
+			}
+			mr_free(tmp);
+		}
 		prev_part_no = curr_part_no;
 	}
 
 	/* Over-allocated the disk? Unallocated space on disk? */
-	if (amount_allocated > physical_drive_size)	// Used to be +1, but what if you're 1 MB too high?
-	{
-		sprintf(tmp, " %ld MB over-allocated on %s.",
-				amount_allocated - physical_drive_size, drive);
-		log_it(tmp);
-		strcat(flaws_str, tmp);
-		res++;
-	} else if (amount_allocated < physical_drive_size - 1) {	/* NOT AN ERROR, JUST A WARNING :-) */
-		sprintf(tmp, " %ld MB unallocated on %s.",
+	if (amount_allocated < physical_drive_size - 1) {	/* NOT AN ERROR, JUST A WARNING :-) */
+		mr_asprintf(&tmp, " %ld MB unallocated on %s.",
 				physical_drive_size - amount_allocated, drive);
 		log_it(tmp);
-		strcat(flaws_str, tmp);
+		mr_strcat(flaws_str, "%s", tmp);
+		mr_free(tmp);
 	}
 	if (res) {
-		return (FALSE);
+		return (NULL);
 	} else {
-		return (TRUE);
+		return (flaws_str);
 	}
 }
 
@@ -387,44 +370,48 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 	int curr_part_no = 0;
 	int pos = 0;
 	int res = 0;
-	int mountpoint_copies = 0;
 	int device_copies = 0;
 	int i = 0;
 
 	/*@ buffers ******************************************************** */
-	char *tmp;
-	char *device;
+	char *tmp = NULL;
+	char *device = NULL;
+	char *flaws_str = NULL;
 
 	/*@ long *********************************************************** */
 	long physical_drive_size = 0L;
 	long amount_allocated = 0L;
 
 	/*@ pointers ******************************************************* */
-	char *part_table_fmt;
+	char *part_table_fmt = NULL;
 
 	/*@ initialize ***************************************************** */
 	assert_string_is_neither_NULL_nor_zerolength(drive);
 	assert(mountlist != NULL);
-	assert(flaws_str != NULL);
+	mr_asprintf(&flaws_str, "%s", "");
 
-	malloc_string(tmp);
 	malloc_string(device);
-	flaws_str[0] = '\0';
 	prev_part_no = 0;
-	tmp[0] = '\0';
-
 
 	physical_drive_size = get_phys_size_of_drive(drive);
 
 	if (physical_drive_size < 0) {
-		sprintf(tmp, " %s does not exist.", drive);
-		strcat(flaws_str, tmp);
-		res++;
-		log_msg(1, tmp);
-		goto endoffunc;
+		if (strstr(drive,"/dev/dm-") == NULL) {
+			mr_asprintf(&tmp, " %s does not exist.", drive);
+			log_it(tmp);
+			mr_strcat(flaws_str, "%s", tmp);
+			res++;
+			mr_free(tmp);
+			goto endoffunc;
+		} else {
+			mr_asprintf(&tmp, " %s (dm) will be setup later on", drive);
+			log_it(tmp);
+			mr_free(tmp);
+		}
 	} else {
-		sprintf(tmp, "%s is %ld MB", drive, physical_drive_size);
+		mr_asprintf(&tmp, "%s is %ld MB", drive, physical_drive_size);
 		log_it(tmp);
+		mr_free(tmp);
 	}
 
 	for (curr_part_no = 1; curr_part_no < 99; curr_part_no++) {
@@ -433,19 +420,22 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 		if (pos < 0) {
 			continue;
 		}
+		log_msg(2, "Processing partition %s on %s #%d in mountlist", device, drive, pos);
 		/* gap in the partition list? */
 		if (curr_part_no - prev_part_no > 1) {
 			if (prev_part_no == 0) {
-				sprintf(tmp, " Gap prior to %s.", device);
+				mr_asprintf(&tmp, " Gap prior to %s.", device);
 				log_it(tmp);
-				strcat(flaws_str, tmp);
+				mr_strcat(flaws_str, "%s", tmp);
+				mr_free(tmp);
 				res++;
 			} else if (curr_part_no > 5
 					   || (curr_part_no <= 4 && prev_part_no > 0)) {
-				sprintf(tmp, " Gap on %s between %d and %d.", drive,
+				mr_asprintf(&tmp, " Gap on %s between %d and %d.", drive,
 						prev_part_no, curr_part_no);
 				log_it(tmp);
-				strcat(flaws_str, tmp);
+				mr_strcat(flaws_str, "%s", tmp);
+				mr_free(tmp);
 				res++;
 			}
 		}
@@ -454,33 +444,36 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 		/* no spare primary partitions to help accommodate the logical(s)? */
 		if ((curr_part_no >= 5 && prev_part_no == 4)
 			&& (strcmp(part_table_fmt, "MBR") == 0)) {
-			sprintf(tmp, " Partition 4 of %s is occupied.", drive);
+			mr_asprintf(&tmp, " Partition 4 of %s is occupied.", drive);
 			log_it(tmp);
-			strcat(flaws_str, tmp);
+			mr_strcat(flaws_str, "%s", tmp);
+			mr_free(tmp);
 			res++;
 		}
 		/* does partition /dev/hdNX exist more than once in the mountlist? */
-		for (i = 0, mountpoint_copies = 0, device_copies = 0;
+		for (i = 0, device_copies = 0;
 			 i < mountlist->entries; i++) {
 			if (!strcmp(device, mountlist->el[i].device)) {
 				device_copies++;
 			}
 		}
 		if (device_copies > 1) {
-			sprintf(tmp, " %s %s's.", number_to_text(device_copies),
+			mr_asprintf(&tmp, " %s %s's.", number_to_text(device_copies),
 					device);
 			if (!strstr(flaws_str, tmp)) {
 				log_it(tmp);
-				strcat(flaws_str, tmp);
+				mr_strcat(flaws_str, "%s", tmp);
 				res++;
 			}
+			mr_free(tmp);
 		}
 		/* silly partition size? */
 		if (mountlist->el[pos].size < 8192
 			&& strcmp(mountlist->el[pos].mountpoint, "lvm")) {
-			sprintf(tmp, " %s is tiny!", device);
+			mr_asprintf(&tmp, " %s is tiny!", device);
 			log_it(tmp);
-			strcat(flaws_str, tmp);
+			mr_strcat(flaws_str, "%s", tmp);
+			mr_free(tmp);
 			res++;
 		}
 		/* mountpoint should begin with / unless it is swap, lvm or raid */
@@ -489,85 +482,84 @@ int evaluate_drive_within_mountlist(struct mountlist_itself *mountlist,
 			&& strcmp(mountlist->el[pos].mountpoint, "raid")
 			&& strcmp(mountlist->el[pos].mountpoint, "image")
 			&& mountlist->el[pos].mountpoint[0] != '/') {
-			sprintf(tmp, " %s has a weird mountpoint.", device);
+			mr_asprintf(&tmp, " %s has a weird mountpoint.", device);
 			log_it(tmp);
-			strcat(flaws_str, tmp);
+			mr_strcat(flaws_str, "%s", tmp);
+			mr_free(tmp);
 			res++;
 		}
 		/* is format sensible? */
 		if (!is_this_a_valid_disk_format(mountlist->el[pos].format)) {
-			sprintf(tmp, " %s has unsupported format %s.", device, mountlist->el[pos].format);
+			mr_asprintf(&tmp, " %s has unsupported format %s.", device, mountlist->el[pos].format);
 			log_it(tmp);
-			strcat(flaws_str, tmp);
+			mr_strcat(flaws_str, "%s", tmp);
+			mr_free(tmp);
 			res++;
 		}
 		/* OK, continue with main loop */
-		amount_allocated += mountlist->el[pos].size / 1024;
+		amount_allocated += mountlist->el[pos].size / 1024L;
 		prev_part_no = curr_part_no;
 	}
 
 	/* Over-allocated the disk? Unallocated space on disk? */
 	if (amount_allocated > physical_drive_size + 1) {
-		sprintf(tmp, " %ld MB over-allocated on %s.",
+		mr_asprintf(&tmp, " %ld MB over-allocated on %s.",
 				amount_allocated - physical_drive_size, drive);
 		log_it(tmp);
-		strcat(flaws_str, tmp);
+		mr_strcat(flaws_str, "%s", tmp);
+		mr_free(tmp);
 		res++;
 	} else if (amount_allocated < physical_drive_size - 1) {	/* NOT AN ERROR, JUST A WARNING :-) */
-		sprintf(tmp, " %ld MB unallocated on %s.",
+		mr_asprintf(&tmp, " %ld MB unallocated on %s.",
 				physical_drive_size - amount_allocated, drive);
 		log_it(tmp);
-		strcat(flaws_str, tmp);
+		mr_strcat(flaws_str, "%s", tmp);
+		mr_free(tmp);
 	}
 
   endoffunc:
-	paranoid_free(tmp);
 	paranoid_free(device);
 
-	if (res) {
-		return (FALSE);
+	if (res == 0) {
+		mr_free(flaws_str);
+		log_msg(2, "Fine, no error in evaluate_drive_within_mountlist");
+		return (NULL);
 	} else {
-		return (TRUE);
+		log_msg(2, "Error in evaluate_drive_within_mountlist: %s", flaws_str);
+		return (flaws_str);
 	}
 }
 #endif
 
 
-
 /**
  * Evaluate a whole mountlist for flaws. Calls evaluate_drive_within_mountlist()
  * for each drive, and then spreads the flaws across three lines.
  * @param mountlist The mountlist to evaluate.
- * @param flaws_str_A Where to put the first line listing errors found.
- * @param flaws_str_B Where to put the second line listing errors found.
- * @param flaws_str_C Where to put the third line listing errors found.
- * @return The number of flaws found (0 for success).
+ * @return The flaws string (NULL for success).
  * @see evaluate_drive_within_mountlist
  */
-int
-evaluate_mountlist(struct mountlist_itself *mountlist, char *flaws_str_A,
-				   char *flaws_str_B, char *flaws_str_C)
-{
+char *evaluate_mountlist(struct mountlist_itself *mountlist) {
 
 	/*@ buffer *********************************************************** */
 	struct list_of_disks *drivelist;
-	char *tmp;
-	char *flaws_str;
+	char *tmp = NULL;
+	char *flaws_str = NULL;
+
+	int currline = 0;
+	int copies = 0;
+	int last_copy = 0;
+
+	/*@ buffetr ********************************************************* */
+	char *curr_mountpoint = NULL;
 
 	/*@ int ************************************************************** */
 	int i = 0;
-	int res = 0;
 
 	/*@ initialize ******************************************************* */
 
 	drivelist = malloc(sizeof(struct list_of_disks));
-	malloc_string(tmp);
-	malloc_string(flaws_str);
 	assert(mountlist != NULL);
-	assert(flaws_str_A != NULL);
-	assert(flaws_str_B != NULL);
-	assert(flaws_str_C != NULL);
-	flaws_str[0] = '\0';
 
 	make_list_of_drives_in_mountlist(mountlist, drivelist);
 
@@ -577,24 +569,44 @@ evaluate_mountlist(struct mountlist_itself *mountlist, char *flaws_str_A,
 		if (strstr
 			(drivelist->el[i].device,
 			 DONT_KNOW_HOW_TO_EVALUATE_THIS_DEVICE_TYPE)) {
-			sprintf(tmp, " Not evaluating %s (I don't know how yet)",
-					drivelist->el[i].device);
+			mr_asprintf(&tmp, " Not evaluating %s (I don't know how yet)", drivelist->el[i].device);
 			log_it(tmp);
-			tmp[0] = '\0';
+			paranoid_free(tmp);
 		} else {
-			if (!evaluate_drive_within_mountlist
-				(mountlist, drivelist->el[i].device, tmp)) {
-				res++;
+			log_msg(8, "Evaluating drive #%d (%s) within mountlist", i, drivelist->el[i].device);
+			tmp = evaluate_drive_within_mountlist(mountlist, drivelist->el[i].device);
+		}
+		log_msg(8,"Entry: %d (%s)", i, drivelist->el[i].device);
+		/* BCO: tmp can be NULL */
+		if (tmp != NULL) {
+			log_msg(8,"Adding: %s to flaws_str", tmp);
+			mr_strcat(flaws_str, "%s", tmp);
+			paranoid_free(tmp);
+		}
+	}
+
+	/* Look for duplicate mountpoints in mountlist. */
+	for (currline = 0; currline < mountlist->entries; currline++) {
+		mr_asprintf(&curr_mountpoint, "%s", mountlist->el[currline].mountpoint);
+		for (i = 0, copies = 0, last_copy = -1; i < mountlist->entries; i++) {
+			if (!strcmp(mountlist->el[i].mountpoint, curr_mountpoint)
+				&& strcmp(mountlist->el[i].mountpoint, "lvm")
+				&& strcmp(mountlist->el[i].mountpoint, "swap")) {
+				last_copy = i;
+				copies++;
 			}
 		}
-		strcat(flaws_str, tmp);
+		if (copies > 1 && last_copy == currline
+			&& strcmp(curr_mountpoint, "raid")) {
+			mr_asprintf(&tmp, " %s %s's.", number_to_text(copies), curr_mountpoint);
+			mr_strcat(flaws_str, "%s", tmp);
+			log_msg(8,"Adding: %s to flaws_str", tmp);
+			paranoid_free(tmp);
+		}
+		paranoid_free(curr_mountpoint);
 	}
-	res += look_for_duplicate_mountpoints(mountlist, flaws_str);
-/*  res+=look_for_weird_formats(mountlist,flaws_str); .. not necessary, now that we can check to see
- which formarts are actually _supported_ by the kernel */
-	/* log_it(flaws_str); */
-	return (spread_flaws_across_three_lines
-			(flaws_str, flaws_str_A, flaws_str_B, flaws_str_C, res));
+
+	return(flaws_str);
 }
 
 
@@ -611,11 +623,6 @@ find_device_in_mountlist(struct mountlist_itself *mountlist, char *device)
 
 	/*@ int ************************************************************** */
 	int i = 0;
-	char *tmp;
-	char *flaws_str;
-
-	malloc_string(tmp);
-	malloc_string(flaws_str);
 
 	assert(mountlist != NULL);
 	assert_string_is_neither_NULL_nor_zerolength(device);
@@ -623,9 +630,6 @@ find_device_in_mountlist(struct mountlist_itself *mountlist, char *device)
 		 i < mountlist->entries
 		 && strcmp(mountlist->el[i].device, device) != 0; i++);
 
-	paranoid_free(tmp);
-	paranoid_free(flaws_str);
-
 	if (i == mountlist->entries) {
 		return (-1);
 	} else {
@@ -634,113 +638,6 @@ find_device_in_mountlist(struct mountlist_itself *mountlist, char *device)
 }
 
 
-
-
-
-/**
- * Look for duplicate mountpoints in @p mountlist.
- * @param mountlist The mountlist to check.
- * @param flaws_str The flaws string to append the results to.
- * @return The number of mountpoints that have duplicates, or 0 for success.
- */
-int
-look_for_duplicate_mountpoints(struct mountlist_itself *mountlist,
-							   char *flaws_str)
-{
-
-	/*@ int ************************************************************* */
-	int res = 0;
-	int currline = 0;
-	int i = 0;
-	int copies = 0;
-	int last_copy = 0;
-
-	/*@ buffetr ********************************************************* */
-	char *curr_mountpoint;
-	char *tmp;
-
-	malloc_string(curr_mountpoint);
-	malloc_string(tmp);
-	assert(mountlist != NULL);
-	assert(flaws_str != NULL);
-	for (currline = 0; currline < mountlist->entries; currline++) {
-		strcpy(curr_mountpoint, mountlist->el[currline].mountpoint);
-		for (i = 0, copies = 0, last_copy = -1; i < mountlist->entries;
-			 i++) {
-			if (!strcmp(mountlist->el[i].mountpoint, curr_mountpoint)
-				&& strcmp(mountlist->el[i].mountpoint, "lvm")
-				&& strcmp(mountlist->el[i].mountpoint, "swap")) {
-				last_copy = i;
-				copies++;
-			}
-		}
-		if (copies > 1 && last_copy == currline
-			&& strcmp(curr_mountpoint, "raid")) {
-			sprintf(tmp, " %s %s's.", number_to_text(copies),
-					curr_mountpoint);
-			strcat(flaws_str, tmp);
-			log_it(tmp);
-			res++;
-		}
-	}
-	paranoid_free(curr_mountpoint);
-	paranoid_free(tmp);
-	return (res);
-}
-
-/**
- * Look for strange formats. Does not respect /proc/filesystems.
- * @param mountlist The mountlist to check.
- * @param flaws_str The flaws string to append the results to.
- * @return The number of weird formats found, or 0 for success.
- * @bug Seems orphaned; please remove.
- */
-int
-look_for_weird_formats(struct mountlist_itself *mountlist, char *flaws_str)
-{
-
-	/*@ int ************************************************************* */
-	int i = 0;
-	int res = 0;
-
-	/*@ buffers ********************************************************* */
-	char *tmp;
-	char *format_sz;
-
-	malloc_string(tmp);
-	malloc_string(format_sz);
-
-	assert(mountlist != NULL);
-	assert(flaws_str != NULL);
-
-	for (i = 0; i < mountlist->entries; i++) {
-		sprintf(format_sz, " %s ", mountlist->el[i].format);
-		if (!strstr(SANE_FORMATS, format_sz)
-			&& strcmp(mountlist->el[i].mountpoint, "image") != 0) {
-			sprintf(tmp, " %s has unknown format.",
-					mountlist->el[i].device);
-			log_it(tmp);
-			strcat(flaws_str, tmp);
-			res++;
-		} else if ((!strcmp(mountlist->el[i].format, "swap")
-					&& strcmp(mountlist->el[i].mountpoint, "swap")
-					&& strcmp(mountlist->el[i].mountpoint, "none"))
-				   || (strcmp(mountlist->el[i].format, "swap")
-					   && !strcmp(mountlist->el[i].mountpoint, "swap")
-					   && !strcmp(mountlist->el[i].mountpoint, "none"))) {
-			sprintf(tmp, " %s is half-swap.", mountlist->el[i].device);
-			log_it(tmp);
-			strcat(flaws_str, tmp);
-			res++;
-		}
-	}
-	paranoid_free(tmp);
-	paranoid_free(format_sz);
-	return (res);
-}
-
-
-
 /**
  * Make a list of the drives mentioned in the mountlist.
  * @param mountlist The mountlist to examine.
@@ -758,74 +655,63 @@ make_list_of_drives_in_mountlist(struct mountlist_itself *mountlist,
 	int j;
 
 	/*@ buffers ********************************************************* */
-	char *drive;
-	char *tmp;
+	char *drive = NULL;
+	char *truncdrive = NULL;
 
 	long long size;
 
-	malloc_string(drive);
-	malloc_string(tmp);
 	assert(mountlist != NULL);
 	assert(drivelist != NULL);
 	log_it("Making list of drives");
 	for (lino = 0, noof_drives = 0; lino < mountlist->entries; lino++) {
 
-		strcpy(drive, mountlist->el[lino].device);
+		mr_asprintf(&drive, "%s", mountlist->el[lino].device);
 		if (!strncmp(drive, RAID_DEVICE_STUB, strlen(RAID_DEVICE_STUB))) {
-			sprintf(tmp,
-					"Not putting %s in list of drives: it's a virtual drive",
-					drive);
-			log_msg(8, tmp);
+			log_msg(8, "Not putting %s in list of drives: it's a virtual drive", drive);
+			paranoid_free(drive);
 			continue;
 		}
 
 		size = mountlist->el[lino].size;
 		if (size == 0) {
-			sprintf(tmp,
-					"Not putting %s in list of drives: it has zero size (maybe an LVM volume)",
-					drive);
-			log_msg(8, tmp);
+			log_msg(8, "Not putting %s in list of drives: it has zero size (maybe an LVM volume)", drive);
+			paranoid_free(drive);
 			continue;
 		}
 
-/*
-      for (i = strlen (drive); isdigit (drive[i - 1]); i--);
-      drive[i] = '\0';
-      if (get_phys_size_of_drive (drive) <= 0 && drive[i - 1] == 'p')
-	{
-	  i--;
-	  drive[i] = '\0';
-	}
-      for (j = 0; j < noof_drives && strcmp (drivelist[j], drive) != 0; j++);
-*/
+		log_msg(8, "Putting %s with size %lli in list of drives", drive, size);
 
-		sprintf(tmp,
-				"Putting %s with size %lli in list of drives",
-				drive, size);
-		log_msg(8, tmp);
+		/* memory allocation */
+		truncdrive = truncate_to_drive_name(drive);
+		paranoid_free(drive);
+
+		log_msg(8, "drive truncated to %s", truncdrive);
 
-		(void) truncate_to_drive_name(drive);
 		for (j = 0;
 			 j < noof_drives
-			 && strcmp(drivelist->el[j].device, drive) != 0; j++)
+			 && strcmp(drivelist->el[j].device, truncdrive) != 0; j++) {
 			continue;
+		}
 		if (j == noof_drives) {
-			strcpy(drivelist->el[noof_drives++].device, drive);
+			strncpy(drivelist->el[noof_drives].device, truncdrive, 63);
+			drivelist->el[noof_drives].device[63] = '\0';
+			log_msg(8,"Adding drive %s to list", drivelist->el[noof_drives].device);
+			noof_drives++;
+		}
+		paranoid_free(truncdrive);
+		if (noof_drives >= MAXIMUM_DISKS_PER_RAID_DEV) {
+			log_msg(0, "Unable to handle mountlist with more than %d lines", MAXIMUM_DISKS_PER_RAID_DEV);
+			log_to_screen("Unable to handle a so big mountlist");
+			finish(1);
 		}
 	}
 	drivelist->entries = noof_drives;
-	log_msg(8, "Made list of drives");
-	paranoid_free(drive);
-	paranoid_free(tmp);
+	log_msg(8, "Made list of %d drives",noof_drives);
 
 	return (noof_drives);
 }
 
 
-
-
-
-
 /**
  * Make a list of RAID partitions not currently associated with any RAID device.
  * The user may add any of these partitions to the RAID device.
@@ -847,9 +733,8 @@ void make_list_of_unallocated_raid_partitions(struct mountlist_itself
 	int used_by = 0;
 
 	/*@ buffers ********************************************************* */
-	char *tmp;
+	char *tmp = NULL;
 
-	malloc_string(tmp);
 	assert(output_list != NULL);
 	assert(mountlist != NULL);
 	assert(raidlist != NULL);
@@ -867,22 +752,17 @@ void make_list_of_unallocated_raid_partitions(struct mountlist_itself
 				memcpy((void *) &output_list->el[items++],
 					   (void *) &mountlist->el[i],
 					   sizeof(struct mountlist_line));
-				sprintf(tmp,
-						"%s is available; user may choose to add it to raid device",
-						output_list->el[items - 1].device);
+				mr_asprintf(&tmp, "%s is available; user may choose to add it to raid device", output_list->el[items - 1].device);
 				log_it(tmp);
+				paranoid_free(tmp);
 			}
 		}
 	}
 	output_list->entries = items;
 	log_it("MLUORP -- ending");
-	paranoid_free(tmp);
 }
 
 
-
-
-
 /**
  * Get the size of a mountlist entry by the @c device field.
  * @param mountlist The mountlist to search in.
@@ -911,8 +791,6 @@ size_of_specific_device_in_mountlist(struct mountlist_itself *mountlist,
 }
 
 
-
-
 /**
  * Load a file on disk into @p mountlist.
  * The file on disk should consist of multiple lines, each containing 4 or 5
@@ -925,82 +803,81 @@ size_of_specific_device_in_mountlist(struct mountlist_itself *mountlist,
  */
 int load_mountlist(struct mountlist_itself *mountlist, char *fname)
 {
-	FILE *fin;
+	FILE *fin = NULL;
 	/* malloc ** */
-	char *incoming;
-	char *siz;
-	char *tmp;
-	char *p;
+	char *incoming = NULL;
+	char *siz = NULL;
+	char *tmp = NULL;
+	char *p = NULL;
+	char *q = NULL;
 
-	int items;
-	int j;
+	int items = 0;
+	int j = 0;
+	int res = 0;
 
 	assert(mountlist != NULL);
 	assert_string_is_neither_NULL_nor_zerolength(fname);
-	malloc_string(incoming);
-	malloc_string(siz);
-	malloc_string(tmp);
+
 	if (!(fin = fopen(fname, "r"))) {
 		log_it("Unable to open mountlist - '%s'", fname);
 		log_to_screen("Cannot open mountlist");
-		paranoid_free(incoming);
-		paranoid_free(siz);
-		paranoid_free(tmp);
 		return (1);
 	}
-	items = 0;
-	(void) fgets(incoming, MAX_STR_LEN - 1, fin);
+	malloc_string(incoming);
+	malloc_string(siz);
+	q = fgets(incoming, MAX_STR_LEN - 1, fin);
 	log_it("Loading mountlist...");
-	while (!feof(fin)) {
+	while (!feof(fin) && (q != NULL)) {
 #if linux
-		sscanf(incoming,
-			   "%s %s %s %s %s %s",
+		res = sscanf(incoming,
+			   "%s %s %s %s %s",
 			   mountlist->el[items].device,
 			   mountlist->el[items].mountpoint,
 			   mountlist->el[items].format,
-			   siz, mountlist->el[items].label, mountlist->el[items].uuid);
+			   siz, mountlist->el[items].label);
+		if (res < 5) {
+			/* no label found */
+			log_msg(4, "no label found for %s",mountlist->el[items].device);
+			strcpy(mountlist->el[items].label,"");
+		}
 #elif __FreeBSD__
-		sscanf(incoming,
+		res = sscanf(incoming,
 			   "%s %s %s %s",
 			   mountlist->el[items].device,
 			   mountlist->el[items].mountpoint,
 			   mountlist->el[items].format, siz);
-		strcpy(mountlist->el[items].label, "");
-		strcpy(mountlist->el[items].uuid, "");
+		strcpy(mountlist->el[items].label,"");
 #endif
 
 		if (!strcmp(mountlist->el[items].device, "/proc") ||
 			!strcmp(mountlist->el[items].device, "proc") ||
 			!strcmp(mountlist->el[items].device, "/sys") ||
 			!strcmp(mountlist->el[items].device, "sys") ||
+			!strcmp(mountlist->el[items].device, "/run") ||
+			!strcmp(mountlist->el[items].device, "run") ||
 			!strcmp(mountlist->el[items].device, "/devpts") ||
 			!strcmp(mountlist->el[items].device, "devpts")
 			) {
 			log_msg(1,
 					"Ignoring %s in mountlist - not loading that line :) ",
 					mountlist->el[items].device);
-			(void) fgets(incoming, MAX_STR_LEN - 1, fin);
+			q = fgets(incoming, MAX_STR_LEN - 1, fin);
 			continue;
 		}
 		mountlist->el[items].size = atoll(siz);
 		if (mountlist->el[items].device[0] != '\0'
 			&& mountlist->el[items].device[0] != '#') {
-			if (items >= ARBITRARY_MAXIMUM) {
-				log_to_screen("Too many lines in mountlist.. ABORTING");
-				finish(1);
-			}
 			for (j = 0;
 				 j < items
 				 && strcmp(mountlist->el[j].device,
 						   mountlist->el[items].device); j++);
 			if (j < items) {
 				strcat(mountlist->el[items].device, "_dup");
-				sprintf(tmp,
-						"Duplicate entry in mountlist - renaming to %s",
-						mountlist->el[items].device);
+				mr_asprintf(&tmp, "Duplicate entry in mountlist - renaming to %s", mountlist->el[items].device);
 				log_it(tmp);
+				paranoid_free(tmp);
 			}
-			strcpy(tmp, mountlist->el[items].device);
+			mr_asprintf(&tmp, "%s", mountlist->el[items].device);
 			if (strstr(tmp, "/dev/md/")) {
 				log_it("format_device() --- Contracting %s", tmp);
 				p = strrchr(tmp, '/');
@@ -1013,30 +890,30 @@ int load_mountlist(struct mountlist_itself *mountlist, char *fname)
 					   mountlist->el[items].device, tmp);
 				strcpy(mountlist->el[items].device, tmp);
 			}
+			paranoid_free(tmp);
 
-			sprintf(tmp,
-					"%s %s %s %lld %s %s",
-					mountlist->el[items].device,
-					mountlist->el[items].mountpoint,
-					mountlist->el[items].format,
-					mountlist->el[items].size,
-					mountlist->el[items].label,
-					mountlist->el[items].uuid);
-
-			log_it(tmp);
+			log_it("%s %s %s %lld %s",
+					 mountlist->el[items].device,
+					 mountlist->el[items].mountpoint,
+					 mountlist->el[items].format,
+					 mountlist->el[items].size,
+					 mountlist->el[items].label);
 			items++;
+			if (items >= MAX_MOUNTLIST_ENTRIES) {
+				log_to_screen("Too many lines in mountlist.. ABORTING");
+				finish(1);
+			}
 		}
-		(void) fgets(incoming, MAX_STR_LEN - 1, fin);
+		q = fgets(incoming, MAX_STR_LEN - 1, fin);
 	}
 	paranoid_fclose(fin);
 	mountlist->entries = items;
 
 	log_it("Mountlist loaded successfully.");
-	sprintf(tmp, "%d entries in mountlist", items);
-	log_it(tmp);
+	log_it("%d entries in mountlist", items);
+
 	paranoid_free(incoming);
 	paranoid_free(siz);
-	paranoid_free(tmp);
 	return (0);
 }
 
@@ -1064,17 +941,22 @@ int save_mountlist_to_disk(struct mountlist_itself *mountlist, char *fname)
 	}
 	for (i = 0; i < mountlist->entries; i++) {
 		fprintf(fout,
-				"%-15s %-15s %-15s %-15lld %-15s %-15s\n",
+				"%-15s %-15s %-15s %-15lld %-15s\n",
 				mountlist->el[i].device, mountlist->el[i].mountpoint,
 				mountlist->el[i].format, mountlist->el[i].size,
-				mountlist->el[i].label, mountlist->el[i].uuid);
+				mountlist->el[i].label);
 	}
 	paranoid_fclose(fout);
+	if (!(fout = fopen(MONDO_MNTLISTCHG, "w"))) {
+		log_OS_error("WMTD - Cannot openout "MONDO_MNTLISTCHG);
+		return (1);
+	}
+	fprintf(fout, "the mountlist was changed by mondorestore\n");
+	paranoid_fclose(fout);
 	return (0);
 }
 
 
-
 /**
  * Sort the mountlist alphabetically by device.
  * The sorting is done in-place.
@@ -1100,6 +982,7 @@ void sort_mountlist_by_device(struct mountlist_itself *mountlist)
 	}
 }
 
+
 /**
  * Sort the mountlist alphabetically by mountpoint.
  * The sorting is done in-place.
@@ -1139,9 +1022,9 @@ void
 swap_mountlist_entries(struct mountlist_itself *mountlist, int a, int b)
 {
 	/*@ mallocs *** */
-	char device[64];
-	char mountpoint[256];
-	char format[64];
+	char *device = NULL;
+	char *mountpoint = NULL;
+	char *format = NULL;
 
 	long long size;
 
@@ -1149,9 +1032,9 @@ swap_mountlist_entries(struct mountlist_itself *mountlist, int a, int b)
 	assert(a >= 0);
 	assert(b >= 0);
 
-	strcpy(device, mountlist->el[a].device);
-	strcpy(mountpoint, mountlist->el[a].mountpoint);
-	strcpy(format, mountlist->el[a].format);
+	mr_asprintf(&device, "%s", mountlist->el[a].device);
+	mr_asprintf(&mountpoint, "%s", mountlist->el[a].mountpoint);
+	mr_asprintf(&format, "%s", mountlist->el[a].format);
 
 	size = mountlist->el[a].size;
 
@@ -1166,6 +1049,9 @@ swap_mountlist_entries(struct mountlist_itself *mountlist, int a, int b)
 	strcpy(mountlist->el[b].format, format);
 
 	mountlist->el[b].size = size;
+	paranoid_free(device);
+	paranoid_free(mountpoint);
+	paranoid_free(format);
 }
 
 /* @} - end of mountlistGroup */
