@@ -1,17 +1,11 @@
 /* libmondo-mountlist.h
- * $Id: libmondo-mountlist.h 1295 2007-04-11 18:59:49Z bruno $
+ * $Id: libmondo-mountlist.h 2394 2009-09-12 00:53:02Z bruno $
  */
 
 
-int evaluate_mountlist(struct mountlist_itself *mountlist,
-					   char *flaws_str_A, char *flaws_str_B,
-					   char *flaws_str_C);
+char *evaluate_mountlist(struct mountlist_itself *mountlist);
 int find_device_in_mountlist(struct mountlist_itself *mountlist,
 							 char *device);
-int look_for_duplicate_mountpoints(struct mountlist_itself *mountlist,
-								   char *flaws_str);
-int look_for_weird_formats(struct mountlist_itself *mountlist,
-						   char *flaws_str);
 int make_list_of_drives_in_mountlist(struct mountlist_itself *,
 									 struct list_of_disks *);
 long long size_of_specific_device_in_mountlist(struct mountlist_itself
