@@ -2,6 +2,7 @@
 
 extern void clean_up_KDE_desktop_if_necessary(void);
 extern long get_time();
+extern char *mr_date(void);
 extern void (*log_debug_msg) (int debug_level, const char *szFile,
 							  const char *szFunction, int nLine,
 							  const char *fmt, ...);
@@ -10,29 +11,27 @@ extern void standard_log_debug_msg(int debug_level, const char *szFile,
 								   const char *fmt, ...);
 extern int read_cfg_var(char *config_file, char *label, char *value);
 extern int write_cfg_var(char *config_file, char *label, char *value);
-extern void reset_bkpinfo(struct s_bkpinfo *bkpinfo);
+extern void reset_bkpinfo();
 #ifdef __FreeBSD__
 extern void initialize_raidrec(struct vinum_volume *vv);
 #else
 extern void initialize_raidrec(struct raid_device_record *raidrec);
 #endif
-extern void log_trace(char *o);
 extern int some_basic_system_sanity_checks();
 
 
 extern int g_loglevel;
 
-
 extern void insmod_crucial_modules(void);
 extern int find_and_store_mondoarchives_home(char *home_sz);
 
 extern void unmount_supermounts_if_necessary(void);
 extern void remount_supermounts_if_necessary(void);
 
-extern int post_param_configuration(struct s_bkpinfo *bkpinfo);
+extern int post_param_configuration();
 
 
-extern int pre_param_configuration(struct s_bkpinfo *bkpinfo);
+extern int pre_param_configuration();
 
 
 extern void mount_boot_if_necessary(void);
