@@ -1,98 +1,7 @@
 /***************************************************************************
-libmondo-verify.c  -  description
------------------
-
-begin: Fri Apr 19 16:40:35 EDT 2002
-copyright : (C) 2002 Mondo  Hugo Rabson
-email	  : Hugo Rabson <hugorabson@msn.com>
-edited by : by Stan Benoit 4/2002
-email     : troff@nakedsoul.org
-cvsid     : $Id: libmondo-verify.c 1316 2007-04-16 14:50:28Z bruno $
+$Id: libmondo-verify.c 3116 2013-05-07 03:42:54Z bruno $
 ***************************************************************************/
 
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
-/* mondo-verify
-
-
-07/07
-- added star verify support
-
-04/04/2004
-- added star support
-
-10/23/2003
-- changed "ISO #n" to "<media descriptor> #n"
-
-10/01
-- working on biggiefile verification (CDs)
-
-09/16
-- fixed bug in CD biggie verif'n
-
-09/15
-- state explicitly that we do not verify disk images
-
-09/13
-- working on verify_all_slices_on_CD()
-
-09/01
-- write list of changed biggiefiles (streaming only) to changed.files
-
-05/05
-- exclude /dev/ * from list of changed files
-- add Joshua Oreman's FreeBSD patches
-
-04/24/2003
-- added lots of assert()'s and log_OS_error()'s
-
-09/01 - 09/30/2002
-- run_program_and_log_output() now takes boolean operator to specify
-  whether it will log its activities in the event of _success_
-- eject_device() added
-- cleaned up 'changed file' feedback a little bit
-
-08/01 - 08/31
-- make sure to prefix bkpinfo->restore_path to local biggiefile fname when
-  comparing it to the archived copy of biggiefile; otherwise, biggiefile
-  not found & checksum missing & so on
-- exclude "incheckentry xwait()" from changed.files
-- cleaned up some log_it() calls
-
-07/01 - 07/31
-- renamed libmondo-verify.c
-- say 'bigfile' not 'biggiefile'
-- exclude /dev/ * from changed.files
-
-01/01 - 06/30
-- remove /tmp/changed.files.* dead files
-- don't pclose() tape after calling closein_tape(): the latter does it already
-- fixed bug in verify_cd_image()'s CD-mounting code
-- don't write to screen "old cksum.. curr cksum.."
-- changed the gawks to awks for the benefit of Debian
-- handles files >2GB in size
-- fixed bug in /tmp/changed.files-generator
-- re-enabled a lot of CD-verification code
-- afioballs are saved in %s/tmpfs/ now (%s=tmpdir)
-- changed some labels to make them more user-friendly
-- don't chdir() anywhere before verifying stuff
-- changed files are now detected by verify_tape_backup() and listed in
-  /tmp/changed.files
-- replaced &> with > .. 2>
-- still implementing improved tape support
-
-
-Started late Dec, 2001
-
------------------------------------------------------------------------------
-*/
-
 
 /**
  * @file
@@ -100,6 +9,7 @@ Started late Dec, 2001
  */
 
 #include "my-stuff.h"
+#include "mr_mem.h"
 #include "mondostructures.h"
 #include "libmondo-verify.h"
 #include "libmondo-gui-EXT.h"
@@ -112,10 +22,7 @@ Started late Dec, 2001
 #include "lib-common-externs.h"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-verify.c 1316 2007-04-16 14:50:28Z bruno $";
-
-char *vfy_tball_fname(struct s_bkpinfo *, char *, int);
-
+//static char cvsid[] = "$Id: libmondo-verify.c 3116 2013-05-07 03:42:54Z bruno $";
 
 /**
  * The number of the most recently verified afioball.
@@ -127,6 +34,36 @@ extern char *g_getfacl;
 extern char *g_getfattr;
 extern char *MONDO_LOGFILE;
 
+/* Reference to global bkpinfo */
+extern struct s_bkpinfo *bkpinfo;
+
+
+/**
+ * Generate the filename of a tarball to verify.
+ * @param bkpinfo The backup information structure. @c bkpinfo->zip_suffix is the only field used.
+ * @param mountpoint The directory where the CD/DVD/ISO is mounted.
+ * @param setno The afioball number to get the location of.
+ * @return The absolute path to the afioball.
+ * @note The returned string points to static data that will be overwritten with each call.
+ * @ingroup stringGroup
+ */
+char *vfy_tball_fname(char *mountpoint, int setno)
+{
+	/*@ buffers ******************************************************* */
+	static char output[MAX_STR_LEN];
+
+	assert(bkpinfo != NULL);
+	assert_string_is_neither_NULL_nor_zerolength(mountpoint);
+	sprintf(output, "%s/archives/%d.star.%s", mountpoint, setno,
+			bkpinfo->zip_suffix);
+	if (!does_file_exist(output)) {
+		sprintf(output, "%s/archives/%d.afio.%s", mountpoint, setno,
+				bkpinfo->zip_suffix);
+	}
+	return (output);
+}
+
+
 /**
  * Generate a list of the files that have changed, based on @c afio @c -r
  * messages.
@@ -209,11 +146,12 @@ generate_list_of_changed_files(char *changedfiles_fname,
  * @param mountpoint The location the CD/DVD/ISO is mounted on.
  * @return The number of sets containing differences (0 for success).
  */
-int verify_afioballs_on_CD(struct s_bkpinfo *bkpinfo, char *mountpoint)
+int verify_afioballs_on_CD(char *mountpoint)
 {
 
 	/*@ buffers ********************************************************* */
 	char *tmp;
+	char *mds = NULL;
 
 	/*@ int ************************************************************* */
 	int set_number = 0;
@@ -228,10 +166,9 @@ int verify_afioballs_on_CD(struct s_bkpinfo *bkpinfo, char *mountpoint)
 	for (set_number = 0;
 		 set_number < 9999
 		 &&
-		 !does_file_exist(vfy_tball_fname
-						  (bkpinfo, mountpoint, set_number));
+		 !does_file_exist(vfy_tball_fname(mountpoint, set_number));
 		 set_number++);
-	if (!does_file_exist(vfy_tball_fname(bkpinfo, mountpoint, set_number))) {
+	if (!does_file_exist(vfy_tball_fname(mountpoint, set_number))) {
 		return (0);
 	}
 
@@ -246,18 +183,19 @@ int verify_afioballs_on_CD(struct s_bkpinfo *bkpinfo, char *mountpoint)
 			log_to_screen(tmp);
 		}
 	}
-	sprintf(tmp, "Verifying %s #%d's tarballs",
-			media_descriptor_string(bkpinfo->backup_media_type),
-			g_current_media_number);
+	mds = media_descriptor_string(bkpinfo->backup_media_type);
+	sprintf(tmp, "Verifying %s #%d's tarballs", mds, g_current_media_number);
+	mr_free(mds);
+
 	open_evalcall_form(tmp);
 
 	for (total_sets = set_number;
-		 does_file_exist(vfy_tball_fname(bkpinfo, mountpoint, total_sets));
+		 does_file_exist(vfy_tball_fname(mountpoint, total_sets));
 		 total_sets++) {
 		log_msg(1, "total_sets = %d", total_sets);
 	}
 	for (;
-		 does_file_exist(vfy_tball_fname(bkpinfo, mountpoint, set_number));
+		 does_file_exist(vfy_tball_fname(mountpoint, set_number));
 		 set_number++) {
 		percentage =
 			(set_number - g_last_afioball_number) * 100 / (total_sets -
@@ -265,9 +203,7 @@ int verify_afioballs_on_CD(struct s_bkpinfo *bkpinfo, char *mountpoint)
 		update_evalcall_form(percentage);
 		log_msg(1, "set = %d", set_number);
 		retval +=
-			verify_an_afioball_from_CD(bkpinfo,
-									   vfy_tball_fname(bkpinfo, mountpoint,
-													   set_number));
+			verify_an_afioball_from_CD(vfy_tball_fname(mountpoint, set_number));
 	}
 	g_last_afioball_number = set_number - 1;
 	close_evalcall_form();
@@ -286,7 +222,7 @@ int verify_afioballs_on_CD(struct s_bkpinfo *bkpinfo, char *mountpoint)
  * @param mtpt The mountpoint the CD/DVD/ISO is mounted on.
  * @return The number of differences (0 for perfect biggiefiles).
  */
-int verify_all_slices_on_CD(struct s_bkpinfo *bkpinfo, char *mtpt)
+int verify_all_slices_on_CD(char *mtpt)
 {
 
 	/*@ buffer ********************************************************** */
@@ -294,6 +230,7 @@ int verify_all_slices_on_CD(struct s_bkpinfo *bkpinfo, char *mtpt)
 	char *mountpoint;
 //  char ca, cb;
 	char *command;
+	char *mds = NULL;
 	char *sz_exe;
 	static char *bufblkA = NULL;
 	static char *bufblkB = NULL;
@@ -346,12 +283,13 @@ int verify_all_slices_on_CD(struct s_bkpinfo *bkpinfo, char *mtpt)
 		sz_exe[0] = '\0';
 	}
 
-	iamhere("before vsbf");
-	sprintf(tmp, "Verifying %s#%d's big files",
-			media_descriptor_string(bkpinfo->backup_media_type),
-			g_current_media_number);
+	log_it("before vsbf");
+	mds = media_descriptor_string(bkpinfo->backup_media_type);
+	sprintf(tmp, "Verifying %s#%d's big files", mds, g_current_media_number);
+	mr_free(mds);
+
 	open_evalcall_form(tmp);
-	iamhere("after vsbf");
+	log_it("after vsbf");
 	sprintf(mountpoint, "%s/archives", mtpt);
 	if (last_bigfile_num == -1) {
 		bigfile_num = 0;
@@ -394,10 +332,11 @@ int verify_all_slices_on_CD(struct s_bkpinfo *bkpinfo, char *mtpt)
 				log_to_screen("%s/%s not found on live filesystem",
 							bkpinfo->restore_path,
 							biggiestruct.filename);
-				asprintf(&tmp, "echo \"%s/%s not found\" >> /tmp/biggies.changed",
+				mr_asprintf(&tmp, "echo \"%s/%s not found\" >> %s/biggies.changed",
 							bkpinfo->restore_path,
-							biggiestruct.filename);
-				system(tmp);
+							biggiestruct.filename,
+							bkpinfo->tmpdir);
+				paranoid_system(tmp);
 				paranoid_free(tmp);
 
 				bigfile_num++;
@@ -499,13 +438,13 @@ int verify_all_slices_on_CD(struct s_bkpinfo *bkpinfo, char *mtpt)
  * @param tarball_fname The filename of the afioball to verify.
  * @return 0, always.
  */
-int verify_a_tarball(struct s_bkpinfo *bkpinfo, char *tarball_fname)
+int verify_a_tarball(char *tarball_fname)
 {
 	/*@ buffers ********************************************************* */
 	char *command;
 	char *outlog;
 	char *tmp;
-	//  char *p;
+	char *p;
 
 	/*@ pointers ******************************************************* */
 	FILE *pin;
@@ -554,7 +493,7 @@ int verify_a_tarball(struct s_bkpinfo *bkpinfo, char *tarball_fname)
 		bkpinfo->use_star = TRUE;
 		if (strstr(tarball_fname, ".bz2"))
 			sprintf(command,
-					"star -diff diffopts=mode,size,data file=%s %s >> %s 2>> %s",
+					"star -sparse -diff diffopts=mode,size,data file=%s %s >> %s 2>> %s",
 					tarball_fname,
 					(strstr(tarball_fname, ".bz2")) ? "-bz" : " ", outlog,
 					outlog);
@@ -572,8 +511,8 @@ int verify_a_tarball(struct s_bkpinfo *bkpinfo, char *tarball_fname)
 				(bkpinfo->use_star) ? 1 : 2, outlog);
 		pin = popen(command, "r");
 		if (pin) {
-			for (fgets(tmp, MAX_STR_LEN, pin); !feof(pin);
-				 fgets(tmp, MAX_STR_LEN, pin)) {
+			for (p = fgets(tmp, MAX_STR_LEN, pin); !feof(pin) && (p != NULL);
+				 p = fgets(tmp, MAX_STR_LEN, pin)) {
 				if (bkpinfo->use_star) {
 					if (!strstr(tmp, "diffopts=")) {
 						while (strlen(tmp) > 0
@@ -629,20 +568,19 @@ int verify_a_tarball(struct s_bkpinfo *bkpinfo, char *tarball_fname)
  * @see verify_an_afioball
  */
 int
-verify_an_afioball_from_CD(struct s_bkpinfo *bkpinfo, char *tarball_fname)
+verify_an_afioball_from_CD(char *tarball_fname)
 {
 
 	/*@ int ************************************************************* */
 	int res = 0;
 
-	assert(bkpinfo != NULL);
 	assert_string_is_neither_NULL_nor_zerolength(tarball_fname);
 
 	log_msg(1, "Verifying %s", tarball_fname);
 	if (!does_file_exist(tarball_fname)) {
 		fatal_error("Cannot verify nonexistent afioball");
 	}
-	res = verify_a_tarball(bkpinfo, tarball_fname);
+	res = verify_a_tarball(tarball_fname);
 	return (res);
 }
 
@@ -657,8 +595,7 @@ verify_an_afioball_from_CD(struct s_bkpinfo *bkpinfo, char *tarball_fname)
  * @see verify_an_afioball
  */
 int
-verify_an_afioball_from_stream(struct s_bkpinfo *bkpinfo, char *orig_fname,
-							   long long size)
+verify_an_afioball_from_stream(char *orig_fname, long long size)
 {
 
 	/*@ int ************************************************************** */
@@ -689,8 +626,8 @@ verify_an_afioball_from_stream(struct s_bkpinfo *bkpinfo, char *orig_fname,
 	sprintf(tmp, "Temporarily copying file from tape to '%s'",
 			tarball_fname);
 /*  log_it(tmp); */
-	read_file_from_stream_to_file(bkpinfo, tarball_fname, size);
-	res = verify_a_tarball(bkpinfo, tarball_fname);
+	read_file_from_stream_to_file(tarball_fname, size);
+	res = verify_a_tarball(tarball_fname);
 	if (res) {
 		sprintf(tmp,
 				"Afioball '%s' no longer matches your live filesystem", p);
@@ -712,8 +649,7 @@ verify_an_afioball_from_stream(struct s_bkpinfo *bkpinfo, char *orig_fname,
  * @return 0 for success (even if the file doesn't match); nonzero for a tape error.
  */
 int
-verify_a_biggiefile_from_stream(struct s_bkpinfo *bkpinfo,
-								char *biggie_fname, long long size)
+verify_a_biggiefile_from_stream(char *biggie_fname, long long size)
 {
 
 	/*@ int ************************************************************* */
@@ -763,7 +699,7 @@ verify_a_biggiefile_from_stream(struct s_bkpinfo *bkpinfo,
 		if (ctrl_chr != BLK_START_AN_AFIO_OR_SLICE) {
 			wrong_marker(BLK_START_AN_AFIO_OR_SLICE, ctrl_chr);
 		}
-		res = read_file_from_stream_to_file(bkpinfo, test_file, slice_siz);
+		res = read_file_from_stream_to_file(test_file, slice_siz);
 		unlink(test_file);
 		res =
 			read_header_block_from_stream(&slice_siz, slice_fnam,
@@ -785,9 +721,9 @@ verify_a_biggiefile_from_stream(struct s_bkpinfo *bkpinfo,
 			sprintf(tmp, "%s has changed on live filesystem",
 					biggie_fname);
 			log_to_screen(tmp);
-			sprintf(tmp, "echo \"%s\" >> /tmp/biggies.changed",
-					biggie_fname);
-			system(tmp);
+			sprintf(tmp, "echo \"%s\" >> %s/biggies.changed",
+					biggie_fname, bkpinfo->tmpdir);
+			paranoid_system(tmp);
 		}
 	}
 	paranoid_free(test_file);
@@ -807,7 +743,7 @@ verify_a_biggiefile_from_stream(struct s_bkpinfo *bkpinfo,
  *
  * @return 0 for success (even if there are differences); nonzero for a tape error.
  */
-int verify_afioballs_from_stream(struct s_bkpinfo *bkpinfo)
+int verify_afioballs_from_stream()
 {
 	/*@ int ********************************************************** */
 	int retval = 0;
@@ -840,18 +776,17 @@ int verify_afioballs_from_stream(struct s_bkpinfo *bkpinfo)
 			bkpinfo->tmpdir);
 	}
 	log_to_screen("Verifying regular archives on tape");
-	total_afioballs = get_last_filelist_number(bkpinfo) + 1;
+	total_afioballs = get_last_filelist_number() + 1;
 	open_progress_form("Verifying filesystem",
 					   "I am verifying archives against your live filesystem now.",
 					   "Please wait. This may take a couple of hours.", "",
 					   total_afioballs);
 	res = read_header_block_from_stream(&size, fname, &ctrl_chr);
 	if (ctrl_chr != BLK_START_AFIOBALLS) {
-		iamhere("YOU SHOULD NOT GET HERE");
-		iamhere("Grabbing the EXAT files");
+		log_it("YOU SHOULD NOT GET HERE");
+		log_it("Grabbing the EXAT files");
 		if (ctrl_chr == BLK_START_EXTENDED_ATTRIBUTES) {
-			res =
-				read_EXAT_files_from_tape(bkpinfo, &size, fname, &ctrl_chr,
+			res = read_EXAT_files_from_tape(&size, fname, &ctrl_chr,
 										  curr_xattr_list_fname,
 										  curr_acl_list_fname);
 		}
@@ -872,9 +807,8 @@ int verify_afioballs_from_stream(struct s_bkpinfo *bkpinfo)
 				bkpinfo->tmpdir, current_afioball_number);
 		}
 		if (ctrl_chr == BLK_START_EXTENDED_ATTRIBUTES) {
-			iamhere("Reading EXAT files from tape");
-			res =
-				read_EXAT_files_from_tape(bkpinfo, &size, fname, &ctrl_chr,
+			log_it("Reading EXAT files from tape");
+			res = read_EXAT_files_from_tape(&size, fname, &ctrl_chr,
 										  curr_xattr_list_fname,
 										  curr_acl_list_fname);
 		}
@@ -884,7 +818,7 @@ int verify_afioballs_from_stream(struct s_bkpinfo *bkpinfo)
 		sprintf(tmp, "Verifying fileset #%ld", current_afioball_number);
 		/*log_it(tmp); */
 		update_progress_form(tmp);
-		res = verify_an_afioball_from_stream(bkpinfo, fname, size);
+		res = verify_an_afioball_from_stream(fname, size);
 		if (res) {
 			sprintf(tmp, "Afioball %ld differs from live filesystem",
 					current_afioball_number);
@@ -915,7 +849,7 @@ int verify_afioballs_from_stream(struct s_bkpinfo *bkpinfo)
  *
  * @return 0 for success (even if there are differences); nonzero for a tape error.
  */
-int verify_biggiefiles_from_stream(struct s_bkpinfo *bkpinfo)
+int verify_biggiefiles_from_stream()
 {
 
 	/*@ int ************************************************************ */
@@ -962,9 +896,8 @@ int verify_biggiefiles_from_stream(struct s_bkpinfo *bkpinfo)
 	res = read_header_block_from_stream(&size, orig_fname, &ctrl_chr);
 	if (ctrl_chr != BLK_START_BIGGIEFILES) {
 		if (ctrl_chr == BLK_START_EXTENDED_ATTRIBUTES) {
-			iamhere("Grabbing the EXAT biggiefiles");
-			res =
-				read_EXAT_files_from_tape(bkpinfo, &size, orig_fname,
+			log_it("Grabbing the EXAT biggiefiles");
+			res = read_EXAT_files_from_tape(&size, orig_fname,
 										  &ctrl_chr, curr_xattr_list_fname,
 										  curr_acl_list_fname);
 		}
@@ -996,7 +929,7 @@ int verify_biggiefiles_from_stream(struct s_bkpinfo *bkpinfo)
 		update_progress_form(comment);
 		sprintf(logical_fname, "%s/%s", bkpinfo->restore_path, orig_fname);
 		res =
-			verify_a_biggiefile_from_stream(bkpinfo, logical_fname, size);
+			verify_a_biggiefile_from_stream(logical_fname, size);
 		retval += res;
 		current_biggiefile_number++;
 		g_current_progress++;
@@ -1015,20 +948,117 @@ int verify_biggiefiles_from_stream(struct s_bkpinfo *bkpinfo)
 
 
 /**
- * Verify the CD indicated by @c g_current_media_number.
+ * Verify the USB device
  * @param bkpinfo The backup information structure. Fields used:
  * - @c bkpinfo->isodir
- * - @c bkpinfo->prefix
- * - @c bkpinfo->manual_cd_tray
  * - @c bkpinfo->media_device
- * - @c bkpinfo->nfs_remote_dir
  * - @c bkpinfo->tmpdir
  * - @c bkpinfo->verify_data
  *
  * @return 0 for success (even if differences are found), nonzero for failure.
  * @ingroup verifyGroup
  */
-int verify_cd_image(struct s_bkpinfo *bkpinfo)
+int verify_usb_image()
+{
+
+	/*@ int ************************************************************ */
+	int retval = 0;
+
+	/*@ buffers ******************************************************** */
+	char *mountpoint = NULL;
+	char *tmp = NULL;
+	char *tmp1 = NULL;
+	char *fname = NULL;
+	int ret = 0;
+#ifdef __FreeBSD__
+	char mdd[32];
+	char *mddevice = mdd;
+	int vndev = 2;
+#else
+//skip
+#endif
+
+	assert(bkpinfo != NULL);
+
+	mr_asprintf(&fname, "%s1", bkpinfo->media_device);
+	if (is_this_device_mounted(fname)) {
+		log_msg(1, "USB device mounted. Remounting it at the right place");
+		mr_asprintf(&tmp, "umount %s", fname);
+		run_program_and_log_output(tmp, FALSE);
+		paranoid_free(tmp);
+	}
+	paranoid_free(fname);
+
+	log_msg(1, "Mounting USB device.");
+	mr_asprintf(&mountpoint, "%s/usb", bkpinfo->tmpdir);
+	mr_asprintf(&tmp, "mkdir -p %s", mountpoint);
+	run_program_and_log_output(tmp, FALSE);
+	paranoid_free(tmp);
+	/* Mindi always create one single parition on the USB dev */
+	mr_asprintf(&tmp, "mount %s1 %s", bkpinfo->media_device, mountpoint);
+	ret = run_program_and_log_output(tmp, FALSE);
+	paranoid_free(tmp);
+	if (ret) {
+		paranoid_free(mountpoint);
+		return(ret);
+	}
+
+	sync();
+	log_msg(2, "OK, I've mounted the USB Disk/Key\n");
+	mr_asprintf(&tmp, "%s/archives/NOT-THE-LAST", mountpoint);
+	if (!does_file_exist(tmp)) {
+		log_msg
+			(2,
+			 "This is the last USB device. I am therefore setting bkpinfo->verify_data to FALSE.");
+		bkpinfo->verify_data = FALSE;
+/*
+   (a) It's an easy way to tell the calling subroutine that we've finished &
+   there are no more device to be verified; (b) It stops the post-backup verifier
+   from running after the per-device verifier has run too.
+*/
+	}
+	paranoid_free(tmp);
+	verify_afioballs_on_CD(mountpoint);
+	log_it("before verify_all_slices");
+	verify_all_slices_on_CD(mountpoint);
+
+	mr_asprintf(&tmp1, "umount -d %s", mountpoint);
+#ifdef __FreeBSD__
+	ret += system(tmp1);
+	ret += kick_vn(mddevice);
+	if (ret)
+#else
+	if (system(tmp1))
+#endif
+	{
+		mr_asprintf(&tmp, "%s failed; unable to unmount USB device\n", tmp1);
+		log_to_screen(tmp);
+		paranoid_free(tmp);
+		retval++;
+	} else {
+		log_msg(2, "OK, I've unmounted the USB device\n");
+	}
+	paranoid_free(tmp1);
+	paranoid_free(mountpoint);
+	return (retval);
+}
+
+
+/**
+ * Verify the CD indicated by @c g_current_media_number.
+ * @param bkpinfo The backup information structure. Fields used:
+ * - @c bkpinfo->isodir
+ * - @c bkpinfo->prefix
+ * - @c bkpinfo->manual_cd_tray
+ * - @c bkpinfo->media_device
+ * - @c bkpinfo->netfs_remote_dir
+ * - @c bkpinfo->tmpdir
+ * - @c bkpinfo->verify_data
+ *
+ * @return 0 for success (even if differences are found), nonzero for failure.
+ * @ingroup verifyGroup
+ */
+int verify_cd_image()
 {
 
 	/*@ int ************************************************************ */
@@ -1056,7 +1086,7 @@ int verify_cd_image(struct s_bkpinfo *bkpinfo)
 	assert(bkpinfo != NULL);
 
 	sprintf(mountpoint, "%s/cdrom", bkpinfo->tmpdir);
-	sprintf(fname, "%s/%s/%s-%d.iso", bkpinfo->isodir, bkpinfo->nfs_remote_dir,
+	sprintf(fname, "%s/%s/%s-%d.iso", bkpinfo->isodir, bkpinfo->netfs_remote_dir,
 			bkpinfo->prefix, g_current_media_number);
 
 	mkdir(mountpoint, 1777);
@@ -1069,7 +1099,7 @@ int verify_cd_image(struct s_bkpinfo *bkpinfo)
 		if (bkpinfo->manual_cd_tray) {
 			popup_and_OK("Please push CD tray closed.");
 		}
-		if (find_and_mount_actual_cd(bkpinfo, mountpoint)) {
+		if (find_and_mount_actual_cd(mountpoint)) {
 			log_to_screen("failed to mount actual CD");
 			return (1);
 		}
@@ -1110,18 +1140,18 @@ int verify_cd_image(struct s_bkpinfo *bkpinfo)
    from running after the per-CD verifier has run too.
 */
 	}
-	verify_afioballs_on_CD(bkpinfo, mountpoint);
-	iamhere("before verify_all_slices");
-	verify_all_slices_on_CD(bkpinfo, mountpoint);
+	verify_afioballs_on_CD(mountpoint);
+	log_it("before verify_all_slices");
+	verify_all_slices_on_CD(mountpoint);
 
 #ifdef __FreeBSD__
 	ret = 0;
-	sprintf(command, "umount %s", mountpoint);
+	sprintf(command, "umount -d %s", mountpoint);
 	ret += system(command);
 	ret += kick_vn(mddevice);
 	if (ret)
 #else
-	sprintf(command, "umount %s", mountpoint);
+	sprintf(command, "umount -d %s", mountpoint);
 	if (system(command))
 #endif
 	{
@@ -1132,7 +1162,7 @@ int verify_cd_image(struct s_bkpinfo *bkpinfo)
 		log_msg(2, "OK, I've unmounted the ISO file\n");
 	}
 	if (!does_file_exist(fname)) {
-		sprintf(command, "umount %s", bkpinfo->media_device);
+		sprintf(command, "umount -d %s", bkpinfo->media_device);
 		run_program_and_log_output(command, 2);
 		if (!bkpinfo->please_dont_eject
 			&& eject_device(bkpinfo->media_device)) {
@@ -1153,7 +1183,7 @@ int verify_cd_image(struct s_bkpinfo *bkpinfo)
  * @return 0 for success (even if thee were differences), nonzero for failure.
  * @ingroup verifyGroup
  */
-int verify_tape_backups(struct s_bkpinfo *bkpinfo)
+int verify_tape_backups()
 {
 
 	/*@ int ************************************************************ */
@@ -1170,20 +1200,19 @@ int verify_tape_backups(struct s_bkpinfo *bkpinfo)
 
 	log_msg(3, "verify_tape_backups --- starting");
 	log_to_screen("Verifying backups");
-	openin_tape(bkpinfo);
+	openin_tape();
 /* verify archives themselves */
-	retval += verify_afioballs_from_stream(bkpinfo);
-	retval += verify_biggiefiles_from_stream(bkpinfo);
+	retval += verify_afioballs_from_stream();
+	retval += verify_biggiefiles_from_stream();
 /* find the final blocks */
 	paranoid_system("sync");
 	sleep(2);
-	closein_tape(bkpinfo);
+	closein_tape();
 /* close tape; exit */
 //  fclose(g_tape_stream); <-- not needed; is handled by closein_tape()
-	paranoid_system
-		("rm -f /tmp/biggies.changed /tmp/changed.files.[0-9]* 2> /dev/null");
-	sprintf(changed_files_fname, "/tmp/changed.files.%d",
-			(int) (random() % 32767));
+	sprintf(tmp, "rm -f %s/biggies.changed %s/changed.files 2> /dev/null", bkpinfo->tmpdir, bkpinfo->tmpdir);
+	paranoid_system(tmp);
+	sprintf(changed_files_fname, "%s/changed.files", bkpinfo->tmpdir);
 	sprintf(tmp,
 			"grep -E '^%s:.*$' %s | cut -d'\"' -f2 | sort -u | awk '{print \"/\"$0;};' | tr -s '/' '/' | grep -v \"(total of\" | grep -v \"incheckentry.*xwait\" | grep -vE '^/afio:.*$' | grep -vE '^dev/.*$'  > %s",
 			(bkpinfo->use_star) ? "star" : "afio", MONDO_LOGFILE, 
@@ -1200,22 +1229,20 @@ int verify_tape_backups(struct s_bkpinfo *bkpinfo)
 				("No differences found. Therefore, no 'changed.files' text file.");
 		}
 	}
-	sprintf(tmp, "cat /tmp/biggies.changed >> %s", changed_files_fname);
+	sprintf(tmp, "cat %s/biggies.changed >> %s", bkpinfo->tmpdir, changed_files_fname);
 	paranoid_system(tmp);
 
 	diffs = count_lines_in_file(changed_files_fname);
 	if (diffs > 0) {
-		sprintf(tmp, "cp -f %s %s", changed_files_fname,
-				"/tmp/changed.files");
+		sprintf(tmp, "cp -f %s %s/changed.files", changed_files_fname,
+				MONDO_CACHE);
 		run_program_and_log_output(tmp, FALSE);
 		sprintf(tmp,
-				"%ld files differed from live filesystem; type less %s or less %s to see",
-				diffs, changed_files_fname, "/tmp/changed.files");
+				"%ld files differed from live filesystem; type less %s or less %s/changed.files to see",
+				diffs, changed_files_fname, MONDO_CACHE);
 		log_msg(0, tmp);
-		log_to_screen
-			("See /tmp/changed.files for a list of nonmatching files.");
-		log_to_screen
-			("The files probably changed on filesystem, not on backup media.");
+		log_to_screen("See "MONDO_CACHE"/changed.files for a list of nonmatching files.");
+		log_to_screen("The files probably changed on filesystem, not on backup media.");
 		//      retval++;
 	}
 	return (retval);
@@ -1223,28 +1250,3 @@ int verify_tape_backups(struct s_bkpinfo *bkpinfo)
 
 
 
-/**
- * Generate the filename of a tarball to verify.
- * @param bkpinfo The backup information structure. @c bkpinfo->zip_suffix is the only field used.
- * @param mountpoint The directory where the CD/DVD/ISO is mounted.
- * @param setno The afioball number to get the location of.
- * @return The absolute path to the afioball.
- * @note The returned string points to static data that will be overwritten with each call.
- * @ingroup stringGroup
- */
-char *vfy_tball_fname(struct s_bkpinfo *bkpinfo, char *mountpoint,
-					  int setno)
-{
-	/*@ buffers ******************************************************* */
-	static char output[MAX_STR_LEN];
-
-	assert(bkpinfo != NULL);
-	assert_string_is_neither_NULL_nor_zerolength(mountpoint);
-	sprintf(output, "%s/archives/%d.star.%s", mountpoint, setno,
-			bkpinfo->zip_suffix);
-	if (!does_file_exist(output)) {
-		sprintf(output, "%s/archives/%d.afio.%s", mountpoint, setno,
-				bkpinfo->zip_suffix);
-	}
-	return (output);
-}
