@@ -4,7 +4,7 @@
     begin                : Fri Apr 19 2002
     copyright            : (C) 2002 by Stan Benoit
     email                : troff@nakedsoul.org
-    cvsid                : $Id: mondostructures.h 1295 2007-04-11 18:59:49Z bruno $
+    cvsid                : $Id: mondostructures.h 3150 2013-06-21 04:48:23Z bruno $
  ***************************************************************************/
 
 /***************************************************************************
@@ -23,13 +23,6 @@
  */
 
 
-/** @def MAX_NOOF_MEDIA The maximum number of media that can be used in any one backup. */
-
-///* So we can override it in config.h: */
-//#ifndef MAX_NOOF_MEDIA
-#define MAX_NOOF_MEDIA 50
-//#endif
-
 /**
  * Structure indicating one entry in the mountlist.
  * There is one mountlist_line for each device we're keeping track of in the mountlist.
@@ -58,14 +51,10 @@ struct mountlist_line {
 	long long size;
 
   /**
-   * For ext2 and ext3, this is the filesystem label (if there is one). If not, this should be "".
+   * For ext2 and ext3, this is the filesystem label or uuid (if there is one). If not, this should be "".
    */
 	char label[256];
 
-	/**
-	 * This is the filesystem UUID (if there is one). If not, this should be "".
-	 */
-	char uuid[64];
 };
 
 /**
@@ -82,7 +71,7 @@ struct mountlist_itself {
   /**
    * The list of entries, all @p entries of them.
    */
-	struct mountlist_line el[MAX_TAPECATALOG_ENTRIES];
+	struct mountlist_line el[MAX_MOUNTLIST_ENTRIES];
 };
 
 /**
@@ -154,12 +143,12 @@ struct s_disk {
    * Index number of this entry in the whole disklist.
    */
 	int index;
-	
+
   /**
    * Type of disk.
    */
 	char type;  // ' ' = data (default), S = spare, F = faulty
-	
+
 };
 
 /**
@@ -186,19 +175,33 @@ typedef enum { none = 0,		///< No type has been set yet.
 	cdrw,						///< Back up to CD-RWs and blank them first.
 	dvd,						///< Back up to DVD+R[W] or DVD-R[W] disks.
 	cdstream,					///< Back up to recordable CDs but treat them like a tape streamer.
-	nfs,						///< Back up to an NFS mount on the local subnet.
+	netfs,						///< Back up to an NETFS mount on the local subnet.
 	tape,						///< Back up to tapes.
+	usb,						///< Back up to USB devices.
 	udev						///< Back up to another unsupported device; just send a stream of bytes.
 } t_bkptype;
 
 /**
  * A type of file in the catalog of recent archives.
  */
-typedef enum { other,			///< Some other kind of file.
+typedef enum { 
+	other,						///< Some other kind of file.
 	fileset,					///< An afioball (fileset), optionally compressed.
 	biggieslice					///< A slice of a biggiefile, optionally compressed.
 } t_archtype;
 
+/**
+ * A type of file in the catalog of recent archives.
+ */
+typedef enum {
+	nuke = 0,					/// Nuke mode
+	interactive,					/// Interactive mode
+	compare,					/// Compare mode
+	mbr,						/// MBR mode
+	isoonly,					/// ISO mode
+	isonuke,					/// ISO+Nuke mode
+} t_restore_mode;
+
 
 #ifdef __FreeBSD__
 
@@ -251,7 +254,7 @@ struct raid_device_record {
 	   * The chunk size of this RAID device.
 	   */
 	int chunk_size;
-	
+
 	  /**
 	   * The parity algorithm of this RAID device. (RAID5 only)
 	   */
@@ -327,19 +330,16 @@ struct s_bkpinfo {
   /**
    * The device we're backing up to.
    * If backup_media_type is @b cdr, @b cdrw, or @b cdstream, this should be the SCSI node (e.g. 0,1,0).
-   * If backup_media_type is @b dvd, @b tape, or @b udev, this should be a /dev entry.
+   * If backup_media_type is @b dvd, @b tape, @b usb or @b udev, this should be a /dev entry.
    * If backup_media_type is anything else, this should be blank.
    */
 	char media_device[MAX_STR_LEN / 4];
 
   /**
-   * An array containing the sizes of each media in our backup set, in MB.
-   * For example, media 1's size would be stored in media_size[1].
-   * Element 0 is unused.
+   * A field containing the sizes of the media in our backup set, in MB.
    * If the size should be autodetected, make it -1 (preferable) or 0.
-   * @bug This should probably be only one variable, not an array.
    */
-	long media_size[MAX_NOOF_MEDIA + 1];
+	long media_size;
 
   /**
    * The boot loader that is installed. Available choices are:
@@ -512,7 +512,13 @@ struct s_bkpinfo {
    * Multiple directories should be separated by spaces. /tmp, /proc,
    * the scratchdir, and the tempdir are automatically excluded.
    */
-	char exclude_paths[MAX_STR_LEN*4];
+	char *exclude_paths;
+
+  /**
+   * Devices to NOT back up.
+   * Multiple devices should be separated by spaces.
+   */
+	char *exclude_devs;
 
   /**
    * The path to restore files relative to during a restore.
@@ -548,18 +554,33 @@ struct s_bkpinfo {
 	char kernel_path[MAX_STR_LEN];
 
   /**
-   * The NFS mount to back up to/restore from.
-   * If backup_media_type is not @b nfs, this is ignored.
+   * The NETFS mount to back up to/restore from.
+   * If backup_media_type is not @b netfs, this is ignored.
    * It must contain a colon, and the server's address should be in dotted-decimal IP
    * address form. (Domain names will be resolved in post_param_configuration().)
    */
-	char nfs_mount[MAX_STR_LEN];
+	char netfs_mount[MAX_STR_LEN];
 
   /**
-   * The directory, relative to the root of @p nfs_mount, to put
+   * The directory, relative to the root of @p netfs_mount, to put
    * the backups in.
    */
-	char nfs_remote_dir[MAX_STR_LEN];
+	char netfs_remote_dir[MAX_STR_LEN];
+
+  /**
+   * The potential user to use for NETFS backup
+   */
+	char *netfs_user;
+
+  /**
+   * The potential subdirectory under which are located ISO images on HDD (restore mode only)
+   */
+	char *subdir;
+
+  /**
+   * The protocol to use for Network backup (NFS, SSHFS, ...)
+   */
+	char *netfs_proto;
 
   /**
    * A tarball containing a program called "usr/bin/post-nuke" that will be run
@@ -573,13 +594,13 @@ struct s_bkpinfo {
    */
 	bool wipe_media_first;
 
-// patch by Herman Kuster  
+// patch by Herman Kuster
   /**
    * The differential level of this backup. Currently only 0 (full backup) and 1
    * (files changed since last full backup) are supported.
    */
 	int differential;
-// end patch  
+// end patch
 
   /**
    * If TRUE, then don't eject media when backing up or restoring.
@@ -608,6 +629,16 @@ struct s_bkpinfo {
    * If TRUE, make the bootable CD use LILO/ELILO. If FALSE, use isolinux (the default).
    */
 	bool make_cd_use_lilo;
+
+  /**
+   * If TRUE, make the the tape bootable. If FALSE, normal tape, the default
+   */
+	bool use_obdr;
+
+  /**
+   * Nature of the restore
+   */
+	t_restore_mode restore_mode;
 };
 
 
