@@ -1,100 +1,8 @@
 /* newt-specific.c
-   $Id: newt-specific.c 1463 2007-05-29 12:48:16Z bruno $
+   $Id: newt-specific.c 3154 2013-06-22 22:34:33Z bruno $
 
   subroutines which do display-type things
   and use the newt library to do them
-.
-
-10/02
-- tell user not to bother mailing list unless snapshot
-
-07/09
-- finish() calls g_erase_tmpdir_and_scratchdir to erase
-  tmpdir and scratchdir at end
-
-06/25
-- tried to fix newtFinished() segfault
-
-04/13/2004
-- lots of char[]'s are now *char's
-
-10/09/2003
-- allow DVD option for text-mode which_backup_type() too
-
-09/28
-- log_to_screen() now takes printf-like parameters
-
-09/26
-- b5 now means dvd (was udev)
-
-09/16
-- echo fatal error to screen before exiting
-- working on fatal_error()
-- swapped g_blurb2 and g_blurb3 in a few places to fix a GUI problem;
-  it shouldn't have worked! :) I think there's a mem-leak somewhere
-
-09/13
-- changed some '\n' occurrences to '\r\n'
-
-09/09
-- fixed silly bug in log_to_screen()
-
-07/02
-- modified popup_and_get_string()
-
-05/03
-- cleaned up fatal_error()
-
-04/30
-- fixed popup_changelist_*()
-
-04/27
-- replace newtFinished() and newtInit() with
-  newtSuspend() and newtResume()
-
-04/25
-- after comparing, popup the changelist for the user
-- moved the relevant code from compare_to_cds() into
-  compare_mode(), so that tape users may benefit too
-
-04/24
-- added lots of assert()'s and log_OS_error()'s
-- cleaned up a few uninitialized strings (valgrind)
-
-04/22
-- line 1181 - 'read from' (not 'restoring from')
-
-03/15/2003
-- fixed potentially infinite loop in log_to_screen (Tom Mortell)
-
-12/04/2002
-- changed "Pick file" to "Non-matching files"
-
-11/28
-- finish() now unregisters Mondo's pid
-
-10/28
-- unmount tmpfs if fatal_error
-
-10/04
-- more XMondo-related work
-- added some =NULL's in a few places
-
-09/01 - 09/30
-- write log_to_screen()'s string to stdout if in text mode
-- update_progress_form_FULL(), update_evalcall_form(),
-  popup_and_*(), ask_me_yes_or_no() now write
-  XMondo-friendly info to stdout if in text mode
-- if fatal error then popup and say what the fatal error is
-- run_program_and_log_output() now takes boolean operator to specify
-  whether it will log its activities in the event of _success_
-- added popup_changelist_from_file(char*source_file, char*topic, void*call_if_entry_selected(void*));
-
-08/07
-- added some functions to let user choose backup dev + format
-
-07/27
-- created
 */
 
 
@@ -110,6 +18,7 @@ extern "C" {
 #endif
 
 #include "my-stuff.h"
+#include "mr_mem.h"
 #include "mondostructures.h"
 #include "newt-specific.h"
 #include "libmondo-string-EXT.h"
@@ -122,13 +31,16 @@ extern "C" {
 
 extern char *MONDO_LOGFILE;
 
+extern struct s_bkpinfo *bkpinfo;
+
 /*@unused@*/
-//static char cvsid[] = "$Id: newt-specific.c 1463 2007-05-29 12:48:16Z bruno $";
+//static char cvsid[] = "$Id: newt-specific.c 3154 2013-06-22 22:34:33Z bruno $";
 
 	extern pid_t g_mastermind_pid;
+	/*
 	extern char *g_tmpfs_mountpt;
+	*/
 	extern char *g_boot_mountpt;
-	extern char *g_mondo_home;
 	extern char *ps_options;
 	extern char *ps_proc_id;
 
@@ -143,6 +55,9 @@ extern char *MONDO_LOGFILE;
  */
 	bool g_exiting = FALSE;
 
+// Decide whether we should continue to ask questions or exit (cron use case)
+extern bool g_fail_immediately;
+
 /**
  * Padding below the Newt components, to overcome bugs in Newt.
  */
@@ -188,8 +103,7 @@ extern char *MONDO_LOGFILE;
 	char g_xmondo_stdin[MAX_NEWT_COMMENT_LEN],	///< ... @bug Unneeded w/current XMondo.
 	 g_xmondo_stdout[MAX_NEWT_COMMENT_LEN];	///< .... @bug Unneeded w/current XMondo.
 	bool g_called_by_xmondo = FALSE;	///< @bug Unneeded w/current XMondo.
-	char *g_erase_tmpdir_and_scratchdir;	///< The command to run to erase the tmpdir and scratchdir at the end of Mondo.
-	char *g_selfmounted_isodir;	///< Holds the NFS mountpoint if mounted via mondoarchive.
+	char *g_selfmounted_isodir;	///< Holds the NETFS mountpoint if mounted via mondoarchive.
 
 /* @} - end of globalGroup */
 
@@ -210,19 +124,30 @@ extern char *MONDO_LOGFILE;
 	 bool ask_me_yes_or_no(char *prompt) {
 
 		/*@ buffers ********************************************************** */
-		char *tmp;
 		int i;
+		char *p;
 
-		 tmp = malloc(MAX_NEWT_COMMENT_LEN);
-		 assert_string_is_neither_NULL_nor_zerolength(prompt);
+		assert_string_is_neither_NULL_nor_zerolength(prompt);
 
+		if (g_fail_immediately) {
+			// We consider the user aborted by using the -F option
+			log_msg(3, "Exiting at first interaction request due to -F");
+			finish(1);
+		}
 		if (g_text_mode) {
+
+			char *tmp;
+		 	tmp = malloc(MAX_NEWT_COMMENT_LEN);
+
 			while (1) {
-				system("sync");
+				paranoid_system("sync");
 				printf
-					("---promptdialogYN---1--- %s\r\n---promptdialogYN---Q--- [yes] [no] ---\r\n--> ",
+					("---promptdialogYN---1--- %s\n---promptdialogYN---Q--- [yes] [no] ---\n--> ",
 					 prompt);
-				(void) fgets(tmp, MAX_NEWT_COMMENT_LEN, stdin);
+				p= fgets(tmp, MAX_NEWT_COMMENT_LEN, stdin);
+				if (!p) {
+					//FIXME
+				}
 				if (tmp[strlen(tmp) - 1] == '\n')
 					tmp[strlen(tmp) - 1] = '\0';
 
@@ -237,13 +162,12 @@ extern char *MONDO_LOGFILE;
 					paranoid_free(tmp);
 					return (FALSE);
 				} else {
-					system("sync");
+					paranoid_system("sync");
 					printf
 						("Please enter either YES or NO (or yes or no, or y or n, or...)\n");
 				}
 			}
 		} else {
-			paranoid_free(tmp);
 			return (popup_with_buttons(prompt, "Yes", "No"));
 		}
 	}
@@ -259,16 +183,25 @@ extern char *MONDO_LOGFILE;
 
 		/*@ buffer *********************************************************** */
 		char *tmp;
+		char *p;
 		int i;
 
-		tmp = malloc(MAX_NEWT_COMMENT_LEN);
 		assert_string_is_neither_NULL_nor_zerolength(prompt);
+		if (g_fail_immediately) {
+			// We consider the user aborted by using the -F option
+			log_msg(3, "Exiting at first interaction request due to -F");
+			finish(1);
+		}
+		tmp = malloc(MAX_NEWT_COMMENT_LEN);
 		if (g_text_mode) {
-			system("sync");
+			paranoid_system("sync");
 			printf
-				("---promptdialogOKC---1--- %s\r\n---promptdialogOKC---Q--- [OK] [Cancel] ---\r\n--> ",
+				("---promptdialogOKC---1--- %s\n---promptdialogOKC---Q--- [OK] [Cancel] ---\n--> ",
 				 prompt);
-			(void) fgets(tmp, MAX_NEWT_COMMENT_LEN, stdin);
+			p = fgets(tmp, MAX_NEWT_COMMENT_LEN, stdin);
+			if (!p) {
+				// FIXME
+			}
 			if (tmp[strlen(tmp) - 1] == '\n')
 				tmp[strlen(tmp) - 1] = '\0';
 
@@ -311,10 +244,15 @@ extern char *MONDO_LOGFILE;
 		newtPopHelpLine();
 		newtFormDestroy(g_isoform_main);
 		newtPopWindow();
+
+		/* Reset globals */
 		g_isoform_main = NULL;
 		g_isoform_old_progress = -1;
-	}
-
+		g_isoform_header = NULL;
+		g_isoform_scale = NULL;
+		g_isoform_timeline = NULL;
+		g_isoform_pcline = NULL;
+}
 
 /**
  * Close the currently opened progress form.
@@ -339,8 +277,14 @@ extern char *MONDO_LOGFILE;
 		newtPopHelpLine();
 		newtFormDestroy(g_progressForm);
 		newtPopWindow();
+
+		/* Reset globals */
 		g_progressForm = NULL;
 		g_current_progress = -999;
+		g_timeline = NULL;
+		g_percentline = NULL;
+		g_scale = NULL;
+
 	}
 
 /**
@@ -349,9 +293,15 @@ extern char *MONDO_LOGFILE;
 void kill_anything_like_this(char *str) {
 
 char *tmp = NULL;
+char *tmp1 = NULL;
 
-	asprintf(&tmp,"kill `ps %s | grep \" %s \" | awk '{print %s;}' | grep -v \"grep\"`", ps_options, str , ps_proc_id);
+	mr_asprintf(&tmp,"ps %s | grep \" %s \" | grep -v \"grep\" | awk '{print %s;}'", ps_options, str , ps_proc_id);
 	run_program_and_log_output(tmp, TRUE);
+	if (strlen(tmp) > 0) {
+		mr_asprintf(&tmp1,"kill `%s`", tmp);
+		run_program_and_log_output(tmp1, TRUE);
+		paranoid_free(tmp1);
+	}
 	paranoid_free(tmp);
 }
 
@@ -369,7 +319,6 @@ char *tmp = NULL;
 		char *tmp;
 		char *command;
 		static bool already_exiting = FALSE;
-		int i;
 
 		/*@ end vars **************************************************** */
 
@@ -406,23 +355,23 @@ char *tmp = NULL;
 		log_msg(2, "I'm going to do some cleaning up now.");
 		paranoid_system("killall mindi 2> /dev/null");
 		kill_anything_like_this("/mondo/do-not");
-		kill_anything_like_this("tmp.mondo");
+		kill_anything_like_this("mondo.tmp");
 		kill_anything_like_this("ntfsclone");
 		sync();
+		/*
 		sprintf(tmp, "umount %s", g_tmpfs_mountpt);
-		chdir("/");
 		for (i = 0; i < 10 && run_program_and_log_output(tmp, 5); i++) {
 			log_msg(2, "Waiting for child processes to terminate");
 			sleep(1);
 			run_program_and_log_output(tmp, 5);
 		}
+		*/
 
-		if (g_erase_tmpdir_and_scratchdir[0]) {
-			run_program_and_log_output(g_erase_tmpdir_and_scratchdir, 5);
+		if (chdir("/")) {
+			// FIXME
 		}
-
 		if (g_selfmounted_isodir) {
-			sprintf(command, "umount %s", g_selfmounted_isodir);
+			sprintf(command, "umount -d %s", g_selfmounted_isodir);
 			run_program_and_log_output(command, 5);
 			sprintf(command, "rmdir %s", g_selfmounted_isodir);
 			run_program_and_log_output(command, 5);
@@ -436,20 +385,13 @@ char *tmp = NULL;
 		}
 
 		printf("---FATALERROR--- %s\n", error_string);
-		sprintf(command, "gzip -9c %s > /tmp/MA.log.gz 2> /dev/null", MONDO_LOGFILE);
-		system(command);
 		printf
 				("If you require technical support, please contact the mailing list.\n");
 		printf("See http://www.mondorescue.org for details.\n");
 		printf
 				("The list's members can help you, if you attach that file to your e-mail.\n");
 		printf("Log file: %s\n", MONDO_LOGFILE);
-		if (does_file_exist("/tmp/MA.log.gz")) {
-			printf
-				("FYI, I have gzipped the log and saved it to /tmp/MA.log.gz\n");
-		}
 		printf("Mondo has aborted.\n");
-		register_pid(0, "mondo");	// finish() does this too, FYI
 		if (!g_main_pid) {
 			log_msg(3, "FYI - g_main_pid is blank");
 		}
@@ -458,6 +400,17 @@ char *tmp = NULL;
 		finish(254);
 	}
 
+/* Free memory allocated for newt */
+void free_newt_stuff() {
+
+int i;
+
+for (i = 0; i < g_noof_log_lines; i++) {
+	mr_free(err_log_lines[i]);
+}
+mr_free(err_log_lines);
+}
+
 
 
 /**
@@ -468,30 +421,25 @@ char *tmp = NULL;
  */
 	void
 	 finish(int signal) {
-		char *command;
-		malloc_string(command);
+		char *command = NULL;
 
 		/*  if (signal==0) { popup_and_OK("Please press <enter> to quit."); } */
 
 		/* newtPopHelpLine(); */
 
-		register_pid(0, "mondo");
-		chdir("/");
-		run_program_and_log_output("umount " MNT_CDROM, FALSE);
-		run_program_and_log_output("rm -Rf /mondo.scratch.* /tmp.mondo.*",
-								   FALSE);
-		if (g_erase_tmpdir_and_scratchdir) {
-			run_program_and_log_output(g_erase_tmpdir_and_scratchdir, 1);
+		/*  Before removing dir, make sure we're out of them */
+		if (chdir("/tmp")) {
+			// FIXME
 		}
+		run_program_and_log_output("umount -d " MNT_CDROM, FALSE);
 		if (g_selfmounted_isodir) {
-			sprintf(command, "umount %s", g_selfmounted_isodir);
+			mr_asprintf(&command, "umount -d %s", g_selfmounted_isodir);
 			run_program_and_log_output(command, 1);
-			sprintf(command, "rmdir %s", g_selfmounted_isodir);
+			paranoid_free(command);
+			mr_asprintf(&command, "rmdir %s", g_selfmounted_isodir);
 			run_program_and_log_output(command, 1);
+			paranoid_free(command);
 		}
-//  iamhere("foo");
-		/* system("clear"); */
-//  iamhere("About to call newtFinished");
 		if (!g_text_mode) {
 			if (does_file_exist("/THIS-IS-A-RAMDISK")) {
 				log_msg(1, "Calling newtFinished()");
@@ -501,11 +449,37 @@ char *tmp = NULL;
 				newtSuspend();
 			}
 		}
-//  system("clear");
-//  iamhere("Finished calling newtFinished");
 		printf("Execution run ended; result=%d\n", signal);
 		printf("Type 'less %s' to see the output log\n", MONDO_LOGFILE);
+		if (bkpinfo->tmpdir) {
+			log_msg(8,"tempdir is %s",bkpinfo->tmpdir);
+			if (strstr(bkpinfo->tmpdir ,"mondo.tmp.") != NULL) {
+				log_msg(8,"erasing tempdir %s",bkpinfo->tmpdir);
+				mr_asprintf(&command, "rm -Rf %s", bkpinfo->tmpdir);
+				paranoid_system(command);
+				mr_free(command);
+			}
+		}
+		if (bkpinfo->scratchdir) {
+			log_msg(8,"scratchdir is %s", bkpinfo->scratchdir);
+			if (strstr(bkpinfo->scratchdir ,"mondo.scratch.") != NULL) {
+				log_msg(8,"erasing scratchdir %s",bkpinfo->scratchdir);
+				mr_asprintf(&command, "rm -Rf %s", bkpinfo->scratchdir);
+				paranoid_system(command);
+				mr_free(command);
+			}
+		}
+		/* Free all allocated strings in bkpinfo */
+		mr_free(bkpinfo->netfs_user);
+		mr_free(bkpinfo->netfs_proto);
+		mr_free(bkpinfo->exclude_devs);
+		mr_free(bkpinfo->exclude_paths);
+		mr_free(bkpinfo->subdir);
+
+		/* Then free the structure */
+		paranoid_free(bkpinfo);
 		free_libmondo_global_strings();
+		free_newt_stuff();
 		exit(signal);
 	}
 
@@ -523,8 +497,9 @@ char *tmp = NULL;
 	 log_file_end_to_screen(char *filename, char *grep_for_me) {
 
 		/*@ buffers ********************************************************** */
-		char *command;
-		char *tmp;
+		char *command = NULL;
+		char *tmp = NULL;
+		char *p = NULL;
 
 		/*@ pointers ********************************************************* */
 		FILE *fin;
@@ -532,21 +507,17 @@ char *tmp = NULL;
 		/*@ int ************************************************************** */
 		int i = 0;
 
-		malloc_string(command);
-		malloc_string(tmp);
 		assert_string_is_neither_NULL_nor_zerolength(filename);
 		assert(grep_for_me != NULL);
 
 		if (!does_file_exist(filename)) {
-			paranoid_free(command);
-			paranoid_free(tmp);
 			return;
 		}
 		if (grep_for_me[0] != '\0') {
-			sprintf(command, "grep '%s' %s | tail -n%d",
+			mr_asprintf(&command, "grep '%s' %s | tail -n%d",
 					grep_for_me, filename, g_noof_log_lines);
 		} else {
-			sprintf(command, "tail -n%d %s", g_noof_log_lines,
+			mr_asprintf(&command, "tail -n%d %s", g_noof_log_lines,
 					filename);
 		}
 		fin = popen(command, "r");
@@ -556,14 +527,17 @@ char *tmp = NULL;
 			for (i = 0; i < g_noof_log_lines; i++) {
 				for (err_log_lines[i][0] = '\0';
 					 strlen(err_log_lines[i]) < 2 && !feof(fin);) {
-					(void) fgets(err_log_lines[i], MAX_NEWT_COMMENT_LEN,
-								 fin);
+					p = fgets(err_log_lines[i], MAX_NEWT_COMMENT_LEN, fin);
+					if (!p) {
+						// FIXME
+					}
 					/* Commented to make valgrind happy and avoid crash
 					strip_spaces(err_log_lines[i]);
 					*/
 					if (!strncmp(err_log_lines[i], "root:", 5)) {
-						strcpy(tmp, err_log_lines[i] + 6);
-						strcpy(err_log_lines[i], tmp);
+						mr_asprintf(&tmp, "%s", err_log_lines[i] + 6);
+						strncpy(err_log_lines[i], tmp, (size_t)MAX_NEWT_COMMENT_LEN);
+						mr_free(tmp);
 					}
 					if (feof(fin)) {
 						break;
@@ -572,9 +546,8 @@ char *tmp = NULL;
 			}
 			paranoid_pclose(fin);
 		}
+		mr_free(command);
 		refresh_log_screen();
-		paranoid_free(command);
-		paranoid_free(tmp);
 	}
 
 
@@ -597,7 +570,7 @@ char *tmp = NULL;
 		malloc_string(output);
 
 		va_start(args, fmt);
-		vsprintf(output, fmt, args);
+		vsnprintf(output, MAX_STR_LEN-1, fmt, args);
 		log_msg(0, output);
 		output[80] = '\0';
 		va_end(args);
@@ -693,11 +666,12 @@ char *tmp = NULL;
 			//      newtOpenWindow (20, 6, 40, 7, title);      // "Please Wait");
 			newtCenteredWindow(40, 7, title);
 			g_isoform_main = newtForm(NULL, NULL, 0);
-			g_isoform_timeline = newtLabel(1, 5, "This is the timeline");
-			g_isoform_pcline = newtLabel(1, 6, "This is the pcline");
+			g_isoform_timeline = newtLabel(1, 5, "                    ");
+			g_isoform_pcline = newtLabel(1, 6, "                  ");
 			newtFormAddComponents(g_isoform_main, g_isoform_timeline,
 								  g_isoform_pcline, g_isoform_header,
 								  g_isoform_scale, NULL);
+			newtDrawForm(g_isoform_main);
 			newtRefresh();
 		}
 		update_evalcall_form(0);
@@ -766,12 +740,13 @@ char *tmp = NULL;
 			newtCenteredWindow(60, 11, title);
 			g_scale = newtScale(3, 6, 54, g_maximum_progress);
 			g_progressForm = newtForm(NULL, NULL, 0);
-			g_percentline = newtLabel(10, 9, "This is the percentline");
-			g_timeline = newtLabel(10, 8, "This is the timeline");
+			g_percentline = newtLabel(10, 9, "                       ");
+			g_timeline = newtLabel(10, 8, "                    ");
 			newtFormAddComponents(g_progressForm, g_percentline,
 								  g_timeline, g_scale, g_blurb1, g_blurb3,
 								  g_blurb2, NULL);
 			newtPushHelpLine(b1c);
+			newtDrawForm(g_progressForm);
 			newtRefresh();
 		}
 		update_progress_form_full(blurb1, blurb2, blurb3);
@@ -794,7 +769,7 @@ char *tmp = NULL;
 		log_msg(0, prompt);
 		if (g_text_mode) {
 			printf
-				("---promptpopup---1--- %s\r\n---promptpopup---Q--- [OK] ---\r\n--> ",
+				("---promptpopup---1--- %s\n---promptpopup---Q--- [OK] ---\n--> ",
 				 prompt);
 			while (((ch = getchar()) != '\n') && (ch != EOF));
 		} else {
@@ -826,6 +801,7 @@ char *tmp = NULL;
 
 		/*@ buffers ********************************************************** */
 		char *blurb;
+		char *p;
 		char *original_contents;
 
 		blurb = malloc(MAX_NEWT_COMMENT_LEN);
@@ -836,9 +812,12 @@ char *tmp = NULL;
 
 		if (g_text_mode) {
 			printf
-				("---promptstring---1--- %s\r\n---promptstring---2--- %s\r\n---promptstring---Q---\r\n-->  ",
+				("---promptstring---1--- %s\n---promptstring---2--- %s\n---promptstring---Q---\n-->  ",
 				 title, b);
-			(void) fgets(output, maxsize, stdin);
+			p = fgets(output, maxsize, stdin);
+			if (!p) {
+				// FIXME
+			}
 			if (output[strlen(output) - 1] == '\n')
 				output[strlen(output) - 1] = '\0';
 			paranoid_free(blurb);
@@ -894,7 +873,8 @@ char *tmp = NULL;
 	bool popup_with_buttons(char *p, char *button1, char *button2) {
 
 		/*@ buffers *********************************************************** */
-		char *prompt, *tmp;
+		char *prompt;
+		char *q;
 
 		/*@ newt ************************************************************** */
 		newtComponent myForm;
@@ -903,29 +883,29 @@ char *tmp = NULL;
 		newtComponent b_res;
 		newtComponent text;
 
-		prompt = malloc(MAX_NEWT_COMMENT_LEN);
-		tmp = malloc(MAX_NEWT_COMMENT_LEN);
 		assert_string_is_neither_NULL_nor_zerolength(p);
 		assert(button1 != NULL);
 		assert(button2 != NULL);
+		prompt = malloc(MAX_NEWT_COMMENT_LEN);
 		if (g_text_mode) {
 			if (strlen(button2) == 0) {
 				printf("%s (%s) --> ", p, button1);
 			} else {
 				printf("%s (%s or %s) --> ", p, button1, button2);
 			}
-			for (tmp[0] = '\0';
-				 strcmp(tmp, button1) && (strlen(button2) == 0
-										  || strcmp(tmp, button2));) {
+			for (prompt[0] = '\0';
+				 strcmp(prompt, button1) && (strlen(button2) == 0
+										  || strcmp(prompt, button2));) {
 				printf("--> ");
-				(void) fgets(tmp, MAX_NEWT_COMMENT_LEN, stdin);
+				q = fgets(prompt, MAX_NEWT_COMMENT_LEN, stdin);
+				if (!q) {
+					// FIXME
+				}
 			}
-			if (!strcmp(tmp, button1)) {
-				paranoid_free(tmp);
+			if (!strcmp(prompt, button1)) {
 				paranoid_free(prompt);
 				return (TRUE);
 			} else {
-				paranoid_free(tmp);
 				paranoid_free(prompt);
 				return (FALSE);
 			}
@@ -957,11 +937,9 @@ char *tmp = NULL;
 		newtFormDestroy(myForm);
 		newtPopWindow();
 		if (b_res == b_1) {
-			paranoid_free(tmp);
 			paranoid_free(prompt);
 			return (TRUE);
 		} else {
-			paranoid_free(tmp);
 			paranoid_free(prompt);
 			return (FALSE);
 		}
@@ -1056,8 +1034,8 @@ char *tmp = NULL;
 
 		/*@ buffers ********************************************************** */
 		char *timeline_str;
-		char *pcline_str;
-		char *taskprogress;
+		char *pcline_str = NULL;
+		char *taskprogress = NULL;
 
 		/*@ int ************************************************************** */
 		int percentage = 0;
@@ -1065,8 +1043,6 @@ char *tmp = NULL;
 		int j = 0;
 
 		malloc_string(timeline_str);
-		malloc_string(pcline_str);
-		malloc_string(taskprogress);
 		timeline_str[0] = '\0';
 //  log_it("update_eval_call_form called");
 		if (num * 100 < denom) {
@@ -1084,7 +1060,9 @@ char *tmp = NULL;
 			time_remaining = 0;
 		}
 		if (!g_text_mode) {
-			newtLabelSetText(g_isoform_header, g_isoform_header_str);
+			if (g_isoform_header) {
+				newtLabelSetText(g_isoform_header, g_isoform_header_str);
+			}
 		}
 		g_mysterious_dot_counter = (g_mysterious_dot_counter + 1) % 27;
 		if ((percentage < 3 && g_isoform_old_progress < 3)
@@ -1092,55 +1070,56 @@ char *tmp = NULL;
 			g_isoform_old_progress = percentage;
 			sprintf(timeline_str,
 					"%2ld:%02ld taken            %2ld:%02ld remaining",
-					time_taken / 60, time_taken % 60, time_remaining / 60,
-					time_remaining % 60);
+					(long) time_taken / 60, (long) time_taken % 60, (long) time_remaining / 60,
+					(long) time_remaining % 60);
 			if (percentage < 3) {
-				sprintf(pcline_str, " Working");
+				mr_asprintf(&pcline_str, " Working");
 				for (j = 0; j < g_mysterious_dot_counter; j++) {
-					strcat(pcline_str, ".");
+					mr_strcat(pcline_str, ".");
 				}
 				for (; j < 27; j++) {
-					strcat(pcline_str, " ");
+					mr_strcat(pcline_str, " ");
 				}
-				sprintf(pcline_str + strlen(pcline_str), " %c",
-						special_dot_char(g_mysterious_dot_counter));
+				mr_strcat(pcline_str, " %c", special_dot_char(g_mysterious_dot_counter));
 			} else {
-				sprintf(pcline_str, " %3d%% done              %3d%% to go",
+				mr_asprintf(&pcline_str, " %3d%% done              %3d%% to go",
 						percentage, 100 - percentage);
 			}
 			if (g_text_mode) {
-				sprintf(taskprogress, "TASK:  [");
+				mr_asprintf(&taskprogress, "TASK:  [");
 				for (i = 0; i < percentage; i += 5) {
-					strcat(taskprogress, "*");
+					mr_strcat(taskprogress, "*");
 				}
 				for (; i < 100; i += 5) {
-					strcat(taskprogress, ".");
+					mr_strcat(taskprogress, ".");
 				}
 				if (percentage >= 3) {
-					sprintf(taskprogress + strlen(taskprogress),
-							"] %3d%% done; %2ld:%02ld to go", percentage,
-							time_remaining / 60, time_remaining % 60);
-					printf("---evalcall---1--- %s\r\n",
+					mr_strcat(taskprogress, "] %3d%% done; %2ld:%02ld to go", percentage,
+							(long) time_remaining / 60, (long) time_remaining % 60);
+					printf("---evalcall---1--- %s\n",
 						   g_isoform_header_str);
-					printf("---evalcall---2--- %s\r\n", taskprogress);
-					printf("---evalcall---E---\r\n");
+					printf("---evalcall---2--- %s\n", taskprogress);
+					printf("---evalcall---E---\n");
 				}
+				paranoid_free(taskprogress);
 			} else {
 				newtScaleSet(g_isoform_scale,
 							 (unsigned long long) percentage);
-				newtLabelSetText(g_isoform_pcline, pcline_str);
-				if (percentage >= 3) {
+				if (g_isoform_pcline) {
+					newtLabelSetText(g_isoform_pcline, pcline_str);
+				}
+				if ((percentage >= 3) && (g_isoform_timeline)) {
 					newtLabelSetText(g_isoform_timeline, timeline_str);
 				}
 			}
+			paranoid_free(pcline_str);
 		}
 		if (!g_text_mode) {
 //      log_it("refreshing");
+			newtDrawForm(g_isoform_main);
 			newtRefresh();
 		}
 		paranoid_free(timeline_str);
-		paranoid_free(pcline_str);
-		paranoid_free(taskprogress);
 	}
 
 
@@ -1183,10 +1162,10 @@ char *tmp = NULL;
 	void
 	 update_progress_form_full(char *blurb1, char *blurb2, char *blurb3) {
 		/*@ long ***************************************************** */
-		long current_time = 0;
-		long time_taken = 0;
-		long time_remaining = 0;
-		long time_total_est = 0;
+		long current_time = 0L;
+		long time_taken = 0L;
+		long time_remaining = 0L;
+		long time_total_est = 0L;
 
 		/*@ int ******************************************************* */
 		int percentage = 0;
@@ -1195,13 +1174,12 @@ char *tmp = NULL;
 		/*@ buffers *************************************************** */
 		char *percentline_str;
 		char *timeline_str;
-		char *taskprogress;
+		char *taskprogress = NULL;
 		char *tmp;
 
 //  log_msg(1, "'%s' '%s' '%s'", blurb1, blurb2, blurb3);
 		percentline_str = malloc(MAX_NEWT_COMMENT_LEN);
 		timeline_str = malloc(MAX_NEWT_COMMENT_LEN);
-		taskprogress = malloc(MAX_NEWT_COMMENT_LEN);
 		malloc_string(tmp);
 		if (!g_text_mode) {
 			assert(blurb1 != NULL);
@@ -1240,54 +1218,60 @@ char *tmp = NULL;
 				(long) (g_current_progress);
 			time_remaining = time_total_est - time_taken;
 		} else {
-			time_remaining = 0;
+			time_remaining = 0L;
 		}
 		g_mysterious_dot_counter = (g_mysterious_dot_counter + 1) % 27;
 		sprintf(timeline_str,
 				"%2ld:%02ld taken               %2ld:%02ld remaining  ",
-				time_taken / 60, time_taken % 60, time_remaining / 60,
+				(long) time_taken / 60, time_taken % 60, time_remaining / 60,
 				time_remaining % 60);
 		sprintf(percentline_str, " %3d%% done                 %3d%% to go",
 				percentage, 100 - percentage);
 
 		if (g_text_mode) {
-			printf("---progress-form---1--- %s\r\n", blurb1);
-			printf("---progress-form---2--- %s\r\n", blurb2);
-			printf("---progress-form---3--- %s\r\n", blurb3);
+			printf("---progress-form---1--- %s\n", blurb1);
+			printf("---progress-form---2--- %s\n", blurb2);
+			printf("---progress-form---3--- %s\n", blurb3);
 			printf("---progress-form---E---\n");
-			sprintf(taskprogress, "TASK:  [");
+			mr_asprintf(&taskprogress, "TASK:  [");
 			for (i = 0; i < percentage; i += 5) {
-				strcat(taskprogress, "*");
+				mr_strcat(taskprogress, "*");
 			}
 			for (; i < 100; i += 5) {
-				strcat(taskprogress, ".");
+				mr_strcat(taskprogress, ".");
 			}
 			if (percentage > 100) {
 				log_msg(2, "percentage = %d", percentage);
 			}
-			sprintf(taskprogress + strlen(taskprogress),
-					"] %3d%c", percentage, '%');
-			sprintf(taskprogress + strlen(taskprogress),
-					" done; %2ld:%02ld to go",
-					time_remaining / 60, time_remaining % 60);
-			printf("---progress-form---4--- %s\r\n", taskprogress);
+			mr_strcat(taskprogress, "] %3d%% done; %2ld:%02ld to go", percentage,
+							(long) time_remaining / 60, (long) time_remaining % 60);
+			printf("---progress-form---4--- %s\n", taskprogress);
+			paranoid_free(taskprogress);
 		} else {
 			center_string(blurb1, 54);
 			center_string(blurb2, 54);
 			center_string(blurb3, 54);
-			newtLabelSetText(g_blurb1, blurb1);
-			newtLabelSetText(g_blurb2, blurb3);
-			newtLabelSetText(g_blurb3, blurb2);
+			if (g_blurb1) {
+				newtLabelSetText(g_blurb1, blurb1);
+			}
+			if (g_blurb2) {
+				newtLabelSetText(g_blurb2, blurb3);
+			}
+			if (g_blurb3) {
+				newtLabelSetText(g_blurb3, blurb2);
+			}
 			newtScaleSet(g_scale, (unsigned long long) g_current_progress);
-			if (percentage >= 2) {
+			if ((percentage >= 2) && (g_timeline)) {
 				newtLabelSetText(g_timeline, timeline_str);
 			}
-			newtLabelSetText(g_percentline, percentline_str);
+			if (g_percentline) {
+				newtLabelSetText(g_percentline, percentline_str);
+			}
+			newtDrawForm(g_progressForm);
 			newtRefresh();
 		}
 		paranoid_free(percentline_str);
 		paranoid_free(timeline_str);
-		paranoid_free(taskprogress);
 		paranoid_free(tmp);
 	}
 
@@ -1307,7 +1291,7 @@ char *tmp = NULL;
 /**
  * Ask the user which backup media type they would like to use.
  * The choices are @p none (exit to shell), @c cdr, @c cdrw, @c dvd,
- * @c tape, @c cdstream, @c udev (only when @p g_text_mode is TRUE), @c nfs,
+ * @c tape, @c cdstream, @c udev (only when @p g_text_mode is TRUE), @c netfs,
  * and @c iso.
  * @param restoring TRUE if we're restoring, FALSE if we're backing up.
  * @return The backup type chosen, or @c none if the user chose "Exit to shell".
@@ -1322,11 +1306,12 @@ char *tmp = NULL;
 		char *title_sz;
 		char *minimsg_sz;
 		static t_bkptype possible_bkptypes[] =
-			{ none, cdr, cdrw, dvd, tape, cdstream, udev, nfs, iso };
+			{ none, cdr, cdrw, dvd, tape, cdstream, udev, netfs, iso };
 		static char *possible_responses[] =
 			{ "none", "cdr", "cdrw", "dvd", "tape", "cdstream", "udev",
-"nfs", "iso", NULL };
+"netfs", "iso", NULL };
 		char *outstr;
+		char *p;
 		t_bkptype backup_type;
 		int i;
 
@@ -1352,7 +1337,10 @@ char *tmp = NULL;
 						   possible_responses[i]);
 				}
 				printf(")\n--> ");
-				(void) fgets(outstr, MAX_NEWT_COMMENT_LEN, stdin);
+				p = fgets(outstr, MAX_NEWT_COMMENT_LEN, stdin);
+				if (!p) {
+					// FIXME
+				}
 				strip_spaces(outstr);
 				for (i = 0; possible_responses[i]; i++) {
 					if (!strcmp(possible_responses[i], outstr)) {
@@ -1381,9 +1369,9 @@ char *tmp = NULL;
 		b1 = newtButton(1, 1, "CD-R disks ");
 		b2 = newtButton(17, 1, "CD-RW disks");
 		b3 = newtButton(1, 9, "Tape drive ");
-		b4 = newtButton(17, 5, "CD streamer");
+		b4 = newtButton(17, 5, "USB Key/Disk");
 		b5 = newtButton(1, 5, " DVD disks ");
-		b6 = newtButton(17, 9, " NFS mount ");
+		b6 = newtButton(17, 9, " Net mount ");
 		b7 = newtButton(1, 13, " Hard disk ");
 		b8 = newtButton(17, 13, "    Exit   ");
 		myForm = newtForm(NULL, NULL, 0);
@@ -1399,11 +1387,11 @@ char *tmp = NULL;
 		} else if (b_res == b3) {
 			output = tape;
 		} else if (b_res == b4) {
-			output = cdstream;
+			output = usb;
 		} else if (b_res == b5) {
 			output = dvd;
 		} else if (b_res == b6) {
-			output = nfs;
+			output = netfs;
 		} else if (b_res == b7) {
 			output = iso;
 		} else {
@@ -1419,6 +1407,54 @@ char *tmp = NULL;
 
 
 
+/**
+ * Ask the user which compression type they would like to use.
+ * The choices are "bzip2", "gzip", "lzo". NULL for exit
+ */
+	char *which_compression_type() {
+
+		/*@ char ************************************************************ */
+		char *output = NULL;
+
+
+		/*@ newt ************************************************************ */
+
+		newtComponent b1;
+		newtComponent b2;
+		newtComponent b3;
+		//newtComponent b4;
+		newtComponent b5;
+		newtComponent b_res;
+		newtComponent myForm;
+
+		newtDrawRootText(18, 0, WELCOME_STRING);
+		newtPushHelpLine
+			("   Please specify the type of compression that you want.");
+		newtCenteredWindow(34, 13, "Type of compression?");
+		b1 = newtButton(4, 1, "bzip2");
+		b2 = newtButton(18, 1, "gzip");
+		b3 = newtButton(4, 5, "lzo");
+		//b4 = newtButton(18, 5, "lzma");
+		b5 = newtButton(4, 9, "         Exit        ");
+		myForm = newtForm(NULL, NULL, 0);
+		newtFormAddComponents(myForm, b1, b3, b2, b5, NULL);
+		b_res = newtRunForm(myForm);
+		newtFormDestroy(myForm);
+		newtPopWindow();
+		if (b_res == b1) {
+			mr_asprintf(&output, "%s", "bzip2");
+		} else if (b_res == b2) {
+			mr_asprintf(&output, "%s", "gzip");
+		} else if (b_res == b3) {
+			mr_asprintf(&output, "%s", "lzo");
+		//} else if (b_res == b4) {
+			//mr_asprintf(&output, "%s", "lzma");
+		}
+		newtPopHelpLine();
+		return(output);
+	}
+
+
 /**
  * Ask the user how much compression they would like to use.
  * The choices are "None" (0), "Minimum" (1), "Average" (4), and "Maximum" (9).
@@ -1446,10 +1482,10 @@ char *tmp = NULL;
 			("   Please specify the level of compression that you want.");
 		//  newtOpenWindow (23, 3, 34, 13, "How much compression?");
 		newtCenteredWindow(34, 13, "How much compression?");
-		b1 = newtButton(4, 1, "Maximum");
-		b2 = newtButton(18, 1, "Average");
-		b3 = newtButton(4, 5, "Minimum");
-		b4 = newtButton(18, 5, " None  ");
+		b1 = newtButton(4, 1, "Maximum (9)");
+		b2 = newtButton(18, 1, "Average (4)");
+		b3 = newtButton(4, 5, "Minimum (1)");
+		b4 = newtButton(18, 5, " None (0) ");
 		b5 = newtButton(4, 9, "         Exit        ");
 		myForm = newtForm(NULL, NULL, 0);
 		newtFormAddComponents(myForm, b1, b3, b2, b4, b5, NULL);
@@ -1484,23 +1520,32 @@ char *tmp = NULL;
 	int load_filelist_into_array(struct s_filelist *filelist,
 								 char *source_file) {
 		int i;
+		int j = 0;
 		bool done;
 		char *tmp;
-		FILE *fin;
+		char *p;
+		char *tmp1 = NULL;
+		FILE *fin, *fout;
 		struct s_filelist_entry dummy_fle;
 
-		malloc_string(tmp);
 		assert(filelist != NULL);
 		assert_string_is_neither_NULL_nor_zerolength(source_file);
 
-		iamhere("entering");
+		log_it("entering");
 		if (!(fin = fopen(source_file, "r"))) {
 			log_OS_error(source_file);
 			log_msg(2, "Can't open %s; therefore, cannot popup list",
 					source_file);
-			paranoid_free(tmp);
 			return (1);
 		}
+
+		mr_asprintf(&tmp1,"%s/icantfindthesefiles.txt",bkpinfo->tmpdir);
+		if (!(fout = fopen(tmp1, "a"))) {
+			log_msg(2, "Can't write to %s", tmp1);
+			return(1);
+			}
+
+		malloc_string(tmp);
 		log_msg(2, "Loading %s", source_file);
 		for (filelist->entries = 0; filelist->entries <= ARBITRARY_MAXIMUM;
 			 filelist->entries++) {
@@ -1508,7 +1553,10 @@ char *tmp = NULL;
 			if (feof(fin)) {
 				break;
 			}
-			(void) fgets(tmp, MAX_NEWT_COMMENT_LEN, fin);
+			p = fgets(tmp, MAX_NEWT_COMMENT_LEN, fin);
+			if (!p) {
+				// FIXME
+			}
 			i = (int) strlen(tmp);
 			if (i < 2) {
 				goto god_i_hate_gotos;
@@ -1519,7 +1567,10 @@ char *tmp = NULL;
 			if (i < 2) {
 				goto god_i_hate_gotos;
 			}
-			if (!does_file_exist(tmp)) {
+			if (!does_file_exist(tmp) && !feof(fin)) {
+				j++;
+				fprintf(fout, "%s\n", tmp);
+
 				goto god_i_hate_gotos;
 			}
 			filelist->el[filelist->entries].severity =
@@ -1530,6 +1581,11 @@ char *tmp = NULL;
 			}
 		}
 		paranoid_fclose(fin);
+		paranoid_fclose(fout);
+		if (j > 0) {
+			log_to_screen("%d files listed in %s/changed.files have been deleted since backup was made\nand are referenced in %s", j, bkpinfo->tmpdir,tmp1);
+		}
+		paranoid_free(tmp1);
 		if (filelist->entries >= ARBITRARY_MAXIMUM) {
 			log_to_screen("Arbitrary limits suck, man!");
 			paranoid_free(tmp);
@@ -1561,7 +1617,7 @@ char *tmp = NULL;
 			}
 		}
 		paranoid_free(tmp);
-		iamhere("leaving");
+		log_it("leaving");
 		return (0);
 	}
 
@@ -1575,26 +1631,25 @@ char *tmp = NULL;
  */
 	char *filelist_entry_to_string(struct s_filelist_entry *flentry) {
 		static char comment[100];
-		char *tmp;
+		char *tmp = NULL;
 
-		iamhere("entering");
-		malloc_string(tmp);
+		log_it("entering");
 		assert(flentry != NULL);
 		if (flentry->severity == 0) {
-			strcpy(tmp, "0   ");
+			mr_asprintf(&tmp, "0   ");
 		} else if (flentry->severity == 1) {
-			strcpy(tmp, "low ");
+			mr_asprintf(&tmp, "low ");
 		} else if (flentry->severity == 2) {
-			strcpy(tmp, "med ");
+			mr_asprintf(&tmp, "med ");
 		} else {
-			strcpy(tmp, "high");
+			mr_asprintf(&tmp, "high");
 		}
-		strcat(tmp, "  ");
-		strncat(tmp, flentry->filename, 100);
-		tmp[98] = '\0';
-		strcpy(comment, tmp);
+		mr_strcat(tmp, "  ");
+		mr_strcat(tmp, flentry->filename);
+		strncpy(comment, tmp, 99);
 		paranoid_free(tmp);
-		iamhere("leaving");
+
+		log_it("leaving");
 		return (comment);
 	}
 
@@ -1725,7 +1780,3 @@ char *tmp = NULL;
 #if __cplusplus
 }								/* extern "C" */
 #endif
-
-
-void wait_until_software_raids_are_prepped(char *mdstat_file,
-										   int wait_for_percentage);
