@@ -1,61 +1,10 @@
 /***************************************************************************
-       mondo-compare.c  -  compares mondoarchive data
-                             -------------------
-    begin                : Fri Apr 25 2003
-    copyright            : (C) 2000 by Hugo Rabson
-    email                : Hugo Rabson <hugorabson@msn.com>
-    cvsid                : $Id: mondo-rstr-compare.c 1321 2007-04-16 19:11:19Z bruno $
+    cvsid                : $Id: mondo-rstr-compare.c 3060 2012-11-10 04:05:37Z bruno $
 ***************************************************************************/
 
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
-
-/***************************************************************************
- *                         Change Log                                      *
- ***************************************************************************
-.
-
-
-
-10/21/2003
-- changed "/mnt/cdrom" to MNT_CDROM
-
-10/18
-- don't say unknown compressor if no compressor at all
-
-09/17
-- cleaned up logging & conversion-to-changed.txt
-- cleaned up compare_mode()
-
-09/16
-- fixed bad malloc(),free() pairs in compare_a_biggiefile()
-
-09/14
-- compare_mode() --- a couple of strings were the wrong way round,
-  e.g. changed.txt and changed.files
-
-05/05
-- exclude /dev/ * from list of changed files
-
-04/30
-- added textonly mode
-
-04/27
-- improved compare_mode() to allow for ISO/cd/crazy people
-
-04/25
-- first incarnation
-*/
-
-
 #include <pthread.h>
-#include "../common/my-stuff.h"
+#include "my-stuff.h"
+#include "mr_mem.h"
 #include "../common/mondostructures.h"
 #include "../common/libmondo.h"
 //#include "../../config.h"
@@ -66,7 +15,12 @@
 
 extern char *MONDO_LOGFILE;
 
-//static char cvsid[] = "$Id: mondo-rstr-compare.c 1321 2007-04-16 19:11:19Z bruno $";
+/* Reference to global bkpinfo */
+extern struct s_bkpinfo *bkpinfo;
+
+extern char *g_mountlist_fname;
+
+//static char cvsid[] = "$Id: mondo-rstr-compare.c 3060 2012-11-10 04:05:37Z bruno $";
 
 void popup_changelist_from_file(char *);
 
@@ -81,7 +35,7 @@ void popup_changelist_from_file(char *);
  * @param bigfileno The biggiefile number (starting from 0) to compare.
  * @note This function uses an MD5 checksum.
  */
-int compare_a_biggiefile(struct s_bkpinfo *bkpinfo, long bigfileno)
+int compare_a_biggiefile(long bigfileno)
 {
 
 	FILE *fin;
@@ -91,7 +45,7 @@ int compare_a_biggiefile(struct s_bkpinfo *bkpinfo, long bigfileno)
 	char *checksum_ptr;
 	char *original_cksum_ptr;
 	char *bigfile_fname_ptr;
-	char *tmp_ptr;
+	char *tmp_ptr = NULL;
 	char *command_ptr;
 
 	char *checksum, *original_cksum, *bigfile_fname, *tmp, *command;
@@ -111,7 +65,6 @@ int compare_a_biggiefile(struct s_bkpinfo *bkpinfo, long bigfileno)
 	malloc_string(original_cksum_ptr);
 	malloc_string(bigfile_fname_ptr);
 	malloc_string(command_ptr);
-	malloc_string(tmp_ptr);
 
   /*********************************************************************
    * allocate memory clear test                sab 16 feb 2003         *
@@ -120,28 +73,28 @@ int compare_a_biggiefile(struct s_bkpinfo *bkpinfo, long bigfileno)
 	memset(checksum_ptr, '\0', sizeof(checksum));
 	memset(original_cksum_ptr, '\0', sizeof(original_cksum));
 	memset(bigfile_fname_ptr, '\0', sizeof(bigfile_fname));
-	memset(tmp_ptr, '\0', sizeof(tmp));
 	memset(command_ptr, '\0', sizeof(command));
   /** end **/
 
 	if (!does_file_exist(slice_fname(bigfileno, 0, ARCHIVES_PATH, ""))) {
 		if (does_file_exist(MNT_CDROM "/archives/NOT-THE-LAST")) {
-			insist_on_this_cd_number(bkpinfo, (++g_current_media_number));
+			insist_on_this_cd_number((++g_current_media_number));
 		} else {
-			sprintf(tmp_ptr,
-					"No CD's left. No biggiefiles left. No prob, Bob.");
+			mr_asprintf(&tmp_ptr, "No CD's left. No biggiefiles left. No prob, Bob.");
 			log_msg(2, tmp_ptr);
+			paranoid_free(tmp_ptr);
 			return (0);
 		}
 	}
 	if (!(fin = fopen(slice_fname(bigfileno, 0, ARCHIVES_PATH, ""), "r"))) {
-		sprintf(tmp_ptr,
-				"Cannot open bigfile %ld (%s)'s info file",
-				bigfileno + 1, bigfile_fname_ptr);
+		mr_asprintf(&tmp_ptr, "Cannot open bigfile %ld (%s)'s info file", bigfileno + 1, bigfile_fname_ptr);
 		log_to_screen(tmp_ptr);
+		paranoid_free(tmp_ptr);
 		return (1);
 	}
-	fread((void *) &biggiestruct, 1, sizeof(biggiestruct), fin);
+	if (fread((void *) &biggiestruct, 1, sizeof(biggiestruct), fin)) {
+		// FIXME
+	}
 	paranoid_fclose(fin);
 
 	strcpy(checksum_ptr, biggiestruct.checksum);
@@ -150,10 +103,10 @@ int compare_a_biggiefile(struct s_bkpinfo *bkpinfo, long bigfileno)
 	log_msg(2, "biggiestruct.filename = %s", biggiestruct.filename);
 	log_msg(2, "biggiestruct.checksum = %s", biggiestruct.checksum);
 
-	sprintf(tmp_ptr, "Comparing %s", bigfile_fname_ptr);
-
 	if (!g_text_mode) {
+		mr_asprintf(&tmp_ptr, "Comparing %s", bigfile_fname_ptr);
 		newtDrawRootText(0, 22, tmp_ptr);
+		paranoid_free(tmp_ptr);
 		newtRefresh();
 	}
 	if (!checksum[0]) {
@@ -167,8 +120,10 @@ int compare_a_biggiefile(struct s_bkpinfo *bkpinfo, long bigfileno)
 				MNT_RESTORING, bigfile_fname_ptr);
 	}
 	log_msg(2, command_ptr);
-	sprintf(tmp_ptr, "cat /tmp/errors >> %s 2> /dev/null", MONDO_LOGFILE);
+	mr_asprintf(&tmp_ptr, "cat /tmp/errors >> %s 2> /dev/null", MONDO_LOGFILE);
 	paranoid_system(tmp_ptr);
+	paranoid_free(tmp_ptr);
+
 	if (system(command_ptr)) {
 		log_OS_error("Warning - command failed");
 		original_cksum[0] = '\0';
@@ -180,7 +135,9 @@ int compare_a_biggiefile(struct s_bkpinfo *bkpinfo, long bigfileno)
 			original_cksum[0] = '\0';
 			return (1);
 		} else {
-			fgets(original_cksum_ptr, MAX_STR_LEN - 1, fin);
+			if (fgets(original_cksum_ptr, MAX_STR_LEN - 1, fin)) {
+				// FIXME
+			}
 			paranoid_fclose(fin);
 			for (i = strlen(original_cksum_ptr);
 				 i > 0 && original_cksum[i - 1] < 32; i--);
@@ -191,17 +148,18 @@ int compare_a_biggiefile(struct s_bkpinfo *bkpinfo, long bigfileno)
 			}
 		}
 	}
-	sprintf(tmp_ptr, "bigfile #%ld ('%s') ", bigfileno + 1,
-			bigfile_fname_ptr);
+	mr_asprintf(&tmp_ptr, "bigfile #%ld ('%s') ", bigfileno + 1, bigfile_fname_ptr);
 	if (!strcmp(checksum_ptr, original_cksum_ptr) != 0) {
-		strcat(tmp_ptr, " ... OK");
+		mr_strcat(tmp_ptr, " ... OK");
 	} else {
-		strcat(tmp_ptr, "... changed");
+		mr_strcat(tmp_ptr, "... changed");
 		retval++;
 	}
 	log_msg(1, tmp_ptr);
+	paranoid_free(tmp_ptr);
+
 	if (retval) {
-		if (!(fout = fopen("/tmp/changed.txt", "a"))) {
+		if (!(fout = fopen(MONDO_CACHE"/changed.txt", "a"))) {
 			fatal_error("Cannot openout changed.txt");
 		}
 		fprintf(fout, "%s\n", bigfile_fname_ptr);
@@ -216,7 +174,6 @@ int compare_a_biggiefile(struct s_bkpinfo *bkpinfo, long bigfileno)
 	paranoid_free(checksum);
 	paranoid_free(command_ptr);
 	paranoid_free(command);
-	paranoid_free(tmp_ptr);
 	paranoid_free(tmp);
 
 	return (retval);
@@ -229,17 +186,15 @@ int compare_a_biggiefile(struct s_bkpinfo *bkpinfo, long bigfileno)
 
 /**
  * Compare all biggiefiles in the backup.
- * @param bkpinfo The backup information structure. Used only in compare_a_biggiefile().
  * @return 0 for success, nonzero for failure.
  */
-int compare_all_biggiefiles(struct s_bkpinfo *bkpinfo)
+int compare_all_biggiefiles()
 {
 	int retval = 0;
 	int res;
 	long noof_biggiefiles, bigfileno = 0;
 	char tmp[MAX_STR_LEN];
 
-	assert(bkpinfo != NULL);
 	log_msg(1, "Comparing biggiefiles");
 
 	if (length_of_file(BIGGIELIST) < 6) {
@@ -262,7 +217,7 @@ int compare_all_biggiefiles(struct s_bkpinfo *bkpinfo)
 		sprintf(tmp, "Comparing big file #%ld", bigfileno + 1);
 		log_msg(1, tmp);
 		update_progress_form(tmp);
-		res = compare_a_biggiefile(bkpinfo, bigfileno);
+		res = compare_a_biggiefile(bigfileno);
 		retval += res;
 		g_current_progress++;
 	}
@@ -293,19 +248,18 @@ int compare_a_tarball(char *tarball_fname, int current_tarball_number)
 {
 	int retval = 0;
 	int res;
-	long noof_lines;
 	long archiver_errors;
 	bool use_star;
 
   /***  needs malloc *********/
-	char *command, *tmp, *filelist_name, *logfile, *archiver_exe,
+	char *command, *tmp, *filelist_name, *logfile,
 		*compressor_exe;
+	char *archiver_exe = NULL;
 
 	malloc_string(command);
 	malloc_string(tmp);
 	malloc_string(filelist_name);
 	malloc_string(logfile);
-	malloc_string(archiver_exe);
 	malloc_string(compressor_exe);
 
 	use_star = (strstr(tarball_fname, ".star")) ? TRUE : FALSE;
@@ -314,8 +268,6 @@ int compare_a_tarball(char *tarball_fname, int current_tarball_number)
 	sprintf(filelist_name, MNT_CDROM "/archives/filelist.%d",
 			current_tarball_number);
 
-	noof_lines = count_lines_in_file(filelist_name);
-
 	if (strstr(tarball_fname, ".bz2")) {
 		strcpy(compressor_exe, "bzip2");
 	} else if (strstr(tarball_fname, ".gz")) {
@@ -327,9 +279,9 @@ int compare_a_tarball(char *tarball_fname, int current_tarball_number)
 	}
 
 	if (use_star) {
-		strcpy(archiver_exe, "star");
+		mr_asprintf(&archiver_exe, "star");
 	} else {
-		strcpy(archiver_exe, "afio");
+		mr_asprintf(&archiver_exe, "afio");
 	}
 
 	if (compressor_exe[0]) {
@@ -340,7 +292,7 @@ int compare_a_tarball(char *tarball_fname, int current_tarball_number)
 		if (use_star)			// star
 		{
 			if (!strcmp(compressor_exe, "bzip2")) {
-				strcat(archiver_exe, " -bz");
+				mr_strcat(archiver_exe, " -bz");
 			} else {
 				fatal_error
 					("(compare_a_tarball) Please use only bzip2 with star");
@@ -360,7 +312,7 @@ int compare_a_tarball(char *tarball_fname, int current_tarball_number)
 	if (use_star)				// doesn't use compressor_exe
 	{
 		sprintf(command,
-				"%s -diff H=star file=%s >> %s 2>> %s",
+				"%s -sparse -diff H=exustar file=%s >> %s 2>> %s",
 				archiver_exe, tarball_fname, logfile, logfile);
 	} else {
 		sprintf(command,
@@ -369,6 +321,8 @@ int compare_a_tarball(char *tarball_fname, int current_tarball_number)
 				TAPE_BLOCK_SIZE,
 				BUFSIZE, compressor_exe, tarball_fname, logfile, logfile);
 	}
+	paranoid_free(archiver_exe);
+
 #undef BUFSIZE
 
 	res = system(command);
@@ -376,12 +330,13 @@ int compare_a_tarball(char *tarball_fname, int current_tarball_number)
 	if (res) {
 		log_OS_error(command);
 		sprintf(tmp, "Warning - afio returned error = %d", res);
+		log_msg(2, tmp);
 	}
 	if (length_of_file(logfile) > 5) {
 		sprintf(command,
-				"sed s/': \\\"'/\\|/ %s | sed s/'\\\": '/\\|/ | cut -d'|' -f2 | sort -u | grep -vE \"^dev/.*\" >> /tmp/changed.txt",
+				"sed s/': \\\"'/\\|/ %s | sed s/'\\\": '/\\|/ | cut -d'|' -f2 | sort -u | grep -vE \"^dev/.*\" >> "MONDO_CACHE"/changed.txt",
 				logfile);
-		system(command);
+		paranoid_system(command);
 		archiver_errors = count_lines_in_file(logfile);
 	} else {
 		archiver_errors = 0;
@@ -400,8 +355,7 @@ int compare_a_tarball(char *tarball_fname, int current_tarball_number)
 	paranoid_free(tmp);
 	paranoid_free(filelist_name);
 	paranoid_free(logfile);
-	malloc_string(archiver_exe);
-	malloc_string(compressor_exe);
+	paranoid_free(compressor_exe);
 	return (retval);
 }
 
@@ -415,15 +369,15 @@ int compare_a_tarball(char *tarball_fname, int current_tarball_number)
  * @param bkpinfo The backup media structure. Passed to other functions.
  * @return 0 for success, nonzero for failure.
  */
-int compare_all_tarballs(struct s_bkpinfo *bkpinfo)
+int compare_all_tarballs()
 {
 	int retval = 0;
-	int res;
 	int current_tarball_number = 0;
 
   /**  needs malloc **********/
 
 	char *tarball_fname, *progress_str, *tmp;
+	char *mds = NULL;
 	long max_val;
 
 	malloc_string(tarball_fname);
@@ -435,9 +389,8 @@ int compare_all_tarballs(struct s_bkpinfo *bkpinfo)
 	read_cfg_var(g_mondo_cfg_file, "last-filelist-number", tmp);
 
 	max_val = atol(tmp);
-	sprintf(progress_str, "Comparing with %s #%d ",
-			media_descriptor_string(bkpinfo->backup_media_type),
-			g_current_media_number);
+	mds = media_descriptor_string(bkpinfo->backup_media_type);
+	sprintf(progress_str, "Comparing with %s #%d ", mds, g_current_media_number);
 
 	open_progress_form("Comparing files",
 					   "Comparing tarballs against filesystem.",
@@ -447,7 +400,7 @@ int compare_all_tarballs(struct s_bkpinfo *bkpinfo)
 	log_to_screen(progress_str);
 
 	for (;;) {
-		insist_on_this_cd_number(bkpinfo, g_current_media_number);
+		insist_on_this_cd_number(g_current_media_number);
 		update_progress_form(progress_str);
 		sprintf(tarball_fname,
 				MNT_CDROM "/archives/%d.afio.bz2", current_tarball_number);
@@ -456,6 +409,10 @@ int compare_all_tarballs(struct s_bkpinfo *bkpinfo)
 			sprintf(tarball_fname, MNT_CDROM "/archives/%d.afio.lzo",
 					current_tarball_number);
 		}
+		if (!does_file_exist(tarball_fname)) {
+			sprintf(tarball_fname, MNT_CDROM "/archives/%d.afio.gz",
+					current_tarball_number);
+		}
 		if (!does_file_exist(tarball_fname)) {
 			sprintf(tarball_fname, MNT_CDROM "/archives/%d.afio.",
 					current_tarball_number);
@@ -478,17 +435,17 @@ int compare_all_tarballs(struct s_bkpinfo *bkpinfo)
 			}
 			log_msg(2, "OK, I think it's time for another CD...");
 			g_current_media_number++;
-			sprintf(progress_str, "Comparing with %s #%d ",
-					media_descriptor_string(bkpinfo->backup_media_type),
-					g_current_media_number);
+			sprintf(progress_str, "Comparing with %s #%d ", mds, g_current_media_number);
 			log_to_screen(progress_str);
 		} else {
-			res = compare_a_tarball(tarball_fname, current_tarball_number);
+			retval += compare_a_tarball(tarball_fname, current_tarball_number);
 
 			g_current_progress++;
 			current_tarball_number++;
 		}
 	}
+	mr_free(mds);
+
 	close_progress_form();
 	if (retval) {
 		mvaddstr_and_log_it(g_currentY++, 74, "Errors.");
@@ -517,7 +474,7 @@ int compare_all_tarballs(struct s_bkpinfo *bkpinfo)
  * @param bkpinfo The backup information structure. Passed to other functions.
  * @return 0 for success, nonzero for failure.
  */
-int compare_to_CD(struct s_bkpinfo *bkpinfo)
+int compare_to_CD()
 {
   /** needs malloc *********/
 	char *tmp, *cwd, *new, *command;
@@ -532,23 +489,30 @@ int compare_to_CD(struct s_bkpinfo *bkpinfo)
 
 	assert(bkpinfo != NULL);
 
-	getcwd(cwd, MAX_STR_LEN - 1);
-	chdir(bkpinfo->restore_path);
-	getcwd(new, MAX_STR_LEN - 1);
-	sprintf(tmp, "new path is %s", new);
-	insist_on_this_cd_number(bkpinfo, g_current_media_number);
-	unlink("/tmp/changed.txt");
+	if (getcwd(cwd, MAX_STR_LEN - 1)) {
+		// FIXME
+	}
+	if (chdir(bkpinfo->restore_path)) {
+		//FIXME
+	}
+	if (getcwd(new, MAX_STR_LEN - 1)) {
+		// FIXME
+	}
+	insist_on_this_cd_number(g_current_media_number);
+	unlink(MONDO_CACHE"/changed.txt");
 
-	resA = compare_all_tarballs(bkpinfo);
-	resB = compare_all_biggiefiles(bkpinfo);
-	chdir(cwd);
-	noof_changed_files = count_lines_in_file("/tmp/changed.txt");
+	resA = compare_all_tarballs();
+	resB = compare_all_biggiefiles();
+	if (chdir(cwd)) {
+		// FIXME
+	}
+	noof_changed_files = count_lines_in_file(MONDO_CACHE"/changed.txt");
 	if (noof_changed_files) {
 		sprintf(tmp, "%ld files do not match the backup            ",
 				noof_changed_files);
 		//      mvaddstr_and_log_it( g_currentY++, 0, tmp );
 		log_to_screen(tmp);
-		sprintf(command, "cat /tmp/changed.txt >> %s", MONDO_LOGFILE);
+		sprintf(command, "cat "MONDO_CACHE"/changed.txt >> %s", MONDO_LOGFILE);
 		paranoid_system(command);
 	} else {
 		sprintf(tmp, "All files match the backup                     ");
@@ -581,15 +545,19 @@ int compare_to_CD(struct s_bkpinfo *bkpinfo)
  * @return The number of errors/differences found.
  */
 int
-compare_mode(struct s_bkpinfo *bkpinfo,
-			 struct mountlist_itself *mountlist,
+compare_mode(struct mountlist_itself *mountlist,
 			 struct raidlist_itself *raidlist)
 {
 	int retval = 0;
+	int res = 0;
 	long q;
 	char *tmp;
+	char *new;
+	char *cwd;
 
 	malloc_string(tmp);
+	malloc_string(new);
+	malloc_string(cwd);
 
   /**************************************************************************
    * also deletes tmp/filelist.full & tmp/biggielist.txt _and_ tries to     *
@@ -599,32 +567,50 @@ compare_mode(struct s_bkpinfo *bkpinfo,
 	assert(mountlist != NULL);
 	assert(raidlist != NULL);
 
-	while (get_cfg_file_from_archive(bkpinfo)) {
+	while (get_cfg_file_from_archive()) {
 		if (!ask_me_yes_or_no
 			("Failed to find config file/archives. Choose another source?"))
 		{
 			fatal_error("Unable to find config file/archives. Aborting.");
 		}
-		interactively_obtain_media_parameters_from_user(bkpinfo, FALSE);
+		interactively_obtain_media_parameters_from_user(FALSE);
 	}
 
-	read_cfg_file_into_bkpinfo(g_mondo_cfg_file, bkpinfo);
+	read_cfg_file_into_bkpinfo(g_mondo_cfg_file);
+
+	/* edit_mountlist if wanted */
+	log_it("About to edit mountlist");
+	if (g_text_mode) {
+		save_mountlist_to_disk(mountlist, g_mountlist_fname);
+		sprintf(tmp, "%s %s", find_my_editor(), g_mountlist_fname);
+		res = system(tmp);
+		load_mountlist(mountlist, g_mountlist_fname);
+	} else {
+		res = edit_mountlist(g_mountlist_fname, mountlist, raidlist);
+	}
+	log_it("Finished editing mountlist");
+	if (res) {
+		paranoid_MR_finish(1);
+	}
+	save_mountlist_to_disk(mountlist, g_mountlist_fname);
+	save_raidlist_to_raidtab(raidlist, RAIDTAB_FNAME);
+
 	g_current_media_number = 1;
 	mvaddstr_and_log_it(1, 30, "Comparing Automatically");
-	iamhere("Pre-MAD");
+	log_it("Pre-MAD");
 	retval = mount_all_devices(mountlist, FALSE);
-	iamhere("Post-MAD");
+	log_it("Post-MAD");
 	if (retval) {
 		unmount_all_devices(mountlist);
 		return (retval);
 	}
 	if (bkpinfo->backup_media_type == tape
 		|| bkpinfo->backup_media_type == udev) {
-		retval += compare_to_tape(bkpinfo);
+		retval += compare_to_tape();
 	} else if (bkpinfo->backup_media_type == cdstream) {
-		retval += compare_to_cdstream(bkpinfo);
+		retval += compare_to_cdstream();
 	} else {
-		retval += compare_to_CD(bkpinfo);
+		retval += compare_to_CD();
 	}
 	if (retval) {
 		mvaddstr_and_log_it(g_currentY++,
@@ -632,19 +618,17 @@ compare_mode(struct s_bkpinfo *bkpinfo,
 							"Warning - differences found during the compare phase");
 	}
 
-	retval += unmount_all_devices(mountlist);
-
-	if (count_lines_in_file("/tmp/changed.txt") > 0) {
+	if (count_lines_in_file(MONDO_CACHE"/changed.txt") > 0) {
 		mvaddstr_and_log_it(g_currentY++, 0,
 							"Differences found while files were being compared.");
-		streamline_changes_file("/tmp/changed.files", "/tmp/changed.txt");
-		if (count_lines_in_file("/tmp/changed.files") <= 0) {
+		streamline_changes_file(MONDO_CACHE"/changed.files", MONDO_CACHE"/changed.txt");
+		if (count_lines_in_file(MONDO_CACHE"/changed.files") <= 0) {
 			mvaddstr_and_log_it(g_currentY++, 0,
 								"...but they were logfiles and temporary files. Your archives are fine.");
 			log_to_screen
 				("The differences were logfiles and temporary files. Your archives are fine.");
 		} else {
-			q = count_lines_in_file("/tmp/changed.files");
+			q = count_lines_in_file(MONDO_CACHE"/changed.files");
 			sprintf(tmp, "%ld significant difference%s found.", q,
 					(q != 1) ? "s" : "");
 			mvaddstr_and_log_it(g_currentY++, 0, tmp);
@@ -656,15 +640,31 @@ compare_mode(struct s_bkpinfo *bkpinfo,
 			log_to_screen(tmp);
 
 			log_msg(2, "calling popup_changelist_from_file()");
-			popup_changelist_from_file("/tmp/changed.files");
+			if (getcwd(cwd, MAX_STR_LEN - 1)) {
+				//FIXME
+			}
+			if (chdir(bkpinfo->restore_path)) {
+				// FIXME
+			}
+			if (getcwd(new, MAX_STR_LEN - 1)) {
+				//FIXME
+			}
+			popup_changelist_from_file(MONDO_CACHE"/changed.files");
+			if (chdir(cwd)) {
+				// FIXME
+			}
 			log_msg(2, "Returning from popup_changelist_from_file()");
 		}
 	} else {
 		log_to_screen
 			("No significant differences were found. Your backup is perfect.");
 	}
+	retval += unmount_all_devices(mountlist);
+
 	kill_petris();
 	paranoid_free(tmp);
+	paranoid_free(new);
+	paranoid_free(cwd);
 	return (retval);
 }
 
@@ -680,7 +680,7 @@ compare_mode(struct s_bkpinfo *bkpinfo,
  * - @c bkpinfo->restore_path
  * @return 0 for success, nonzero for failure.
  */
-int compare_to_cdstream(struct s_bkpinfo *bkpinfo)
+int compare_to_cdstream()
 {
 	int res;
 
@@ -690,8 +690,12 @@ int compare_to_cdstream(struct s_bkpinfo *bkpinfo)
 	assert(bkpinfo != NULL);
 	malloc_string(dir);
 	malloc_string(command);
-	getcwd(dir, MAX_STR_LEN);
-	chdir(bkpinfo->restore_path);
+	if (getcwd(dir, MAX_STR_LEN)) {
+		// FIXME
+	}
+	if (chdir(bkpinfo->restore_path)) {
+		// FIXME
+	}
 
 	sprintf(command, "cp -f /tmp/LAST-FILELIST-NUMBER %s/tmp",
 			bkpinfo->restore_path);
@@ -706,14 +710,16 @@ int compare_to_cdstream(struct s_bkpinfo *bkpinfo)
 	} else {
 		find_cdrom_device(bkpinfo->media_device, FALSE);
 	}
-	res = verify_tape_backups(bkpinfo);
-	chdir(dir);
-	if (length_of_file("/tmp/changed.txt") > 2
-		&& length_of_file("/tmp/changed.files") > 2) {
+	res = verify_tape_backups();
+	if (chdir(dir)) {
+		// FIXME
+	}
+	if (length_of_file(MONDO_CACHE"/changed.txt") > 2
+		&& length_of_file(MONDO_CACHE"/changed.files") > 2) {
 		log_msg(0,
-				"Type 'less /tmp/changed.files' to see which files don't match the archives");
+				"Type 'less "MONDO_CACHE"/changed.files' to see which files don't match the archives");
 		log_msg(2, "Calling popup_changelist_from_file()");
-		popup_changelist_from_file("/tmp/changed.files");
+		popup_changelist_from_file(MONDO_CACHE"/changed.files");
 		log_msg(2, "Returned from popup_changelist_from_file()");
 	}
 
@@ -739,7 +745,7 @@ int compare_to_cdstream(struct s_bkpinfo *bkpinfo)
  *                                                                        *
  * returns: int                                                           *
  **************************************************************************/
-int compare_to_tape(struct s_bkpinfo *bkpinfo)
+int compare_to_tape()
 {
 	int res;
 	char *dir, *command;
@@ -748,15 +754,21 @@ int compare_to_tape(struct s_bkpinfo *bkpinfo)
 	malloc_string(dir);
 	malloc_string(command);
 
-	getcwd(dir, MAX_STR_LEN);
-	chdir(bkpinfo->restore_path);
+	if (getcwd(dir, MAX_STR_LEN)) {
+		// FIXME
+	}
+	if (chdir(bkpinfo->restore_path)) {
+		// FIXME
+	}
 	sprintf(command, "cp -f /tmp/LAST-FILELIST-NUMBER %s/tmp",
 			bkpinfo->restore_path);
 	run_program_and_log_output(command, FALSE);
 	mvaddstr_and_log_it(g_currentY,
 						0, "Verifying archives against filesystem");
-	res = verify_tape_backups(bkpinfo);
-	chdir(dir);
+	res = verify_tape_backups();
+	if (chdir(dir)) {
+		// FIXME
+	}
 	if (res) {
 		mvaddstr_and_log_it(g_currentY++, 74, "Failed.");
 	} else {
