@@ -1,98 +1,20 @@
 /***************************************************************************
-mondo-rstr-newt.c  -  description
------------------
-
-begin: Fri Apr 19 16:40:35 EDT 2002
-copyright : (C) 2002 Mondo  Hugo Rabson
-email	  : Hugo Rabson <hugorabson@msn.com>
-edited by : by Stan Benoit 4/2002
-email     : troff@nakedsoul.org
-cvsid     : $Id: mondo-rstr-newt.c 1967 2008-05-29 15:40:05Z bruno $
-***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
-/* mondo-rstr-newt.c               Hugo Rabson
-
-
-07/12
-- wider mountlist entry editor window
-
-03/28/2004
-- removed g_mountlist_fname
-
-07/02/2003
-- fixed calls to popup_and_get_string()
-
-06/05
-- cleaned up get_isodir_info()
-
-05/02
-- added strip_path() to improve file list display for long paths (Conor)
-
-04/24
-- added lots of assert()'s and log_OS_error()'s
-
-04/08/2003
-- commented out call to sort_mountlist_by_device(), just in case it's 
-  futzing with LVM/RAID users
-
-08/21/2002
-- fix MOUNTLIST_FNAME and RAIDTAB_FNAME if Reload button finds them empty
-
-08/19
-- fixed what_number_cd_is_this() to take bkpinfo as parameter
-
-07/01 - 07/31
-- renamed from mondo-newt.c to mondo-rstr-newt.c
-- moved lots of subroutines to libmondo-newt.c
-- added hooks to libmondo
-- better handling of yes/no/YES/NO
-- better "% done" feedback (Philippe de Muyter)
-
-05/01 - 06/30
-- trim trailing \n from fgets() in a couple of functions
-- expanded "May I write mountlist?" message
-- replace scanf() with a properly restricted fgets() (Troff)
-
-02/01 - 02/28/2002
-- allow up to 50 chars in popup_and_get_string()'s input field
-- if g_current_progress > g_maximum_progress then g_current_progress=g_max...
-  to stop the progress bar from shooting off the RHS of the screen :)
-- beefed up the RAID-related logging a bit
-- don't complain if partition format is a number (1-255)
-- if hard disk does not exist then warn user when editing mountlist
-- when running 'eject -t', log its output
-
-[...]
-
-08/09/2001
-- created
-*/
-
-
-/**
- * @file
+ * $Id: mondo-rstr-newt.c 3158 2013-06-24 08:20:16Z bruno $
  * Functions for handling GUI interfaces in the restore.
  */
 
 #ifdef __FreeBSD__
 #define OSSWAP(linux,fbsd) fbsd
-//#include <libgen.h>
 #else
 #define OSSWAP(linux,fbsd) linux
 #endif
 
 #include "mondo-rstr-newt.h"
+#include "mr_mem.h"
 
-//static char cvsid[] = "$Id: mondo-rstr-newt.c 1967 2008-05-29 15:40:05Z bruno $";
+//static char cvsid[] = "$Id: mondo-rstr-newt.c 3158 2013-06-24 08:20:16Z bruno $";
 
+extern bool popup_with_buttons(char *p, char *button1, char *button2);
 extern char err_log_lines[NOOF_ERR_LINES][MAX_STR_LEN];
 
 /**
@@ -213,7 +135,6 @@ add_mountlist_entry(struct mountlist_itself *mountlist,
 
 	/** int **************************************************************/
 	int i = 0;
-	int num_to_add = 0;
 
 	/** newt *************************************************************/
 	newtComponent myForm;
@@ -308,13 +229,12 @@ add_mountlist_entry(struct mountlist_itself *mountlist,
 	}
 	strcpy(drive_to_add, device_str);
 	for (i = strlen(drive_to_add); isdigit(drive_to_add[i - 1]); i--);
-	num_to_add = atoi(drive_to_add + i);
 	drive_to_add[i] = '\0';
 	currline = mountlist->entries;
 	strcpy(mountlist->el[currline].device, device_str);
 	strcpy(mountlist->el[currline].mountpoint, mountpoint_str);
 	strcpy(mountlist->el[currline].format, format_str);
-	mountlist->el[currline].size = atol(size_str) * 1024;
+	mountlist->el[currline].size = atol(size_str) * 1024L;
 	mountlist->entries++;
 	if (strstr(mountlist->el[currline].device, RAID_DEVICE_STUB)) {
 		initiate_new_raidlist_entry(raidlist, mountlist, currline,
@@ -602,8 +522,7 @@ choose_raid_level(struct OSSWAP (raid_device_record, vinum_plex) * raidrec)
 
 
 	assert(raidrec != NULL);
-	system
-		("grep Pers /proc/mdstat > /tmp/raid-personalities.txt 2> /dev/null");
+	paranoid_system("grep Pers /proc/mdstat > /tmp/raid-personalities.txt 2> /dev/null");
 	strcpy(personalities,
 		   last_line_of_file("/tmp/raid-personalities.txt"));
 	sprintf(prompt, "Please enter the RAID level you want. %s",
@@ -1267,7 +1186,7 @@ edit_mountlist_entry(struct mountlist_itself *mountlist,
 	strcpy(mountpoint_str, mountlist->el[currline].mountpoint);
 	strcpy(mountpt_used_to_be, mountlist->el[currline].mountpoint);
 	strcpy(format_str, mountlist->el[currline].format);
-	sprintf(size_str, "%lld", mountlist->el[currline].size / 1024);
+	sprintf(size_str, "%lld", mountlist->el[currline].size / 1024L);
 	newtOpenWindow(20, 5, 48, 10, "Edit entry");
 	label0 = newtLabel(2, 1, "Device:");
 	label1 = newtLabel(2, 2, "Mountpoint:");
@@ -1370,7 +1289,7 @@ edit_mountlist_entry(struct mountlist_itself *mountlist,
 				calculate_raid_device_size(mountlist, raidlist,
 									   mountlist->el[currline].device);
 		} else {
-			mountlist->el[currline].size = atol(size_str) * 1024;
+			mountlist->el[currline].size = atol(size_str) * 1024L;
 		}
 	}
 	newtListboxSetEntry(listbox, (long) keylist[currline],
@@ -1883,6 +1802,7 @@ edit_raidlist_plex(struct mountlist_itself *mountlist,
 #undef UCOMP
 #undef ACOMP
 
+		newtDrawForm(editraidForm);
 		newtRefresh();
 		b_res = newtRunForm(editraidForm);
 		if (b_res == bOK || b_res == bCancel) {
@@ -2016,22 +1936,19 @@ edit_mountlist_in_newt(char *mountlist_fname,
 
 	/** buffers **********************************************************/
 	char tmp[MAX_STR_LEN];
-	char flaws_str_A[MAX_STR_LEN];
-	char flaws_str_B[MAX_STR_LEN];
-	char flaws_str_C[MAX_STR_LEN];
+	char *flaws_str = NULL;
+	char *flaws_str_A = NULL;
+	char *flaws_str_B = NULL;
+	char *flaws_str_C = NULL;
 
 	assert(mountlist != NULL);
 	assert(raidlist != NULL);
 
-	strcpy(flaws_str_A, "xxxxxxxxx");
-	strcpy(flaws_str_B, "xxxxxxxxx");
-	strcpy(flaws_str_C, "xxxxxxxxx");
 	if (mountlist->entries > ARBITRARY_MAXIMUM) {
 		log_to_screen("Arbitrary limits suck, man!");
 		finish(1);
 	}
-	newtPushHelpLine
-		("   Please edit the mountlist to your satisfaction, then click OK or Cancel.");
+	newtPushHelpLine("WARNING: No LVM modification possible here. Edit /tmp/i-want-my-lvm instead");
 	i = 4;
 	bAdd = newtCompactButton(i, 17, " Add ");
 	bEdit = newtCompactButton(i += 11, 17, " Edit ");
@@ -2039,12 +1956,11 @@ edit_mountlist_in_newt(char *mountlist_fname,
 	bReload = newtCompactButton(i += 12, 17, "Reload");
 	bCancel = newtCompactButton(i += 12, 17, "Cancel");
 	bOK = newtCompactButton(i += 12, 17, "  OK  ");
-	sprintf(tmp, "%-24s %-24s %-8s  %s", "Device", "Mountpoint", "Format",
-			"Size (MB)");
+	sprintf(tmp, "%-24s %-24s %-8s  %s", "Device", "Mountpoint", "Format", "Size (MB)");
 	headerMsg = newtLabel(2, 1, tmp);
-	flawsLabelA = newtLabel(2, 13, flaws_str_A);
-	flawsLabelB = newtLabel(2, 14, flaws_str_B);
-	flawsLabelC = newtLabel(2, 15, flaws_str_C);
+	flawsLabelA = newtLabel(2, 13, "         ");
+	flawsLabelB = newtLabel(2, 14, "         ");
+	flawsLabelC = newtLabel(2, 15, "         ");
 	partitionsListbox =
 		newtListbox(2, 2, 10, NEWT_FLAG_SCROLL | NEWT_FLAG_RETURNEXIT);
 	redraw_mountlist(mountlist, keylist, partitionsListbox);
@@ -2054,15 +1970,38 @@ edit_mountlist_in_newt(char *mountlist_fname,
 						  flawsLabelA, flawsLabelB, flawsLabelC, bAdd,
 						  bEdit, bDelete, bReload, bCancel, bOK, NULL);
 	while (!finished) {
-		evaluate_mountlist(mountlist, flaws_str_A, flaws_str_B,
-						   flaws_str_C);
+		flaws_str = evaluate_mountlist(mountlist);
+		/* flaws_str can be NULL */
+		if ((flaws_str == NULL) || (strlen(flaws_str) == 0)) {
+			mr_asprintf(&flaws_str_A, "");
+		} else {
+			mr_asprintf(&flaws_str_A, "%s", flaws_str + 1);
+		}
+		if (strlen(flaws_str_A) >= 74) {
+			for (i = 74; flaws_str_A[i] != ' '; i--);
+			mr_asprintf(&flaws_str_B, "%s", flaws_str_A + i + 1);
+			flaws_str_A[i] = '\0';
+		} else {
+			mr_asprintf(&flaws_str_B, "");
+		}
+		if (strlen(flaws_str_B) >= 74) {
+			for (i = 74; flaws_str_B[i] != ' '; i--);
+			mr_asprintf(&flaws_str_C, "%s", flaws_str_B + i + 1);
+			flaws_str_B[i] = '\0';
+		} else {
+			mr_asprintf(&flaws_str_C, "");
+		}
+
 		newtLabelSetText(flawsLabelA, flaws_str_A);
 		newtLabelSetText(flawsLabelB, flaws_str_B);
 		newtLabelSetText(flawsLabelC, flaws_str_C);
 		b_res = newtRunForm(myForm);
+		mr_free(flaws_str_A);
+		mr_free(flaws_str_B);
+		mr_free(flaws_str_C);
+
 		if (b_res == bOK) {
-			if (!evaluate_mountlist
-				(mountlist, flaws_str_A, flaws_str_B, flaws_str_C)) {
+			if (flaws_str != NULL) {
 				finished =
 					ask_me_yes_or_no
 					("Your mountlist might not work. Continue anyway?");
@@ -2115,6 +2054,7 @@ This would be really dumb. RAIDTAB_FNAME is #define'd.   --- Hugo, 2003/04/24
 				}
 			}
 		}
+		mr_free(flaws_str);
 	}
 	newtFormDestroy(myForm);
 	newtPopWindow();
@@ -2141,9 +2081,8 @@ edit_mountlist(char *mountlist_fname, struct mountlist_itself *mountlist,
 			   struct raidlist_itself *raidlist)
 {
 	int res = 0;
-//  char tmp[MAX_STR_LEN];
 
-	iamhere("entering eml");
+	log_it("entering eml");
 
 	if (g_text_mode) {
 		fatal_error("Don't call edit_mountlist() in text mode");
@@ -2152,7 +2091,7 @@ edit_mountlist(char *mountlist_fname, struct mountlist_itself *mountlist,
 			("I'm in GUI mode, so I shall edit mountlist using edit_mountlist()");
 		res = edit_mountlist_in_newt(mountlist_fname, mountlist, raidlist);
 	}
-	iamhere("leaving eml");
+	log_it("leaving eml");
 	return (res);
 }
 
@@ -2325,65 +2264,6 @@ find_raid_device_in_raidlist(struct raidlist_itself *raidlist,
 }
 
 
-/**
- * Get information about the location of ISO images from the user.
- * @param isodir_device Where to put the device (e.g. /dev/hda4) the user enters.
- * @param isodir_format Where to put the format (e.g. ext2) the user enters.
- * @param isodir_path Where to put the path (e.g. /var/cache/mondo) the user enters.
- * @param nuke_me_please Whether we're planning on nuking or not.
- * @return TRUE if OK was pressed, FALSE otherwise.
- */
-bool
-get_isodir_info(char *isodir_device, char *isodir_format,
-				char *isodir_path, bool nuke_me_please)
-{
-
-	/** initialize ********************************************************/
-
-	assert(isodir_device != NULL);
-	assert(isodir_format != NULL);
-	assert(isodir_path != NULL);
-
-	log_it("%d - AAA - isodir_path = %s", isodir_path);
-	isodir_format[0] = '\0';
-	if (isodir_device[0] == '\0') {
-		strcpy(isodir_device, "/dev/");
-	}
-	if (isodir_path[0] == '\0') {
-		strcpy(isodir_path, "/");
-	}
-	if (does_file_exist("/tmp/NFS-SERVER-PATH")) {
-		strcpy(isodir_device, last_line_of_file("/tmp/NFS-SERVER-MOUNT"));
-		strcpy(isodir_format, "nfs");
-		strcpy(isodir_path, last_line_of_file("/tmp/NFS-SERVER-PATH"));
-	}
-	if (nuke_me_please) {
-		return (TRUE);
-	}
-
-	if (popup_and_get_string
-		("ISO Mode - device", "On what device do the ISO files live?",
-		 isodir_device, MAX_STR_LEN / 4)) {
-		if (popup_and_get_string
-			("ISO Mode - format",
-			 "What is the disk format of the device? (Hit ENTER if you don't know.)",
-			 isodir_format, 16)) {
-			if (popup_and_get_string
-				("ISO Mode - path",
-				 "At what path on this device can the ISO files be found?",
-				 isodir_path, MAX_STR_LEN / 4)) {
-				strip_spaces(isodir_device);
-				strip_spaces(isodir_format);
-				strip_spaces(isodir_path);
-				log_it("%d - BBB - isodir_path = %s", isodir_path);
-				return (TRUE);
-			}
-		}
-	}
-	return (FALSE);
-}
-
-
 /**
  * Create a new raidtab entry for @p device in @p raidlist.
  * @param raidlist The raidlist to add the device to.
@@ -2464,7 +2344,6 @@ void nuke_mode_dummy()
 	newtComponent b1;
 	newtComponent b2;
 	newtComponent b3;
-	newtComponent b_res;
 
 
 	newtPushHelpLine
@@ -2475,7 +2354,6 @@ void nuke_mode_dummy()
 	b3 = newtButton(7, 9, "Quickly");
 	myForm = newtForm(NULL, NULL, 0);
 	newtFormAddComponents(myForm, b1, b2, b3, NULL);
-	b_res = newtRunForm(myForm);
 	newtFormDestroy(myForm);
 	newtPopWindow();
 	newtPopHelpLine();
@@ -2579,7 +2457,7 @@ void redraw_unallocpartnslist(struct mountlist_itself
 	for (i = 0; i < unallocated_raid_partitions->entries; i++) {
 		sprintf(tmp, "%-22s %8lld",
 				unallocated_raid_partitions->el[i].device,
-				unallocated_raid_partitions->el[i].size / 1024);
+				unallocated_raid_partitions->el[i].size / 1024L);
 		newtListboxAppendEntry(listbox, tmp, keylist[i]);
 	}
 }
@@ -2808,7 +2686,7 @@ select_raid_disks(struct mountlist_itself *mountlist_dontedit,
 	assert(description_of_list != NULL);
 	assert(disklist != NULL);
 
-	iamhere("malloc'ing");
+	log_it("malloc'ing");
 	malloc_string(tmp);
 	malloc_string(help_text);
 	malloc_string(title_of_window);
@@ -2836,7 +2714,7 @@ select_raid_disks(struct mountlist_itself *mountlist_dontedit,
 	memcpy((void *) bkp_disklist, (void *) disklist,
 		   sizeof(struct list_of_disks));
 
-	iamhere("Post-malloc");
+	log_it("Post-malloc");
 	strcpy(help_text,
 		   "   Edit this RAID device's list of partitions. Choose OK or Cancel when done.");
 	sprintf(header_text, "%-24s    %s", "Device", "Index");
@@ -2957,7 +2835,9 @@ char which_restore_mode()
 		for (output = 'z'; !strchr("AICE", output); output = tmp[0]) {
 			printf
 				("Which mode - (A)utomatic, (I)nteractive, \n(C)ompare only, or (E)xit to shell?\n--> ");
-			fgets(tmp, MAX_STR_LEN - 1, stdin);
+			if (fgets(tmp, MAX_STR_LEN - 1, stdin)) {
+				// FIXME
+			}
 		}
 		return (output);
 	}
