@@ -4,7 +4,7 @@
     begin                : Sat Apr 20 2002
     copyright            : (C) 2002 by Stan Benoit
     email                : troff@nakedsoul.org
-    cvsid                : $Id: mondo-rstr-newt.h 1295 2007-04-11 18:59:49Z bruno $
+    cvsid                : $Id: mondo-rstr-newt.h 3158 2013-06-24 08:20:16Z bruno $
  ***************************************************************************/
 
 /***************************************************************************
@@ -17,7 +17,7 @@
  ***************************************************************************/
 
 
-#include "../common/my-stuff.h"
+#include "my-stuff.h"
 #include "../common/mondostructures.h"
 #include "../common/libmondo.h"
 #ifdef   __FreeBSD__
@@ -55,7 +55,7 @@ extern void toggle_path_selection(struct s_node *, char *, bool);
 extern void toggle_node_selection(struct s_node *, bool);
 extern struct s_node *find_node_in_filelist(struct s_node *,
 											char *filename);
-extern int what_number_cd_is_this(struct s_bkpinfo *);
+extern int what_number_cd_is_this();
 //extern void fatal_error (char *);
 extern void sort_mountlist_by_device(struct mountlist_itself *);
 extern int load_mountlist(struct mountlist_itself *a, char *b);
@@ -136,19 +136,16 @@ int edit_mountlist_in_newt(char *mountlist_fname,
 int edit_mountlist(char *mountlist_fname, struct mountlist_itself *,
 				   struct raidlist_itself *);
 void edit_raidrec_additional_vars(struct raid_device_record *);
-int evaluate_mountlist(struct mountlist_itself *, char *, char *, char *);
+char *evaluate_mountlist(struct mountlist_itself *);
 int find_device_in_mountlist(struct mountlist_itself *, char *);
 int find_next_free_index_in_disklist(struct list_of_disks *);
 int find_raid_device_in_raidlist(struct raidlist_itself *, char *);
-bool get_isodir_info(char *, char *, char *, bool);
 void initiate_new_raidlist_entry(struct raidlist_itself *,
 								 struct mountlist_itself *, int, char *);
 void insert_essential_additionalvars(struct raid_device_record *);
 bool is_this_raid_personality_registered(int);
 void log_file_end_to_screen(char *, char *);
 void log_to_screen(const char *fmt, ...);
-int look_for_duplicate_mountpoints(struct mountlist_itself *, char *);
-int look_for_weird_formats(struct mountlist_itself *, char *);
 void make_list_of_unallocated_raid_partitions(struct mountlist_itself *,
 											  struct mountlist_itself *,
 											  struct raidlist_itself *);
@@ -180,7 +177,6 @@ void select_raid_disks(struct mountlist_itself *, struct raidlist_itself *,
 					   struct list_of_disks *);
 void setup_newt_stuff(void);
 long size_of_specific_device(struct mountlist_itself *, char *);
-bool spread_flaws_across_three_lines(char *, char *, char *, char *, int);
 char *turn_raid_level_number_to_string(int);
 void update_evalcall_form(int);
 void update_progress_form(char *);
