@@ -3,17 +3,15 @@
  */
 
 void free_global_filenames();
-void get_cfg_file_from_archive_or_bust(struct s_bkpinfo *);
+void get_cfg_file_from_archive_or_bust();
 bool is_file_in_list(char *, char *, char *);	/* needle, haystack, preamble */
-int iso_fiddly_bits(struct s_bkpinfo *bkpinfo, bool nuke_me_please);
+int iso_fiddly_bits(bool nuke_me_please);
 void kill_petris(void);
-int modify_rclocal_one_time(char *path);
-int mount_cdrom(struct s_bkpinfo *bkpinfo);
 int mount_device(char *, char *, char *, bool);
 int mount_all_devices(struct mountlist_itself *, bool);
 void protect_against_braindead_sysadmins(void);
-int read_cfg_file_into_bkpinfo(char *cfg_file, struct s_bkpinfo *bkpinfo);
-struct s_node *process_filelist_and_biggielist(struct s_bkpinfo *);
+int read_cfg_file_into_bkpinfo(char *cfg_file);
+struct s_node *process_filelist_and_biggielist();
 int backup_crucial_file(char *path_root, char *filename);
 
 int run_boot_loader(bool);
@@ -24,12 +22,12 @@ int run_raw_mbr(bool offer_to_hack_scripts, char *bd);
 char *find_my_editor(void);
 void streamline_changes_file(char *, char *);
 void set_signals(int on);
-void setup_global_filenames(struct s_bkpinfo *bkpinfo);
+void setup_global_filenames();
 //void setup_signals(int);
 void twenty_seconds_til_yikes(void);
 int run_raw_mbr(bool offer_to_hack_scripts, char *bd);
 void terminate_daemon(int);
 void termination_in_progress(int);
 int unmount_all_devices(struct mountlist_itself *);
-int get_cfg_file_from_archive(struct s_bkpinfo *bkpinfo);
+int get_cfg_file_from_archive();
 void ask_about_these_imagedevs(char *infname, char *outfname);
