@@ -1,5 +1,5 @@
 /*
- * $Id: mr-externs.h 684 2006-06-26 11:12:48Z bcornec $
+ * $Id: mr-externs.h 3158 2013-06-24 08:20:16Z bruno $
  *
  * mondo-restore.c's externs
  *
@@ -15,7 +15,7 @@
 
 extern bool ask_me_yes_or_no(char *);
 extern char *calc_checksum_of_file(char *);
-extern int closein_tape(struct s_bkpinfo *);
+extern int closein_tape();
 extern void close_evalcall_form(void);
 extern char *call_program_and_get_last_line_of_output(char *);
 extern void close_progress_form(void);
@@ -31,7 +31,6 @@ extern int format_device(char *, char *, struct raidlist_itself *);
 extern void finish(int);
 extern void free_filelist(struct s_node *);
 extern long get_time(void);
-extern bool get_isodir_info(char *, char *, char *, bool);
 extern void fatal_error(char *);
 extern void initialize_raid_record(struct raid_device_record *);
 extern bool is_this_device_mounted(char *);
@@ -50,8 +49,8 @@ extern int make_list_of_drives(struct mountlist_itself *,
 extern bool mountlist_contains_raid_devices(struct mountlist_itself *);
 extern void open_evalcall_form(char *);
 extern void open_progress_form(char *, char *, char *, char *, long);
-extern int openin_cdstream(struct s_bkpinfo *);
-extern int openin_tape(struct s_bkpinfo *);
+extern int openin_cdstream();
+extern int openin_tape();
 extern int partition_device(char *, int, int, char *, long);
 extern int partition_device_with_fdisk(char *, int, int, char *, long);
 extern int partition_device_with_parted(char *, int, int, char *, long);
@@ -60,13 +59,13 @@ extern int partition_everything(struct mountlist_itself *);
 extern void popup_and_OK(char *);
 extern bool popup_and_get_string(char *, char *, char *, int);
 extern void setup_newt_stuff(void);
-extern void reset_bkpinfo(struct s_bkpinfo *);
+extern void reset_bkpinfo();
 extern int read_cfg_var(char *, char *, char *);
-extern int read_file_from_stream_to_file(struct s_bkpinfo *, char *,
+extern int read_file_from_stream_to_file(char *,
 										 long long);
-extern int read_file_from_stream_to_stream(struct s_bkpinfo *, FILE *,
+extern int read_file_from_stream_to_stream(FILE *,
 										   long long);
-extern int read_file_from_stream_FULL(struct s_bkpinfo *, char *, FILE *,
+extern int read_file_from_stream_FULL(char *, FILE *,
 									  long long);
 extern int read_header_block_from_stream(long long *, char *, int *);
 extern void save_filelist(struct s_node *, char *);
@@ -77,7 +76,7 @@ extern int stop_raid_device(char *);
 extern int stop_all_raid_devices(struct mountlist_itself *);
 extern void update_evalcall_form(int);
 extern void update_progress_form(char *);
-extern int verify_tape_backups(struct s_bkpinfo *);
+extern int verify_tape_backups();
 extern char which_restore_mode(void);
 extern int which_format_command_do_i_need(char *, char *);
 extern int write_cfg_var(char *, char *, char *);
@@ -90,7 +89,7 @@ extern void resize_drive_proportionately_to_suit_new_drives(struct
 extern void resize_mountlist_proportionately_to_suit_new_drives(struct
 																mountlist_itself
 																*mountlist);
-extern int get_cfg_file_from_archive(struct s_bkpinfo *);
+extern int get_cfg_file_from_archive();
 
 
 
@@ -104,19 +103,11 @@ extern int g_currentY;
 extern int g_current_media_number;	/* set to 1 in mondo-tools.c (tape) */
 extern long long g_tape_posK;
 extern FILE *g_tape_stream;
-extern bool g_cd_recovery;
 extern bool g_text_mode;
 extern bool g_restoring_live_from_cd;
+extern bool g_restoring_live_from_netfs;
 extern int fput_string_one_char_at_a_time(FILE *, char *);
-
-
-
-
-
-extern int
-evaluate_mountlist(struct mountlist_itself *mountlist, char *flaws_str_A,
-				   char *flaws_str_B, char *flaws_str_C);
-
+extern char *evaluate_mountlist(struct mountlist_itself *mountlist);
 
 
 #ifdef __FreeBSD__
